/********************************************************************
 * AFrame.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: AFrame.cpp,v 1.4 1999/12/16 08:13:02 root Exp root $
 *
 ********************************************************************/

#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <iostream>
#include <vector>
#include <qobject.h>
#include <kapp.h>
#include <ksock.h>
#include <qlabel.h>
#include <qframe.h>
#include <qmsgbox.h>
#include <qstring.h>
#include <qpushbt.h>
#include <qlistbox.h>
#include <qlineedit.h>

#include <config.h>
#include <AnlageParm.h>
#include <AnlPort.h>
#include <TlnInfo.h>
#include <EPacket.h>
#include <led.h>
#include <nst.h>
#include <amt.h>
#include <Login.h>
#include <Q931.h>
#include <AFrame.moc>

AFrame::AFrame(QWidget *_parent=0L, const char *name=0L)
	: QFrame(_parent, name)
{
	Login login(this,"Login");
	if (!login.exec()) exit(1);

	for (int i=0; i < 4; i++) {
		sprt[i]=0L;
	}

	for (int i=0; i < 50; i++) {
		stln[i]=0L;
		nst[i]=0L;
	}

	pbx1 = new Anlage();
	pbx2 = new Anlage();

	QString l_appl(qApp->argv()[0]);

	mh = new MsgHandlerQt(
						login.Host(),
						login.Port(),
						&l_appl.data()[l_appl.findRev('/') + 1],
          						login.User(),
						login.Pass(),
						MsgHandler::AUTH_CLEARTXT,
#ifdef USE_CTI
						MsgHandler::PERM_CTI |
#endif
						MsgHandler::PERM_BMON | MsgHandler::PERM_S0MON 
						);

	QObject::connect(mh, SIGNAL( failMsg(MsgHandlerQt*) ), SLOT( failure(MsgHandlerQt*)));
	QObject::connect(mh, SIGNAL( rdyMsg(ESPMsgHdr *) ), SLOT( start(ESPMsgHdr *)));

	mh->switchDebug(false);
	mh->esp_conn_req();
}

AFrame::~AFrame()
{
}

void AFrame::build()
{
	QLabel	*qlN,*qlA;
	QString	s;
	char dummy[10];
	int x = 0;
	int y = 1;
	int i = 0;


	qlN = new QLabel("Nebenstellen:",this);
	qlN->move(5,0);

	f1 = new QFrame(this);
	f1->setFrameStyle(Panel | Sunken);
	f1->setLineWidth(1);

	i = 0;
	while (stln[i]) {
		nst[i] = new Nst(f1,stln[i]->Nummer(),stln[i]);
		nst[i]->move(x * (nst[i]->width() + 5) + 5, (y-1) * (nst[i]->height() + 8) + 3);
		i++;
		x++;
		if (x == 10) {
			if (stln[i+1]) y++;
			x = 0;
		}
	}

	f1->resize((nst[0]->width() + 5) * 10 + 8, y * nst[0]->height() +((y - 1) * 8) + 10);

	qlN->resize(f1->width(), 12);
	f1->move(0,qlN->frameGeometry().bottom() + 5);

	qlA = new QLabel("S0-Busse:",this);
	qlA->resize(f1->width(), 12);
	qlA->move(5,f1->frameGeometry().bottom() + 5);

	f2 = new QFrame(this);
	f2->setFrameStyle(Panel | Sunken);
	f2->setLineWidth(1);
	f2->move(0,qlA->frameGeometry().bottom() + 5);

	x = f1->width() / (pbx1->S0Sum() + 1);
	for (i=0; i < pbx1->S0Sum(); i++) {
		sprintf(dummy,"%d %s",sprt[i]->Idx(), sprt[i]->GetTypName(sprt[i]->Mode()));
		amt[i]   = new Amt(f2,dummy,sprt[i]->Idx());
		amt[i]->move((i + 1) * x - (amt[i]->width() >> 1), 3);
		x++;
	}
/*
	x = f1->width() / (pbx1->S0Ext() + 1);
	for (i=0; i < pbx1->S0Ext(); i++) {
		sprintf(dummy,"Amt %d",i+1);
		amt[i]   = new Amt(f2,dummy,i);
		amt[i]->move((i + 1) * x - (amt[i]->width() >> 1), 3);
		x++;
	}
*/

	f2->resize(f1->width(), amt[0]->height() + 8);

	QFont f("helvetica", 10);

	date = new QLabel("Datum/Zeit:",this);
	date->resize((f1->width() >> 1) - 2, 15);
	date->move(0,f2->frameGeometry().bottom() + 5);
	date->setFrameStyle(Panel | Sunken);
	date->setLineWidth(1);
	date->setFont(f);


	typ = new QLabel("T",this);
	typ->resize((f1->width() >> 1) - 2, 15);
	typ->move(0,date->frameGeometry().bottom() + 5);
	typ->setFrameStyle(Panel | Sunken);
	typ->setLineWidth(1);
	s.sprintf("Anlagetyp: Euracom %d", pbx1->Typ());
	typ->setText(s);
	typ->setFont(f);

	vers = new QLabel("V",this);
	vers->resize((f1->width() >> 1) - 2, 15);
	vers->move(0,typ->frameGeometry().bottom() + 5);
	vers->setFrameStyle(Panel | Sunken);
	vers->setLineWidth(1);
	s.sprintf("Firmware: %s", pbx1->Vers());
	vers->setText(s);
	vers->setFont(f);

	icall = new QListBox(this,"xxx");
	icall->resize((f1->width() >> 1) - 2, vers->frameGeometry().bottom() - date->frameGeometry().top());
	icall->move((f1->width() >> 1) + 2,f2->frameGeometry().bottom() + 5);
	icall->setAutoBottomScrollBar(false);
	icall->setFont(f);


	QPushButton *q = new QPushButton("Quit", this);
	q->resize(80,20);
	q->move((f1->width() >> 1) - 40, icall->frameGeometry().bottom() + 8);

	resize(f1->width(), q->frameGeometry().bottom() + 5);

	setMaximumWidth(width());
	setMaximumHeight(height());
	setMinimumWidth(width());
	setMinimumHeight(height());

	QObject::connect(q, SIGNAL(clicked()), qApp, SLOT(quit()));
	QObject::connect(mh, SIGNAL( evtMsg(ESPMsgHdr *) ), SLOT( draw(ESPMsgHdr *)));
	QObject::connect(mh, SIGNAL( staMsg(ESPMsgHdr *) ), SLOT( stat(ESPMsgHdr *)));
	show();
}

void AFrame::start(ESPMsgHdr *esp)
{
	QObject::connect(mh, SIGNAL( anlMsg(ESPMsgHdr *) ), SLOT( anlage(ESPMsgHdr *)));
	QObject::connect(mh, SIGNAL( porMsg(ESPMsgHdr *) ), SLOT( port(ESPMsgHdr *)));
	mh->asyncSend(MsgHandler::G_ANL, MsgHandler::ANL_MAX_PARA);
}

void AFrame::anlage(ESPMsgHdr *esp)
{
	static BYTE index = 0;
	static BYTE l_ind = 0;

	switch (esp->element) {
		case MsgHandler::ANL_MAX_PARA :
			pbx1->setMaxParm(esp->data);
			mh->asyncSend( MsgHandler::G_ANL, MsgHandler::ANL_TYP);
			break;

		case MsgHandler::ANL_TYP :
			pbx1->setTyp(esp->data);
			mh->asyncSend( MsgHandler::G_ANL,
				       MsgHandler::ANL_BETRIEBSART);
			break;

		case MsgHandler::ANL_BETRIEBSART :
			pbx1->setBArt(esp->data[0]);
			mh->asyncSend( MsgHandler::G_ANL,
				       MsgHandler::ANL_SW_VERSION);
			break;

		case MsgHandler::ANL_SW_VERSION :
			pbx1->setSWVers(esp->data);

			mh->asyncSend( MsgHandler::G_ANL,
				       MsgHandler::ANL_TLN_INFO, 0);
			break;

		case MsgHandler::ANL_TLN_INFO :
			int n = (int)(esp->data.size() / 6);
			for (int i=0; i < n; i++) {
				if (esp->data[(i * 6) + 4] != 0xFF) {
					stln[l_ind++] = new TlnInfo((char*)&esp->data[(i * 6) + 0]);
				}
			}
			if (index < 2) {
				index++;
				mh->asyncSend(	MsgHandler::G_ANL,
								MsgHandler::ANL_TLN_INFO, 
								index);
			} else {
				vector<BYTE> data(2);
				data[0] = 0x40;
				data[1] = 0x00;
				mh->asyncSend(	MsgHandler::G_POR,
								MsgHandler::POR_MODE, 
								data);
			}
	}
}

void AFrame::port(ESPMsgHdr *esp)
{
cout << "AFrame::port" << endl;
	switch (esp->element) {
		case MsgHandler::POR_MODE :
			sprt[esp->data[1]] = new AnlPort(esp->data);
			if (esp->data[1] < (pbx1->S0Sum() - 1)) {
				vector<BYTE> data(2);
				data[0] = 0x40;
				data[1] = (esp->data[1] + 1);
				mh->asyncSend(	MsgHandler::G_POR,
								MsgHandler::POR_MODE, 
								data);
			} else { 
				build();
			}
			break;
	}
}

void AFrame::stat(ESPMsgHdr *esp)
{
	QString a;
	q931Pack p(esp,false);

	switch (esp->element) {
		case MsgHandler::STA_TEST :
			if ((p.protocol == PROT_ACKERMANN) && (p.msgtype == AKM_DATETIME)) {
				a.sprintf("Datum/Zeit: %s", p.DateTime.c_str());
				date->setText(a);
			} else if ((p.protocol == PROT_EURO) && (p.msgtype == 0x05)){
				if (((p.Caller.size() > 2) || p.Caller.empty()) && ((p.Called.size() > 2) || p.Called.empty())) {
					a.sprintf("%s -> %s", p.Caller.empty() ? "Unbekannt" : p.Caller.c_str(), p.Called.empty() ? "Unbekannt" : p.Called.c_str());
					icall->insertItem(a,0);
				}
			}
			break;
		default :
			break;
	}
}

void AFrame::draw(ESPMsgHdr *esp)
{
	if (esp->element == 0x01) {

		int i = 0;
		while (nst[i]) nst[i++]->setStatus(esp->data,pbx1->EvtBytes());

		for (i=0; i < pbx1->S0Sum(); i++) {
			amt[i]->setStatus(esp->data,pbx1->EvtBytes());
		}
	}
}

void AFrame::failure(MsgHandlerQt*)
{
	QString l_appl(qApp->argv()[0]);

	if (mh->failFatal())
		QMessageBox::critical( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString(mh->failReason()));
	else
		QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString(mh->failReason()));
}

