/********************************************************************
 * Gabi/auswert.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: auswert.c,v 1.5 1999/12/30 20:50:59 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include <config.h>
#include <maKs.h>
#include <Packet.h>
#include <Decode.h>

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
#include <db_util.h>
#endif

#include "server_conn.h"
#include "auswert.h"
#include "handle.h"

extern int		m_sock;
extern int		delflag;

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
extern int		db_errors;
extern int		db_save;
extern int		done;
extern DB_CONN	*conn;
#else
int		done;
#endif

WORD	last_ds		= 0;
int		got_data	= 0;
float	fw_version	= 0.0;

/********************************************************************
 * Analyse and output data for outgoing calls
 ********************************************************************/
void showOutCalls(FILE *fp,BYTE *data, int cnt)
{
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
	char insert[1024];
#endif
	V3OCallsType *ocp;
	char *numbr;
	int	monat;
	int jahr;
	int tag;
	int stunde;
	int minute;
	int msn;
	int port;
	int grp;
	int length;
	int costs;
	int appnum;
	int i;
	float geb;
	WORD dummy;

	for (i=0 ;i < cnt; i++){

		ocp = (V3OCallsType *)data;

		costs = ocp[i].Gebuehr1 + (ocp[i].Gebuehr2 << 8) + (ocp[i].Gebuehr3 << 16);
		numbr = ocp[i].Nummer;
		length= ocp[i].Dauer + (ocp[i].Dauer1 << 16);
		appnum= (ocp[i].Appnum == 255) ? 0 : ocp[i].Appnum ;
		
		jahr = (ocp[i].Year > 90) ? ocp[i].Year + 1900 : ocp[i].Year + 2000;
		monat  = ocp[i].Month;
		tag   = ocp[i].Day;
		stunde= ocp[i].Hour;
		minute= ocp[i].Minute;
		port  = ocp[i].Port + 1;
		msn   = ocp[i].Msn;
		grp   = ocp[i].Grp;
		grp   = (grp > 999) ? -1 : grp;

		if (monat == 0 || tag == 0) {
			jahr = 1996;
			monat = 1;
			tag = 1;
		}


#if defined(DB_POSTGRES) || defined (DB_MYSQL)
		if (db_save)
		{
			sprintf(insert,
				"INSERT INTO calls VALUES ('%s',"
				DATESTRING
				",'%02d:%02d',%d,%5.3f,%x,%d,%d,%d,%d)%c",
				get_Phonenumber(numbr),
				DB_DATE_FIRST(tag,monat,jahr),
				DB_DATE_SECND(tag,monat,jahr),
				DB_DATE_THIRD(tag,monat,jahr),
				stunde,minute,
				length,
				(float)costs / 1000.0,
				appnum,
				grp,
				port,
				msn,
				0 /* Outgoing */,
				DB_END_C
			);
			if (!exec_DB(conn,insert)) {
				db_errors++;
				fprintf(stderr,
					"\n%s\nKonnte nicht in DB abgelegt werden!\n",
					insert
				);
			}
		}
		else
#endif

		fprintf(fp,
			"%c;%d.%d.%d;%02d:%02d;%02x;%s;%d;%5.3f;%d;%d;%d%s",
			'O',
			tag,monat,jahr,
			stunde,minute,
			appnum,
			get_Phonenumber(numbr),
			length,
			(float)costs / 1000.0,
			grp, port, msn,
			"\n"
			);
	}
}

/********************************************************************
 * Analyse and output data for incoming calls
 ********************************************************************/
void showInCalls(FILE *fp,InCallsType icp[], int cnt)
{
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
	char insert[1024];
#endif
	int tag;
	int monat;
	int jahr;
	int stunde;
	int minute;
	int msn;
	int port;
	int i;
	LONG dummy;

	for (i=0 ;i < cnt; i++){

		dummy = (icp[i].HHFVal << 24) |
			(icp[i].HLFVal << 16) |
			(icp[i].LHFVal <<  8) | 
			 icp[i].LLFVal;

		jahr  = (dummy & 0x3F) + 1997;
		tag   = ((dummy >>  6) & 0x1F);
		stunde= ((dummy >> 11) & 0x1F);
		monat = ((dummy >> 16) & 0x0F);
		minute= ((dummy >> 20) & 0x3F);
		port  =	((dummy >> 26) & 0x03) + 1;
		msn   = ((dummy >> 28) & 0x0F);

		if (monat == 0 || tag == 0) {
			jahr = 1996;
			monat = 1;
			tag = 1;
		}

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
		if (db_save)
		{
			sprintf(insert,
				"INSERT INTO calls VALUES ('%s',"
				DATESTRING
				",'%02d:%02d',%d,%5.3f,%x,%d,%d,%d,%d)%c",
				get_Phonenumber(icp[i].Nummer),
				DB_DATE_FIRST(tag,monat,jahr),
				DB_DATE_SECND(tag,monat,jahr),
				DB_DATE_THIRD(tag,monat,jahr),
				stunde,minute,
				icp[i].Dauer,
				0.0 /* No Costs */,
				(icp[i].Appnum == 0xAA) ? 0 : icp[i].Appnum,
				-1 /* No Group */,
				port,
				msn,
				1 /* Incomming */,
				DB_END_C
			);
			if (!exec_DB(conn,insert)) {
				db_errors++;
				fprintf(stderr,
					"\n%s\nKonnte nicht in DB abgelegt werden!\n",
					insert
				);
			}
		}
		else
#endif

		fprintf(fp,
			"%c;%d.%d.%d;%02d:%02d;%02x;%s;%d;%5.2f;%d;%d;%d%s",
			'I',
			tag,monat,jahr,
			stunde,minute,
			icp[i].Appnum,
			get_Phonenumber(&icp[i].Nummer[0]),
			icp[i].Dauer,
			0.0,				/* No Costs */
			-1,			/* No Group */
			port, msn,
			"\n"
			);
	}
}

/********************************************************************
 * Calculate index for set of data and request next set (oytgoing)
 ********************************************************************/
int handle_outcalls(FILE *fp, EspMsg  *esp)
{
	BYTE *data = &esp->data[2];
	int len;
	int nds;
	int ret;

	len = ((int)esp->length & 0x00FF) - 2;
	nds = (int)(len / 22);

	got_data++;

	last_ds += nds;

	if (nds) {
		showOutCalls(fp,data,nds);
	}
	return(nds);
}

/********************************************************************
 * Calculate index for set of data and request next set (incoming)
 ********************************************************************/
int handle_incalls(FILE *fp, EspMsg  *esp)
{
	InCallsType *data = (InCallsType *)&esp->data[2];
	int len = esp->length - 2;
	int nds = (int)(len / 19);
	int ret;

	last_ds += nds;

	if (nds) {
		showInCalls(fp,data,nds);
	} else {
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
		if (db_save) {
			exec_DB(conn,"END");
			close_DB(conn);
			fprintf(stdout,"Daten in DB abgelegt...\n");
		}
#endif
	}
	return(nds);
}

/********************************************************************
 * Request data
 ********************************************************************/
EspMsg *getCalls(int fd, BYTE in_out, WORD dsnum)
{
    BYTE   ldsn[2];

    ldsn[0] = (dsnum & 0x00FF);
    ldsn[1] = (dsnum >> 8);

	return((EspMsg *)send_request(fd, 0, K_CTI_OFF, in_out, 2, ldsn));
}

/********************************************************************
 * Handle all data
 ********************************************************************/
void RequestData(FILE *fp)
{
	EspMsg  *esp;
	int		ret = 0;
	BYTE	ldsn[2];

	fprintf(stderr,"Lese Gespraechsdaten...\n");
	fflush(stderr);
	last_ds = 0;
	do {
		esp = getCalls(m_sock, OFF_TLN_ANRUFLISTE, last_ds);
		if (esp) {
			ret = handle_outcalls(fp,esp);
			free(esp);
		} else
			exit(1);
	} while (ret);

	fprintf(stderr,"Lese Anrufdaten...\n");
	fflush(stderr);
	last_ds = 0;
	do {
		esp = getCalls(m_sock, OFF_ANL_ANRUFLISTE, last_ds);
		if (esp) {
			ret = handle_incalls(fp,esp);
			free(esp);
		} else
			exit(1);
	} while (ret);

	if (
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
		!db_errors &&
#endif
		delflag) {
			do_delete_calls();
	} else {
			done++;
	}
}

/********************************************************************
 * Delete incoming and outgoing data from pbx
 ********************************************************************/
void do_delete_calls()
{
	EspMsg  *esp;
	BYTE    data[60];

	data[0] = 0xFF;
	data[1] = 0xFF;

	fprintf(stderr,"Loesche Gespraechsdaten...");
	fflush(stderr);
	esp = send_request(m_sock, 0, K_CTI_OFF | K_CTI_SET | K_CTI_DEL, OFF_TLN_ANRUFLISTE, 2, data);
	if (esp) free(esp);
	fprintf(stderr,"\tgeloescht!\nLoesche Anrufdaten...");
	fflush(stderr);
	esp = send_request(m_sock, 0, K_CTI_OFF | K_CTI_SET | K_CTI_DEL, OFF_ANL_ANRUFLISTE, 2, data);
	if (esp) free(esp);
	fprintf(stderr,"\tgeloescht!\n");
}

