/********************************************************************
 * Kim/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.1 1999/12/30 20:54:38 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <version.h>
#include <config.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "handle.h"

extern int    done;

FILE	*outf;
FILE	*inf;
int		m_sock;

/********************************************************************
 *
 ********************************************************************/
void SplashScreen()
{
	fprintf(stderr,"\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"        #                  K I M                       #\n");
	fprintf(stderr,"        #    Konfigurations und Informations Modul     #\n");
	fprintf(stderr,"        #      Copyright 1998  by Chris Benndorf       #\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"\n");
	fflush(stderr);
}

char *teltyp(BYTE typ)
{
	switch(typ) {
		case 0x00 :	return("S0-Amt");
		case 0x01 :	return("S0-Int");
		case 0x02 :	return("S0-X  ");
		case 0x03 :	return("Analog");
		case 0x04 :	return("TFE   ");
		default:	return("------");
	}
}

char *telutyp(BYTE utyp)
{
	switch(utyp) {
		case 0x00 :	return("Telefon");
		case 0x01 :	return("Fax    ");
		case 0x02 :	return("AB     ");
		case 0x03 :	return("Modem  ");
		case 0x04 :	return("Multi  ");
		case 0x05 :	return("S0-Card");
		default:	return("-------");
	}
}

char *anltyp(BYTE typ)
{
	switch(typ) {
		case 0x01 :	return("Einzelanlage");
		case 0x02 :	return("NahUnterAnl.");
		case 0x04 :	return("FernUntrAnl.");
		case 0x08 :	return("Vernetzt    ");
		default:	return("-------");
	}
}

/********************************************************************
 *
 ********************************************************************/
void start(int fd)
{
	EspMsg  *esp;
	BYTE	data[60];
	int		i;

	esp = send_request(fd, 0, K_CTI_ANL, ANL_TYP, 0, NULL);
	if (esp) {
		fprintf(stdout,"ANLAGE:\t%s\n",esp->data);
		free(esp);
	}

	esp = send_request(fd, 0, K_CTI_ANL, ANL_SW_VERSION, 0, NULL);
	if (esp) {
		fprintf(stdout,"VERS:\t%s\n",&esp->data[4]);
		free(esp);
	}

	data[0] = 0x40;
	data[1] = 0x00;
	data[2] = 0x00;
	esp = send_request(fd, 0, K_CTI_POR, POR_LAND, 3, data);
	if (esp) {
		fprintf(stdout,"LKZ:\t%s\n",get_Phonenumber(&esp->data[3]));
		free(esp);
	}

	data[0] = 0x40;
	data[1] = 0x00;
	data[2] = 0x00;
	esp = send_request(fd, 0, K_CTI_POR, POR_ORT, 3, data);
	if (esp) {
		fprintf(stdout,"OKZ:\t%s\n",get_Phonenumber(&esp->data[3]));
		free(esp);
	}

	for (i = 0; i < 10; i++) {
		int	dummy;

		data[0] = 0x40;
		data[1] = 0x00;
		data[2] = (BYTE)i;
		esp = send_request(fd, 0, K_CTI_POR, POR_AMT_RNR, 3, data);
		if (esp) {
			sscanf(get_Phonenumber(&esp->data[4]),"%d",&dummy);
			fprintf(stdout,"MSN%d:\t%d\n",esp->data[2],dummy);
			free(esp);
		}
	}

	data[0] = 0;
	esp = send_request(fd, 0, K_CTI_ANL, ANL_TLN_INFO, 1, data);
	while (esp && esp->length) {
		for (i=0; i < (esp->length / 6); i++) {
			if (!((esp->data[(i*6)+2]==0x00) || (esp->data[(i*6)+2]==0xFF)))
				fprintf(stdout,"NST:\t%s\t%s\t%s\n",
					get_Phonenumber(&esp->data[(i*6) + 4]),
					teltyp(esp->data[(i*6)+2]),
					telutyp(esp->data[(i*6)+3]));
		}
		free(esp);
		data[0] = data[0]++;
		esp = send_request(fd, 0, K_CTI_ANL, ANL_TLN_INFO, 1, data);
	}
	fprintf(stdout,"\n");
	if (esp) 
		free(esp);
}

/********************************************************************
 *
 ********************************************************************/
void quit()
{
	done++;
	close(m_sock);
}

/********************************************************************
 *
 ********************************************************************/
void usage(char *prog)
{
	SplashScreen();

	fprintf(stderr,"usage: %s [OPTIONS]\n"
		"\t[-H<Host>] Host that runs Timm\n"
		"\t[-P<Port>] Port that Timm is running on\n"
		"\t[-o<file>] Filename to save\n"
		"\t[-i<file>] Filename to load\n"
		"\tOutput defaults to stdout\n"
		"\t%s\n",prog,rcsid);
	exit(1);
}

/********************************************************************
 *
 ********************************************************************/
int main(int argc, char **argv)
{
	extern char	*optarg;
	extern int	optind;
	char 		*outfile = NULL;
	char 		*infile = NULL;
	char 		*machine = "localhost";
	int			port=TCPPORT;
	char		*user    = getenv("LOGNAME");
	char		*passwd  = NULL;
	int			c;

	read_maKsrc(&user, &passwd, &machine, &port);

#define OPTSTRING	"H:P:o:i:h"
	while ((c=getopt(argc,argv,OPTSTRING)) != EOF)
		switch(c) {
			case 'P':
				port=atoi(optarg);;
				break;
			case 'H':
				machine=optarg;
				break;
			case 'i':
				infile=optarg;
				break;
			case 'o':
				outfile=optarg;
				break;
			default:
				usage(argv[0]);
		}

	if (!set_user(user)) usage(argv[0]);
	if (!set_passwd(passwd)) usage(argv[0]);

	signal(SIGINT,quit);
	signal(SIGHUP,quit);
	signal(SIGTERM,quit);

	InitProcs(0, stderr);

	/* Must handle Login myself */
	SetUsrProc(K_CTI_ESP, &handle_CTI_ESP);
	
	if((m_sock=Server_Connect(machine,port))== -1)
		exit(1);

	if (!infile)
		inf = stdin;
	else 
		inf=fopen(infile,"r");

	if (!outfile)
		outf = stdout;
	else 
		outf=fopen(outfile,"w+");

	conn_request(m_sock, PERM_USER, AUTH_CLEARTXT, "Kim", ASYNC);
	PacketWaitAsync(m_sock);
	if (done == 100)
		start(m_sock);

	close(m_sock);
	return(0);
}
