/********************************************************************
 * Lion/handle.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: handle.c,v 1.2 1999/12/30 20:55:57 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "handle.h"

extern int		m_sock;
extern int		done;
extern FILE		*outf;
extern char		*intnr;
extern char		*extnr;
extern char		*msg;
extern WORD		prjid;

/********************************************************************
 * Gebuehreninfo umrechnen und anzeigen
 ********************************************************************/
void do_charge_info(EspMsg *esp)
{
	int sum = esp->data[2] + (esp->data[3] << 8) + (esp->data[4] << 16) + (esp->data[5] << 24);
	fprintf(stdout,"06 Gebuehren: %d Pfennige\n",sum);
	fflush(stdout);
}

/********************************************************************
 * Interne Verbindung aufbauen
 ********************************************************************/
void do_dial_intern(int fd)
{
	BYTE	data[255];
	char	*ptr;
	int		len = 0;
	int		i;

	ptr = set_Phonenumber(extnr, &len);

    data[0] = 0x00;
    data[1] = 0x00;
    data[2] = 0x00;
    data[3] = ptr[0];
    data[4] = 0xFF;
    data[5] = 0x01;
	data[6] = 0x00;

	if (msg != NULL && strlen(msg)) {
		len = strlen(msg);
		data[6] = (len + 1) & 0x00FF;
		for (i=0; i < len; i++)
    		data[7+i] = msg[i];
    	data[7+i] = 0x00;
		len++;
	}

    send_server(fd, 0, K_CTI_VBG | K_CTI_SET, VBG_TLN_SETUP, (7 + len), data);
}

/********************************************************************
 * (Externe) Verbindung aufbauen
 ********************************************************************/
void do_dial(int fd)
{
	BYTE	data[255];
	char	*ptr;
	int		len = 0;
	int		i;

	if (strlen(extnr) == 2) {
		do_dial_intern(fd);
		return;
	}

	ptr = set_Phonenumber(extnr, &len);

    data[0] = 0x00;
    data[1] = 0x00;
    data[2] = (prjid != 0xFFFF) ? ( prjid       & 0x00FF) : 0xFF;
    data[3] = (prjid != 0xFFFF) ? ((prjid >> 8) & 0x00FF) : 0xFF;
    data[4] = 0x00;
    data[5] = 0x00;
    data[6] = 0x01;

	for (i=0; i < len; i++)
    	data[7+i] = ptr[i];

    data[19] = 0x00;

	if (msg != NULL && strlen(msg)) {
		len = strlen(msg) ;
		data[19] = (len + 1) & 0x00FF;
		for (i=0; i < len; i++)
    		data[20+i] = msg[i];
    	data[20+i] = 0x00;
		len++;
	}

    send_server(fd, 0, K_CTI_VBG | K_CTI_SET, VBG_AMT_SETUP, (20 + len), data);
}

/********************************************************************
 * 
 ********************************************************************/
void do_register(int fd, int onoff)
{
	BYTE	data[2];
	char	*ptr;
	int		flags;
	int		len;

	ptr = set_Phonenumber(intnr, &len);

	if (onoff) {
		flags = K_CTI_TLN | K_CTI_SET;
	} else {
		flags = K_CTI_TLN | K_CTI_SET | K_CTI_DEL;
	}
	
	data[0] = ptr[0];
	data[1] = ptr[1];
	send_server(fd, 0, flags, TLN_REGISTER, 2, data);
}

/********************************************************************
 * Handle ESP group commands
 ********************************************************************/
int handle_CTI_ESP(int fd, EspMsg  *esp)
{
	switch(esp->element) {
		case ESP_CONN_REQ :
			if (!S_CTI_ERR(esp->gruppe)) {
				if (esp->data[0] == AUTH_DENY) {
					fprintf(stderr,"Login denied! %s\n", (esp->length > 1) ? (char *)&esp->data[1] : "");
					exit(1);
				} else if (esp->data[0] == AUTH_CLEARTXT) {
					conn_auth_cleartext(fd, get_user(), get_passwd(), ASYNC);
				} else if (esp->data[0] == 0) {
					done = 100;
					sleep(2);
				} else {
					fprintf(stderr,"Wrong authorization method!\n");
					exit(1);
				}
			}
			return(HANDLED);
			break;
		case ESP_CONN_AUTH :
			if (!S_CTI_ERR(esp->gruppe)) {
				do_register(fd, TRUE);
			} else {
				fprintf(stderr,"Login denied! %s\n", (esp->length > 1) ? (char *)&esp->data[0] : "");
				exit(1);
			}
			return(HANDLED);
			break;
	}
	return(NOT_HANDLED);
}


/********************************************************************
 * Handle ESP group commands
 ********************************************************************/
int handle_CTI_TLN(int fd, EspMsg  *esp)
{
	switch(esp->element) {

		case TLN_REGISTER :
			if (!S_CTI_DEL(esp->gruppe) && !S_CTI_ERR(esp->gruppe)) {
				do_dial(fd);
			} else if (S_CTI_DEL(esp->gruppe) && !S_CTI_ERR(esp->gruppe)) {
				done = TRUE;
			} else {
				fprintf(stderr,"99 Telefon konnte nicht reserviert werden!\nExit!\n");
				exit(1);
			}
			return(HANDLED);
			break;
	}
	return(NOT_HANDLED);
}

/********************************************************************
 * Handle ESP group commands
 ********************************************************************/
int handle_CTI_VBG(int fd, EspMsg  *esp)
{
	static int dccount = 0;
	static BYTE vbgid[2] = {0,0};

	switch(esp->element) {

		case VBG_TLN_SETUP :
			if (!S_CTI_ERR(esp->gruppe)) {
				vbgid[0] = esp->data[0];
				vbgid[1] = esp->data[1];
			} else {
				switch (esp->data[2]) {
					case 0x02:
						fprintf(stderr,"99 Teilnehmer besetzt!\n");
						done = TRUE;
						break;
					case 0x7F:
						break;
					default:
						fprintf(stderr,"ERROR: 0x%02X\n",esp->data[2]);
				}
			}
			return(HANDLED);
			break;

		case VBG_AMT_SETUP :
			if (!S_CTI_ERR(esp->gruppe)) {
				vbgid[0] = esp->data[0];
				vbgid[1] = esp->data[1];
			}
			return(HANDLED);
			break;

		case VBG_DISP_HOOK_OFF_MSG :
			if (S_CTI_SET(esp->gruppe) && !S_CTI_DEL(esp->gruppe)) {
				fprintf(stdout,"01 Bitte den Hoerer abheben!\n");
				fflush(stdout);
			} 
			if (S_CTI_SET(esp->gruppe) && S_CTI_DEL(esp->gruppe)) {
				fprintf(stdout,"02 Anwahl...\n");
				fflush(stdout);
				do_dial(fd);
			} 
			return(HANDLED);
			break;
			
		case VBG_CHARGING_INFO :
			do_charge_info(esp);
			return(HANDLED);
			break;

		case VBG_ALERT :
			fprintf(stdout,"20 Klingeln...\n");	
			return(HANDLED);
			break;

		case VBG_CONNECTED_NUMBER :
			fprintf(stdout,"05 Verbunden mit: %s\n",get_Phonenumber(&esp->data[2]));	
			fflush(stdout);
			return(HANDLED);
			break;

		case VBG_CONNECT :
			fprintf(stdout,"03 Verbindung ist hergestellt!\n");
			fflush(stdout);
			return(HANDLED);
			break;

		case VBG_DISCONNECT :
			fprintf(stdout,"10 Auflegen!\n");
			fflush(stdout);
			if (esp->data[0] == vbgid[0] && esp->data[1] == vbgid[1])
				do_register(fd, FALSE);
			return(HANDLED);
			break;

		case VBG_EXT_CALL :
			fprintf(stdout,"04 Eingehender Anruf von %s!\n",get_Phonenumber(&esp->data[3]));
			fflush(stdout);
			return(HANDLED);
			break;

		case VBG_INT_CALL :
			fprintf(stdout,"04 Eingehender Anruf von Nebenstelle %s!\n",get_Phonenumber(&esp->data[2]));
			fflush(stdout);
			return(HANDLED);
			break;

	}
	return(NOT_HANDLED);
}

