/********************************************************************
 * Lion/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.2 1999/12/30 20:55:57 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <version.h>
#include <config.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "handle.h"

extern int    done;

FILE	*outf;
FILE	*inf;
int		m_sock;
char	*intnr;
char	*extnr;
char	*msg;
WORD	prjid = 0xFFFF;

/********************************************************************
 *
 ********************************************************************/
void SplashScreen()
{
	fprintf(stderr,"\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"        #                  L I O N                     #\n");
	fprintf(stderr,"        #          Line Interface On Network           #\n");
	fprintf(stderr,"        #      Copyright 1998  by Chris Benndorf       #\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"\n");
	fflush(stderr);
}

/********************************************************************
 *
 ********************************************************************/
void quit()
{
	done++;
	close(m_sock);
}

/********************************************************************
 *
 ********************************************************************/
void usage(char *prog)
{
	SplashScreen();
	fprintf(stderr,"usage: %s [OPTIONS] [-I]INT_NR [-X]EXT_NR\n"
		"\t[-i<PRID>] Projektnummer (0-999)\n"
		"\t[-m<MSG> ] UserToUser-Message (max. 132 Zeichen)\n"
		"\t[-u<USER>] Username\n"
		"\t[-p<PASS>] Password\n"
		"\t[-H<Host>] Host that runs Timm\n"
		"\t[-P<Port>] Port that Timm is running on\n"
		"\t[-o<file>] File to write to\n"
		"\tOutput defaults to stdout\n"
		"\t[-u,-p,-H,-P] do overwrite ~/.maKsrc\n"
		"\t%s\n",prog,rcsid);
	exit(1);
}

/********************************************************************
 *
 ********************************************************************/
int main(int argc, char **argv)
{
	extern char	*optarg;
	extern int	optind;
	char 		*outfile = NULL;
	char 		*infile	 = NULL;
	char		*user    = getenv("LOGNAME");
	char		*passwd  = NULL;
	char 		*machine = "localhost";
	int			port=TCPPORT;
	int			c;


	read_maKsrc(&user, &passwd, &machine, &port);

#define OPTSTRING	"H:P:u:p:o:I:X:i:m:h"
	while ((c=getopt(argc,argv,OPTSTRING)) != EOF)
		switch(c) {
			case 'P':
				port=atoi(optarg);;
				break;
			case 'H':
				machine=optarg;
				break;
			case 'u':
				user=optarg;
				break;
			case 'p':
				passwd=optarg;
				break;
			case 'I':
				intnr=optarg;
				break;
			case 'X':
				extnr=optarg;
				break;
			case 'i':
				prjid=atoi(optarg);
				if (prjid > 999) usage(argv[0]);
				break;
			case 'm':
				msg=optarg;
				break;
			case 'o':
				outfile=optarg;
				break;
			default:
				usage(argv[0]);
		}

	if (intnr == NULL || extnr == NULL) {
		if (optind + 2 != argc) usage(argv[0]);

		intnr=argv[optind];	optind++;
		extnr=argv[optind];	optind++;
	} 

	if (!set_user(user)) usage(argv[0]);
	if (!set_passwd(passwd)) usage(argv[0]);

	signal(SIGINT,quit);
	signal(SIGHUP,quit);
	signal(SIGTERM,quit);

	InitProcs(0, stderr);

	/* Must handle Login myself */
	SetUsrProc(K_CTI_ESP, &handle_CTI_ESP);
	SetUsrProc(K_CTI_TLN, &handle_CTI_TLN);
	SetUsrProc(K_CTI_VBG, &handle_CTI_VBG);
	
	if((m_sock=Server_Connect(machine,port))== -1)
		exit(1);

	if (!infile)
		inf = stdin;
	else 
		inf=fopen(infile,"r");

	if (!outfile)
		outf = stdout;
	else 
		outf=fopen(outfile,"w+");

	conn_request(m_sock, PERM_CTI, AUTH_CLEARTXT, "Lion", ASYNC);
	PacketWaitAsync(m_sock);
	close(m_sock);
	return(0);
}
