/********************************************************************
 * RiKe/AFrame.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998, 1999, 2000
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: AFrame.cpp,v 1.1 1999/12/30 20:56:45 root Exp root $
 *
 ********************************************************************/

#include <iostream>
#include <qobject.h>
#include <kapp.h>
#include <ksock.h>
#include <kconfig.h>
#include <kprocess.h>
#include <ktopwidget.h>
#include <qframe.h>
#include <qmsgbox.h>
#include <qstring.h>
#include <qlistview.h>
#include <qtextstream.h>
#include <qfiledialog.h>
#include <qbitarray.h> 
#include <qprogressdialog.h> 
#include <qregexp.h> 
#include <kmenubar.h>
#include <qkeycode.h>
#include <qpopupmenu.h>
#include <qtimer.h>

#include <config.h>
#include <AnlageParm.h>
#include <TlnInfo.h>
#include <EPacket.h>
#include <MsgHandlerQt.h>
#include <Login.h>
#include <PNumber.h>
#include "EditD.h"
#include "LprD.h"
#include "Search.h"
#include "Dialer.h"
#include "AFrame.moc"

// #define EBUG 1

extern KConfig *kConf;

AFrame::AFrame(const char *name=0L): KTopLevelWidget(name)
{
	datei = new QPopupMenu;
	anlage = new QPopupMenu;
	eintrag = new QPopupMenu;

	insrID = eintrag->insertItem("Neu", this, SLOT(insertSel()),CTRL+Key_N);
	deleID = eintrag->insertItem("Loeschen", this, SLOT(deleteSel()),CTRL+Key_D);
	editID = eintrag->insertItem("Aendern", this, SLOT(editSel()),CTRL+Key_E);
	moveID = eintrag->insertItem("Verschieben", this, SLOT(moveSel()),CTRL+Key_M);
	eintrag->insertSeparator();
	findID = eintrag->insertItem("Suchen", this, SLOT(find()),CTRL+Key_F);
	fndnID = eintrag->insertItem("Weitersuchen", this, SLOT(find_next()),CTRL+Key_G);
	eintrag->insertSeparator();
	dialID = eintrag->insertItem("Anwaehlen", this, SLOT(dial()),CTRL+Key_T);
	loadID = datei->insertItem("Laden...", this, SLOT(loadList()),CTRL+Key_O);
	saveID = datei->insertItem("Sichern...", this, SLOT(saveList()),CTRL+Key_S);
	datei->insertSeparator();
	prntID = datei->insertItem("Drucken...", this, SLOT(printList()),CTRL+Key_P);
	datei->insertSeparator();
	quitID = datei->insertItem("Beenden", qApp, SLOT(quit()),CTRL+Key_Q);
	sendID = anlage->insertItem("Eintrag programmieren", this, SLOT(programEntry()));
	progID = anlage->insertItem("Liste programmieren", this, SLOT(programList()));
	
	haveData(false);
	haveConn(false);
	haveSel(false);

	KMenuBar *mbar = new KMenuBar(this);

	mbar->insertItem("Datei",datei);
	mbar->insertItem("Eintrag",eintrag);
	mbar->insertItem("Anlage",anlage);
	mbar->enableMoving(true);

	setMenu(mbar);

	sel = 0;
	kwzuse = new QBitArray(200);
	clearUseBits();


	QFont f("Courier", 12);

	list = new QListView(this);
	list->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	list->setFont(f);
	list->addColumn("KWZ",30);
	list->addColumn("Name",340);
	list->addColumn("LKZ",50);
	list->addColumn("OKZ",50);
	list->addColumn("Nummer",106);
	list->addColumn("GRP",30);
	list->setColumnAlignment(0,AlignLeft);
	list->setColumnAlignment(1,AlignLeft);
	list->setColumnAlignment(2,AlignRight);
	list->setColumnAlignment(3,AlignRight);
	list->setColumnAlignment(4,AlignRight);
	list->setColumnAlignment(5,AlignRight);
	list->setSorting(1,FALSE);
	list->setSorting(5,TRUE);
	list->setAllColumnsShowFocus(TRUE);

	resize(list->sizeHint().width() + 24, list->sizeHint().height());

	QObject::connect(list, SIGNAL(rightButtonPressed(QListViewItem*,const QPoint &, int)), SLOT(dopop(QListViewItem*,const QPoint &, int)));
	QObject::connect(list, SIGNAL(currentChanged(QListViewItem*)), SLOT(setSel(QListViewItem *)));
	QObject::connect(list, SIGNAL(selectionChanged(QListViewItem*)), SLOT(checkSel()));
	QObject::connect(list, SIGNAL(doubleClicked (QListViewItem*)), SLOT(editSel(QListViewItem *)));
	
	mbar->show();
	setFrameBorderWidth(3);
	setView(list,TRUE);

	show();

	logon();
}

AFrame::~AFrame()
{
}

/*
void AFrame::resizeEvent(QResizeEvent *)
{
	updateRects();
}
*/

void AFrame::logon() {
	Login login(this,"Login");
	if (!login.exec()) return;

	QString l_appl(qApp->argv()[0]);

	logontimer.start(40000,true);
	QObject::connect(&logontimer, SIGNAL(timeout() ), SLOT(logon_timeout()));

	mh = new MsgHandlerQt(
						login.Host(),
						login.Port(),
						&l_appl.data()[l_appl.findRev('/') + 1],
						login.User(),
						login.Pass(),
						MsgHandler::AUTH_CLEARTXT,
						MsgHandler::PERM_KONFIG | MsgHandler::PERM_CTI
						);

	QObject::connect(mh, SIGNAL(rdyMsg(ESPMsgHdr *) ), SLOT(start(ESPMsgHdr *)));
	QObject::connect(mh, SIGNAL(failMsg(MsgHandler*) ), SLOT(failure(MsgHandler*)));

	//mh->switchDebug(true);
	mh->esp_conn_req();
}

void AFrame::logon_timeout() {
	static int counter = 0;
	QString l_appl(qApp->argv()[0]);

	counter++;
	if (counter > 5) {
		QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString("Anmmeldung abermals fehlgeschlagen, kein weiterer Versuch!"));
		return;
	} 

	QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString("Anmmeldung an Anlage fehlgeschlagen!"));

	delete mh;
	logon();
}

void AFrame::dial() {
	Dialer x(mh, this, "Anwahl");

	x.exec() ;
}

void AFrame::editSel(QListViewItem *_sel) {
	editSel();
}

void AFrame::dopop(QListViewItem* item,const QPoint &point, int index) {
	eintrag->exec(point,index);
}

void AFrame::editSel() {
	if (!sel) return;

	QString title("Kurzwahl aendern");
	title.append(" (");
	title.append(sel->text(0));
	title.append(")");

	EditDialog ed(this,title);

	ed.SetData(sel->text(1), sel->text(0), sel->text(2), sel->text(3), sel->text(4), sel->text(5));

	if (ed.exec()) {
		delete sel;
		sel = new QListViewItem(list,ed.Kurzwahl(),ed.Name(),ed.Lkz(),ed.Okz(),ed.Telnr(),ed.Gruppe());
		list->setSelected(sel,TRUE);
		list->ensureItemVisible(sel);
		haveData(true);
	}
}

void AFrame::moveSel() {
	if (!sel) return;

	QString n;
	QString old(sel->text(0));
	int i = findUnused() + 100;
	if (i > 299) {
		QMessageBox::information( 0, "Kurzwahlen", "Keine freie Kurzwahl vorhanden!");
		return;
	}
	n.setNum(i);

	sel->setText(0,n);

	QString msg = "Eintrag #" + old + " nach #" + n + "verschoben!";
	QMessageBox::information( 0, "Kurzwahlen", msg);

	clearUseBits();
	QListViewItem *item = list->firstChild();
	while (item) {
		old = item->text(0);
		if (!kwzuse->testBit(old.toInt() - 100)) {
			kwzuse->setBit(old.toInt() - 100);
		} else {
			QString msg("Datensatz #");
			msg += old + " ist mehrfach vorhanden und muss auch verschoben werden!" +
					"\n(" + item->text(1) + ")";
			n = "Korrekturlauf";
			QMessageBox::information( 0, n, msg);
		}
		item = item->nextSibling();
	}

	haveData(true);
}

void AFrame::insertSel() {
	QString n;
	int i = findUnused() + 100;
	if (i > 299) {
		QMessageBox::information( 0, "Kurzwahlen", "Keine freie Kurzwahl vorhanden!");
		return;
	}
	n.setNum(i);

	QString title("Neue Kurzwahl");
	title.append(" (");
	title.append(n);
	title.append(")");

	EditDialog ed(this,title.data());

	ed.SetData("Ohne Namen",n.data(),"0049","6356","1234567","0");

	if (ed.exec()) {
		kwzuse->setBit(ed.Kurzwahl().toInt() - 100);
		sel = new QListViewItem(list,ed.Kurzwahl(),ed.Name(),ed.Lkz(),ed.Okz(),ed.Telnr(),ed.Gruppe());
		list->setSelected(sel,TRUE);
		list->ensureItemVisible(sel);
		haveData(true);
	}
}

void AFrame::find_next() {
	find(sel->nextSibling(),0);
}

void AFrame::find() {
	QString n;
	QString title("Suche");

	SearchDialog sd(this,title.data());

	if (sd.exec()) {
		find(list->firstChild(), sd.RegEx());
	}
}

void AFrame::find(QListViewItem *from, const char *search) {
	static QRegExp regx;

	if (search) {
		regx.setCaseSensitive(true);
		regx.setWildcard(true);
		regx = search;
	}

	if (regx.isEmpty()) {
		return;
	}

	QListViewItem *item = from;
	while (item) {
		int len;
		if (regx.match(item->text(1),0,&len) != -1) {
			sel = item;
			list->setSelected(sel,TRUE);
			list->ensureItemVisible(sel);
			break;
		}
		item = item->nextSibling();
	}
}

void AFrame::deleteSel() {
	if (sel == 0) return;

	QListViewItem *dummy = sel->nextSibling();
	if (!dummy) 
		dummy = sel->itemAbove();
	
	QString n;
	n = sel->text(0);
	kwzuse->clearBit(n.toInt() - 100);
	delete sel;
	haveData(true);

	if (dummy || (list->childCount() > 0)) {
		haveSel(true);
		sel = dummy;
		list->setSelected(sel,TRUE);
		list->ensureItemVisible(sel);
	} else {
		haveSel(false);
		sel = 0;
	}
}

void AFrame::checkSel() {
	haveSel((sel != 0));
}

void AFrame::setSel(QListViewItem *_sel) {
	sel = _sel;
	haveSel((sel != 0));
}

bool AFrame::sendEntry(QListViewItem *item) {
		QString number;

		QString lkz(item->text(2));
		QString okz(item->text(3));
		QString tln(item->text(4));
		if ((lkz.isEmpty()) || (lkz == __lkz__))
			if ((okz.isEmpty()) || (okz == __okz__))
				number = tln;
			else
				number = "0" + okz + tln;
		else
			number = lkz + okz + tln;

		PNumber kwz(number.data(), QString(item->text(0)).toInt() - 100, PNumber::KURZWAHL);

		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, kwz.data());
		if (ep.isError()) {
			return false;
		}

	return true;
}

void AFrame::programEntry() {
	bool good = true;

	if (!sel) return;
	if (!mh->config_start(true)) {
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Der Konfigurationswunsch wurde von der Anlage abgewiesen!");
		return;
	}

	if (!sendEntry(sel)) {
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Fehler beim Schreiben des Eintrags.");
		good = false;
	}

	if (!mh->config_end(true)) {
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Fehler beim Sichern der Konfigurationsdaten.");
		return;
	}

	if (good)
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Datensatz programmiert.");
}

void AFrame::programList() {
	QProgressDialog dial(this, "Kurzwahlen programmieren", TRUE);
	dial.setTotalSteps(200);
	dial.setLabelText("Kurzwahlen programmieren");
	dial.setCancelButtonText("Abbrechen");

	if (!mh->config_start(true)) {
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Der Konfigurationswunsch wurde von der Anlage abgewiesen!");
		return;
	}

	QListViewItem *item = list->firstChild();
	int i = 0;
	int progress = 0;

	// Kurzwahlen in Anlage schreiben
	while (item) {
		if (!sendEntry(item)) {
			QString err = "Fehler beim Schreiben des Eintrags \"";
			err += item->text(4);
			err += "\"!";
			QMessageBox::information( 0, "Kurzwahlen programmieren", err);
		}
		
		dial.setProgress(progress++);
		if (dial.wasCancelled()) return;

		item = item->nextSibling();
	}

	// Unbenutzte Kurzwahlen zur Sicherheit loeschen
	while((i=findUnused(i)) < 200) {
		PNumber kwz("", i++, PNumber::KURZWAHL);
#ifndef EBUG
		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, kwz.data());
#else
		cout << kwz;
#endif
		dial.setProgress(progress++);
		if (dial.wasCancelled()) return;
	}

	if (!mh->config_end(true)) {
		QMessageBox::information( 0, "Kurzwahlen programmieren", "Fehler beim Sichern der Konfigurationsdaten.");
		return;
	}
	dial.reset();
}

void AFrame::endPrint(KProcess *) {
	datei->setItemEnabled(prntID,true);
	QFile f("/tmp/RiKe.print");
	f.remove();
}

void AFrame::printList() {
	LprDialog lpr;
	if (!lpr.exec()) {
		return;
	}

	datei->setItemEnabled(prntID,false);

	QFile f("/tmp/RiKe.print");

	if (f.open(IO_WriteOnly)) {    // file opened successfully

		QTextStream t( &f );        // use a text stream

		QListViewItem *item = list->firstChild();
		int	line = 2;
	
		QString head;
		head.sprintf("%3s %-40s %25s %3s\n---------------------------------------------------------------------------",
			list->columnText(0), list->columnText(1), list->columnText(4), list->columnText(5));

		t << head << endl;

		// Fuer alle weiteren header noch nen Seitenvorschub reinsetzen
		head.prepend("");

		while (item) {
			QString n(item->text(2));
	
			if (n == "0049") {
				n.sprintf("0%s/%s",item->text(3),item->text(4));
			} else {
				n.sprintf("(%s) %s/%s",item->text(2),item->text(3),item->text(4));
			}

			QString s;
			s.sprintf("%3s %-40s %25s %3s",item->text(0),item->text(1),n.data(),item->text(5));
			
			if (line >= lpr.LprLines()) {
				t << head << endl;
				line = 2;
			}
	
			t << s << endl;
			line++;

			item = item->nextSibling();
		}
		f.close();
	} else {
		QMessageBox::information( 0, "Druckdatei anlegen", "Die Druckdatei \"/tmp/RiKe.print\" konnte nicht angelegt werden!");
		return;
	}

	KShellProcess *print = new KShellProcess();

	*print << lpr.LprName() << "/tmp/RiKe.print";

	QObject::connect(print, SIGNAL(processExited(KProcess *)), SLOT(endPrint(KProcess *)));

	print->start();
}

void AFrame::readtlb(QString& s) {
	QFile f(s);

	if ( f.open(IO_ReadOnly) ) {    // file opened successfully

		QTextStream t( &f );        // use a text stream
		QString s;
		QString d1;
		QString d2;
		QString d3;
		QString d4;
		QString d5;
		QString d6;

		while ( !t.eof() ) {        // until end of file...

			s = t.readLine();       // line of text excluding '\n'

			if (s[0] == '[' || s[0]=='#') continue;
			
			unsigned int n = 0;
			unsigned int l = s.length();
			unsigned int p = s.find(';',n);

			d1=s.mid(n,p - n);
			n = p + 1;
			p = s.find(';',n);
			d2=s.mid(n,p - n);
			n = p + 1;
			p = s.find(';',n);
			d3=s.mid(n,p - n);
			n = p + 1;
			p = s.find(';',n);
			d4=s.mid(n,p - n);
			n = p + 1;
			p = s.find(';',n);
			if (p < l) {			// Rike-Format
				d5=s.mid(n,p - n);
				n = p + 1;
				p = s.find(';',n);
				d6=s.mid(n,p - n);
			} else {			// Ackermann-Format
				d5=s.mid(n,l - n);
				d6.setNum(0);
			}

			if (d5.toInt() > 99) {
				if (!kwzuse->testBit(d5.toInt() - 100)) {
					kwzuse->setBit(d5.toInt() - 100);
				} else {
					QString msg("Datensatz #");
					msg += d5 + " ist mehrfach vorhanden und muss verschoben werden!" +
							"\n(" + d1 + ")";
					QMessageBox::information( 0, s, msg);
				}
				(void) new QListViewItem(list,d5,d1,d2,d3,d4,d6);
			}
			
		}
		f.close();
	} else 
		QMessageBox::information( 0, s, "Die Datei konnte nicht geoffnet werden!");
}

void AFrame::fileformat(QTextStream& t) {
	QListViewItem *item = list->firstChild();

	while (item) {
		t 	<< item->text(1) << ";" 
			<< item->text(2) << ";" 
			<< item->text(3) << ";" 
			<< item->text(4) << ";" 
			<< item->text(0) << ";" 
			<< item->text(5) << ";"
			<< endl;

		item = item->nextSibling();
	}

	
}

void AFrame::loadList() {
	QString f = QFileDialog::getOpenFileName(kConf->readEntry("TLBFILE", "./standard.tlb").data() , "*.tlb*", this );
	if ( !f.isEmpty() ) {
		kConf->writeEntry("TLBFILE", f);
		list->clear();
		clearUseBits();
		readtlb(f);
//		list->setSorting(1,FALSE);
		list->setSorting(5,TRUE);
		sel = 0;
		checkSel();
		haveData(false);
	} else {
		// the user cancelled the dialog
	}
}

void AFrame::saveList() {
	QString f = QFileDialog::getSaveFileName(kConf->readEntry("TLBFILE", "./standard.tlb").data() , "*.tlb*", this );
	if ( !f.isEmpty() ) {
		kConf->writeEntry("TLBFILE", f);
		QFile sf(f);

		if ( sf.open(IO_WriteOnly) ) {    // file opened successfully
			QTextStream t( &sf );        // use a text stream
			fileformat(t);
			sf.close();
			haveData(false);
		} else {
			QMessageBox::information( 0, f, "Die Datei konnte nicht zum Schreiben geoeffnet werden!");
		}
	} else {
		// the user cancelled the dialog
	}
}

void AFrame::start(ESPMsgHdr *esp)
{
	QString l_appl(qApp->argv()[0]);

	logontimer.stop();

	haveConn(true);

	{
		vector<BYTE> vec(3);
		vec[0] = 0x40;
		vec[1] = 0x00;
		vec[2] = 0x00;
		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_POR, MsgHandler::POR_LAND,vec);
		if ((ep.gruppe & K_CTI_ERR) != K_CTI_ERR) {
			BaseNumber phone(vector<BYTE>(ep.data.begin() + 3, ep.data.end()));
			__lkz__ =  "00";
			__lkz__ += phone.Phone().data();
		} else {
			QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString("Die Landeskennzahl konnte nicht ermittelt werden!"));
		}
	}

	{
		vector<BYTE> vec(3);
		vec[0] = 0x40;
		vec[1] = 0x00;
		vec[2] = 0x00;
		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_POR, MsgHandler::POR_ORT,vec);
		if ((ep.gruppe & K_CTI_ERR) != K_CTI_ERR) {
			BaseNumber phone(vector<BYTE>(ep.data.begin() + 3, ep.data.end()));
			__okz__ =  phone.Phone().data();
		} else {
			QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString("Die Ortsnetzkennzahl konnte nicht ermittelt werden!"));
		}
	}

	QObject::connect(mh, SIGNAL(espMsg(ESPMsgHdr *) ), SLOT(Handle_ESP(ESPMsgHdr *)));
}

void AFrame::failure(MsgHandler* lmh)
{
	QString l_appl(qApp->argv()[0]);

	if (lmh->failFatal())
		QMessageBox::critical( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString(lmh->failReason()));
	else
		QMessageBox::information( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString(lmh->failReason()));
}

void AFrame::Handle_ESP(ESPMsgHdr *esp) {
	QString l_appl(qApp->argv()[0]);

	if (esp->element == 0x02 && (esp->gruppe & 0x04 == 0x04)) { // Scheissegebaut->Rausgeflogen!
		QMessageBox::critical( 0, QString(&l_appl.data()[l_appl.findRev('/') + 1]), QString((const char *)&esp->data[2]));
		qApp->quit();
	}
}

void AFrame::clearUseBits() {
	kwzuse->fill(FALSE,kwzuse->size());
}

int AFrame::findUnused() {
	for (unsigned int i=0; i < kwzuse->size(); i++)
		if (!kwzuse->testBit(i)) return(i);
	return(201);
}

int AFrame::findUnused(int idx) {
	for (unsigned int i=idx; i < kwzuse->size(); i++)
		if (!kwzuse->testBit(i)) return(i);
	return(201);
}


void AFrame::haveData(bool yesno) {
	datei->setItemEnabled(saveID, yesno);
	datei->setItemEnabled(saveID, yesno);
}

void AFrame::haveConn(bool yesno) {
	anlage->setItemEnabled(progID, yesno);
	anlage->setItemEnabled(sendID, (yesno && (sel)));
#ifndef EBUG
	eintrag->setItemEnabled(dialID, false);
#endif
}

void AFrame::haveSel(bool yesno) {
	anlage->setItemEnabled(sendID, (yesno && mh->isConnected()));
	eintrag->setItemEnabled(deleID, yesno);
	eintrag->setItemEnabled(editID, yesno);
	eintrag->setItemEnabled(fndnID, yesno);
	eintrag->setItemEnabled(moveID, yesno);
}


