/********************************************************************
 * Rike/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.3 1999/12/30 20:57:53 root Exp root $
 *
 ********************************************************************/
 
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include <version.h>
#include <config.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "kwz.h"
#include "handle.h"

extern	int	done;
		int	m_sock;
		int	debug = 0;

/********************************************************************
 *
 ********************************************************************/
void SplashScreen()
{
	fprintf(stderr,"\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"        #                     RIKE                     #\n");
	fprintf(stderr,"        #   Rufnummern Integration & Kurzwahl Export   #\n");
	fprintf(stderr,"        #        Copyright 1998 by Chris Benndorf      #\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"\n");
	fflush(stderr);

}

/********************************************************************
 *
 ********************************************************************/
void quit()
{
	done++;
	close(m_sock);
}

/********************************************************************
 *
 ********************************************************************/
void usage(char *prog)
{
	SplashScreen();

    fprintf(stderr,"usage: %s [OPTIONS] [-i]<Phonebookfile> \n"
        "\t[-u<USER>] Username\n"
        "\t[-p<PASS>] Password\n"
        "\t[-H<Host>] Host that runs Timm\n"
        "\t[-P<Port>] Port that Timm is running on\n"
        "\t[-u,-p,-H,-P] do overwrite ~/.maKsrc\n"
        "\t%s\n",prog,rcsid);
    exit(1);
} 

/********************************************************************
 *
 ********************************************************************/
int main(int argc, char **argv)
{
    extern char *optarg;
    extern int optind;
	char			*tbook=NULL;
    char			*machine = "localhost";
    char			*user    = getenv("LOGNAME");
    char			*passwd  = NULL;
    int				 port=TCPPORT;
    int c;

	signal(SIGINT,quit);
	signal(SIGHUP,quit);
	signal(SIGTERM,quit);


	read_maKsrc(&user, &passwd, &machine, &port); 

#define OPTSTRING   "H:P:i:u:p:hd"

	while ((c=getopt(argc,argv,OPTSTRING)) != EOF)
		switch(c) {
			case 'H':
				machine=optarg;
				break;
			case 'P':
				port=atoi(optarg);
				break;
			case 'u':
				user=optarg;
				break;
			case 'p':
				passwd=optarg;
				break;
			case 'i':
				tbook=optarg;
				break;
			case 'd':
				debug++;
				break;
			default:
				usage(argv[0]);
		}

	if (!set_user(user)) usage(argv[0]);
	if (!set_passwd(passwd)) usage(argv[0]);

	if ((optind < argc) && (tbook == NULL)) {
		tbook = argv[optind];
	}

	if (tbook == NULL) usage(argv[0]);

	ReadTBook(tbook);

	InitProcs(0, stderr);

	/* Want to handle Login myself */
	SetUsrProc(K_CTI_ESP, &handle_CTI_ESP);

	/* Need to know if ANL_KFG_START is ok */
	SetUsrProc(K_CTI_ANL, &handle_CTI_ANL);

	/* Need to know if ORG_RNR_FA is ok */
	SetUsrProc(K_CTI_ORG, &handle_CTI_ORG);

	if((m_sock=Server_Connect(machine,port))== -1)
		exit(1);

	conn_request(m_sock, PERM_KONFIG, AUTH_CLEARTXT, "Rike", ASYNC);
	PacketWaitAsync(m_sock);


	return(0);
}
