/********************************************************************
 * analyse.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: analyse.c,v 1.3 1999/12/30 20:58:59 root Exp $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <glob.h>
#include <analyse.h>

/********************************************************************
 *
 ********************************************************************/
const char* showFlags(unsigned char n)
{
	switch (n & 3) {
		case 0:
			return "GET";
		case 1:
			return "SET";
		case 2:
			return "ADD";
		case 3:
			return "DEL";
	}
}

/********************************************************************
 *
 ********************************************************************/
const char* const textGrp[] = {
	"ESP (Client/Server-Daten)",
	"ANL (Anlagendaten)",
	"POR (Portkonfiguration)",
	"GEB (Gebhrenmanagement)",
	"ORG (Organisation)",
	"TLN (Teilnehmereinstellungen)",
	"AUX (Sonderfkt (Alarm...))",
	"XXX (Unbekannt)",
	"XXX (Unbekannt)",
	"XXX (Unbekannt)",
	"STA (CTI-Status)",
	"OFF (CTI-Offline-Steuerung)",
	"VBG (CTI-Verbdg.-Steuerung)",
	"EVT (CTI-Event)",
	"TST (Testmodi (LCB Monitoring))",
	"FAB (Fabriktest)"
};

const char* txtGrp(unsigned char n)
{
	return(textGrp[((n & 0xF0) >> 4)]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textEsp[] = {
	"ESP_CONN_REQ",
	"ESP_CONN_AUTH",
	"ESP_SRVR_MSG",
	"ESP_SRVR_STAT",
	"ESP_SRVR_KILL"
};

const size_t textEspSize = sizeof(textEsp)/sizeof(char*);
const char*  const textEspUnkown = "ESP_UNKNOWN";

const char* txtEsp(unsigned char n) {
	return((n>=textEspSize) ? textEspUnkown : textEsp[n] );
};


/********************************************************************
 *
 ********************************************************************/
const char* const textAnl[] = {
	"ANL_LINK_ID",
	"ANL_HERSTELLER",
	"ANL_TYP",
	"ANL_SW_VERSION",
	"ANL_DB_VERSION",
	"ANL_BETRIEBSART",
	"ANL_ALLE_INT_RNR",
	"ANL_DATUM_ZEIT",
	"ANL_KEYPAD",
	"ANL_UNKNOWN1",
	"ANL_MAX_PARA",
	"ANL_VWZ_OLD",
	"ANL_DEFAULT",
	"ANL_PLUG_AND_GO",
	"ANL_UNKNOWN2",
	"ANL_FERNSCHALTEN",
	"ANL_UNKNOWN3",
	"ANL_X31_KFG",
	"ANL_TLN_INFO",
	"ANL_MASTER_SLAVE",
	"ANL_KFG_START",
	"ANL_KFG_ENDE",
	"ANL_KFG_DATA_START",
	"ANL_KFG_DATA_ENDE",
	"ANL_KFG_DATA_ERROR",
	"ANL_CRC_EPROM",
	"ANL_CRC_EEPROM",
	"ANL_FEHLERPROTOKOLL",
	"ANL_RESET",
	"ANL_FW_KENNWORT",
	"ANL_FW_ENDE",
	"ANL_B_KANAELE",
	"ANL_VERIFY_LINK_ID",
	"ANL_DB_Q_MSG",
	"ANL_DB_Q_ANL_INFO",
	"ANL_FW_VBG",
	"ANL_FW_KW_QUITTUNG",
	"ANL_FW_REL_1_X",
	"ANL_FW_REL_1_X_AKTIV",
	"ANL_SM_Q_MSG"
};
const size_t textAnlSize = sizeof(textAnl)/sizeof(char*);
const char*  const textAnlUnkown = "ANL_UNKNOWN";

const char* txtAnl(unsigned char n)
{
	return((n>=textAnlSize) ? textAnlUnkown : textAnl[n] );
};


/********************************************************************
 *
 ********************************************************************/
const char* const textPor[] = {
	"POR_LAND",
	"POR_ORT",
	"POR_HAUPTANL",
	"POR_FIRMA",
	"POR_MSN_AVA_TLN",
	"POR_MSN_AVA_PROFIL",
	"POR_MSN_AVA_SCHALTZEIT",
	"POR_MSN_AVA_RWS_PROFIL",
	"POR_AVA_RWS_ZEIT",
	"POR_BETRIEBSART",
	"POR_AMT_RNR",
	"POR_INT_RNR",
	"POR_MODE",
	"POR_SER_DIENST",
	"POR_SER_RATE",
	"POR_DDI_LEN",
	"POR_DDI_RNR",
	"POR_TFE_AVA_TLN",
	"POR_TFE_AVA_PROFIL",
	"POR_TFE_AVA_SCHALTZEIT",
	"POR_TFE_RUF_DAUER",
	"POR_TFE_UMLEITUNG",
	"POR_DB_AVA_TLN",
	"POR_NACHBAR_MSN_AVA",
	"POR_NACHBAR_TFE_AVA"
};
const size_t textPorSize = sizeof(textPor)/sizeof(char*);
const char* const textPorUnkown = "POR_UNKNOWN";

const char* txtPor(unsigned char n)
{
	return((n>=textPorSize) ? textPorUnkown : textPor[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textGeb[] = {
	"GEB_WAEHRUNG",
	"GEB_FAKTOR",
	"GEB_PTT_SATZ",
	"GEB_TLN_SATZ",
	"GEB_TLN_KONTO",
	"GEB_ANL_SUMME",
	"GEB_PRJ_SUMME",
	"GEB_FA_SUMME",
	"GEB_TLN_SUMME",
	"GEB_TLN_STATUS",
	"GEB_ANL_STATUS",
	"GEB_TLN_INFO"
};
const size_t textGebSize = sizeof(textGeb)/sizeof(char*);
const char* const textGebUnkown = "GEB_UNKNOWN";

const char* txtGeb(unsigned char n)
{
	return((n>=textGebSize) ? textGebUnkown : textGeb[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textOrg[] = {
	"ORG_UNKNOWN1",
	"ORG_UNKNOWN2",
	"ORG_LOGO_FA",
	"ORG_TLN_FA",
	"ORG_TLN_GRP",
	"ORG_RNR_FA",
	"ORG_RNR_GRP",
	"ORG_FA_STATUS",
	"ORG_FA_CODE",
	"ORG_FA_ANZAHL",
	"ORG_FA_MOH",
	"ORG_LCR_ONKZ",
	"ORG_FA_LCR_PROFIL",
	"ORG_FA_LCR_MODE"
};
const size_t textOrgSize = sizeof(textOrg)/sizeof(char*);
const char* const textOrgUnkown = "ORG_UNKNOWN";

const char* txtOrg(unsigned char n)
{
	return((n>=textOrgSize) ? textOrgUnkown : textOrg[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textTln[] = {
	"TLN_REGISTER",
	"TLN_SCHALTSTATUS",
	"TLN_SCHALTRECHT",
	"TLN_UNKNOWN1",
	"TLN_UNKNOWN2",
	"TLN_EINRICHT_RECHT",
	"TLN_AMTSRECHT",
	"TLN_MSN",
	"TLN_TERMIN",
	"TLN_WECKZEIT",
	"TLN_DIENST",
	"TLN_FUNKTION",
	"TLN_RNR",
	"TLN_PIN",
	"TLN_SCHLOSS",
	"TLN_KONTO",
	"TLN_DISPLAY",
	"TLN_TONRUF",
	"TLN_LAUTSPRECHER",
	"TLN_VIP_FILTER",
	"TLN_RUF_DELAY",
	"TLN_FILTER",
	"TLN_TASTATUR",
	"TLN_MAIL",
	"TLN_UMLEITUNG",
	"TLN_IDENT",
	"TLN_HOTLINE",
	"TLN_TEST_AB_RUFTON",
	"TLN_TEST_AB_HOERTON",
	"TLN_RECHTE",
	"TLN_SONSTIGES",
	"TLN_GRUPPEN_AVA",
	"TLN_CHEF_SEK"
};
const size_t textTlnSize = sizeof(textTln)/sizeof(char*);
const char* const textTlnUnkown = "TLN_UNKNOWN";

const char* txtTln(unsigned char n)
{
	return((n>=textTlnSize) ? textTlnUnkown : textTln[n]);
};


/********************************************************************
 *
 ********************************************************************/
const char* const textAux[] = {
	"AUX_SENSOR_FKT",
	"AUX_ALARM_ZEIT",
	"AUX_ALARM_DELAY",
	"AUX_ALARM_DAUER",
	"AUX_ALARM_RNR",
	"AUX_ALARM_WAHL",
	"AUX_ALARM_UUS",
	"AUX_ALARM_STATUS",
	"AUX_AKTOR_FKT",
	"AUX_AKTOR_ZUSTAND",
	"AUX_AKTOR_MANUELL",
	"AUX_TLN_VIRTUELL",
	"AUX_PIN_VIRTUELL"
};
const size_t textAuxSize = sizeof(textAux)/sizeof(char*);
const char* const textAuxUnkown = "AUX_UNKNOWN";

const char* txtAux(unsigned char n)
{
	return((n>=textAuxSize) ? textAuxUnkown : textAux[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textSta[] = {
	"STA_RESSOURCEN",
	"STA_HOOK",
	"STA_TLN",
	"STA_TLN_LANG",
	"STA_TLN_MITTEL",
	"STA_TLN_KURZ",
	"STA_TEST_DATA"
};
const size_t textStaSize = sizeof(textSta)/sizeof(char*);
const char* const textStaUnkown = "STA_UNKNOWN";

const char* txtSta(unsigned char n)
{
	return((n>=textStaSize) ? textStaUnkown : textSta[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textOff[] = {
	"OFF_STOP_SENSOR_PROZ",
	"OFF_NEUE_SEITE",
	"OFF_POR_UMLEITUNG",
	"OFF_TUER_OFFNER",
	"OFF_ANL_ANRUFLISTE",
	"OFF_TLN_ANRUFLISTE",
	"OFF_TLN_UMLEITUNG",
	"OFF_TLN_DISPLAY",
	"OFF_TLN_MIKROFON",
	"OFF_TLN_NOTIZ",
	"OFF_TLN_WWH",
	"OFF_TLN_ANRUFLISTE_SYS",
	"OFF_TLN_RUFLISTE",
	"OFF_TLN_WAHLMODUS",
	"OFF_TLN_DETAILS",
	"OFF_TLN_0F"
};
const size_t textOffSize = sizeof(textOff)/sizeof(char*);
const char* const textOffUnkown = "OFF_UNKNOWN";

const char* txtOff(unsigned char n)
{
	return((n>=textOffSize) ? textOffUnkown : textOff[n]);
};

/********************************************************************
 *
 ********************************************************************/
const char* const textVbg[] = {
	"VBG_HOOK_OFF",
	"VBG_HOOK_ON",
	"VBG_AMT_SETUP",
	"VBG_TLN_SETUP",
	"VBG_DISCONNECT",
	"VBG_INTRUSION",
	"VBG_OVERLAP_SEND",
	"VBG_CALL_U_TRANSFER",
	"VBG_CALL_I_TRANSFER",
	"VBG_CALL_E_TRANSFER",
	"VBG_3PTY",
	"VBG_CONF",
	"VBG_HOLD",
	/*"VBG_SUSPEND",			/* unbenutzt, nicht definiert */
	"VBG_CONNECT",
	"VBG_TLN_WAHLVORBEREITUNG",
	"VBG_UNKNOWN0F",
	"VBG_UNKNOWN10",
	"VBG_UNKNOWN11",
	"VBG_UNKNOWN12",
	"VBG_UNKNOWN13",
	"VBG_UNKNOWN14",
	"VBG_UNKNOWN15",
	"VBG_UNKNOWN16",
	"VBG_UNKNOWN17",
	"VBG_UNKNOWN18",
	"VBG_UNKNOWN19",
	"VBG_UNKNOWN1A",
	"VBG_UNKNOWN1B",
	"VBG_UNKNOWN1C",
	"VBG_UNKNOWN1D",
	"VBG_UNKNOWN1E",
	"VBG_UNKNOWN1F",
	"VBG_INT_CALL",
	"VBG_EXT_CALL",
	"VBG_CHARGING_AOCS_INF",
	"VBG_CHARGING_INFO",
	"VBG_DISP_HOOK_OFF_MSG",
	"VBG_DISPLAY_STRING",
	"VBG_DISPLAY_FMT",
	"VBG_USER_2_USER",
	"VBG_ALERT",
	"VBG_CONNECTED_NUMBER",
	"VBG_UNKNOWN2A",
	"VBG_UNKNOWN2B",
	"VBG_UNKNOWN2C",
	"VBG_UNKNOWN2D",
	"VBG_UNKNOWN2E",
	"VBG_UNKNOWN2F",
	"VBG_ANL_VERIFY_LINK_ID (VBG_READY)",
	"VBG_UNKNOWN31",
	"VBG_UNKNOWN32",
	"VBG_UNKNOWN33",
	"VBG_UNKNOWN34",
	"VBG_UNKNOWN35",
	"VBG_UNKNOWN36",
	"VBG_UNKNOWN37",
	"VBG_UNKNOWN38",
	"VBG_UNKNOWN39",
	"VBG_UNKNOWN3A",
	"VBG_UNKNOWN3B",
	"VBG_UNKNOWN3C",
	"VBG_UNKNOWN3D",
	"VBG_UNKNOWN3E",
	"VBG_UNKNOWN3F",
	"VBG_CALL_ROUTING_LOG",
	"VBG_CALL_ROUTING"
};
const size_t textVbgSize = sizeof(textVbg)/sizeof(char*);
const char* const textVbgUnkown = "VBG_UNKNOWN";

const char* txtVbg(unsigned char n)
{
	return((n>=textVbgSize) ? textVbgUnkown : textVbg[n]);
};


/********************************************************************
 *
 ********************************************************************/
const char* const textEvt[] = {
	"EVT_RESSOURCEN",
	"EVT_HOOK_STATUS",
	"EVT_TLN_STATUS",
	/*"EVT_TLN_STATUS_MITTEL",	/* undef */
	/*"EVT_TLN_STATUS_KURZ",	/* undef */
	"EVT_ANL",
	"EVT_POR",
	"EVT_GEB",
	"EVT_ORG",
	"EVT_TLN",
	"EVT_AUX"
};
const size_t textEvtSize = sizeof(textEvt)/sizeof(char*);
const char* const textEvtUnkown = "EVT_UNKNOWN";

const char* txtEvt(unsigned char n)
{
	return((n>=textEvtSize) ? textEvtUnkown : textEvt[n]);
};



/********************************************************************
 *
 ********************************************************************/
const char* const txtUnknown = "UNKNOWN";

const char* text(unsigned char group, unsigned char elem)
{
	switch(group & 0xF0) {
		case G_ESP:
			return(txtEsp(elem));
			break;
		case G_ANL:
			return(txtAnl(elem));
			break;
		case G_POR:
			return(txtPor(elem));
			break;
		case G_GEB:
			return(txtGeb(elem));
			break;
		case G_ORG:
			return(txtOrg(elem));
			break;
		case G_TLN:
			return(txtTln(elem));
			break;
		case G_AUX:
			return(txtAux(elem));
			break;
		case G_STA:
			return(txtSta(elem));
			break;
		case G_OFF:
			return(txtOff(elem));
			break;
		case G_VBG:
			return(txtVbg(elem));
			break;
		case G_EVT:
			return(txtEvt(elem));
			break;
		default:
			return(txtUnknown);
	}
}

