/********************************************************************
 * analyse.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: analyse.h,v 1.3 1999/12/30 21:00:15 root Exp root $
 *
 ********************************************************************/
#ifndef __ANALYSE_H__
#define __ANALYSE_H__
 
#define K_CTI_GS_MASK   0x01
#define K_CTI_GET       0x00
#define K_CTI_SET       0x01
#define K_CTI_AD_MASK   0x02
#define K_CTI_ADD       0x00
#define K_CTI_DEL       0x02
#define K_CTI_OE_MASK   0x04
#define K_CTI_OK        0x00
#define K_CTI_ERR       0x04
#define K_CTI_CR_MASK   0x08
#define K_CTI_CMD       0x00
#define K_CTI_RSP       0x08

#define K_CTI_TYP_MASK  HNIBBLE_MASK
#define K_CTI_FLG_MASK  LNIBBLE_MASK
 
#define G_ESP	0x00	/* Euracom Server Protokoll */
#define G_ANL	0x10	/* Anlagendaten */
#define G_POR	0x20	/* Portkonfiguration */
#define G_GEB	0x30	/* Gebhrenmanagement */
#define G_ORG	0x40	/* Organisation */
#define G_TLN	0x50	/* Teilnehmereinstellungen */
#define G_AUX	0x60	/* Sonderfkt (Alarm...) */
#define G_STA	0xA0	/* CTI-Status (Hook Infos, Zustnde, ....) */
#define G_OFF	0xB0	/* CTI-Offline-Stg */
#define G_VBG	0xC0	/* CTI-Verbindungsstg. (Auf- u. Abbau) */
#define G_EVT	0xD0	/* CTI-Event */
#define G_TST	0xE0	/* Testmodi (LCB Monitoring) */
#define G_FAB	0xF0	/* Fabriktest */

/*
 * Elementkennungen - Euracom Server Protokoll
 */
enum {
	ESP_CONN_REQ=0x00,
	ESP_CONN_AUTH,
	ESP_SRVR_MSG,
	ESP_SRVR_STAT,
	ESP_SRVR_KILL,
	ESP_SRVR_WAKE
};

/*
 * Elementkennungen - Anlagenbasisdaten
 */
enum {
	ANL_LINK_ID=0x00,
	ANL_HERSTELLER,			
	ANL_TYP,				
	ANL_SW_VERSION,
	ANL_DB_VERSION,			/* unbenutzt */
	ANL_BETRIEBSART,
	ANL_ALLE_INT_RNR,		/* unbenutzt */
	ANL_DATUM_ZEIT,			/* unbenutzt */
	ANL_KEYPAD,				/* unbenutzt */
	ANL_UNKNOWN1,
	ANL_MAX_PARA,
	ANL_VWZ_OLD,			/* unbenutzt */
	ANL_DEFAULT,
	ANL_PLUG_AND_GO,		/* unbenutzt */
	ANL_UNKNOWN2,
	ANL_FERNSCHALTEN,
	ANL_UNKNOWN3,
	ANL_X31_KFG,
	ANL_TLN_INFO,
	ANL_MASTER_SLAVE,
	ANL_KFG_START,
	ANL_KFG_ENDE,
	ANL_KFG_DATA_START,
	ANL_KFG_DATA_ENDE,
	ANL_KFG_DATA_ERROR,
	ANL_CRC_EPROM,
	ANL_CRC_EEPROM,
	ANL_FEHLERPROTOKOLL,
	ANL_RESET,				/* unbenutzt */
	ANL_FW_KENNWORT,
	ANL_FW_ENDE,
	ANL_B_KANAELE,			/* unbenutzt */
	ANL_VERIFY_LINK_ID,
	ANL_DB_Q_MSG,
	ANL_DB_Q_ANL_INFO,
	ANL_FW_VBG,
	ANL_FW_KW_QUITTUNG,
	ANL_FW_REL_1_X,
	ANL_FW_REL_1_X_AKTIV,
	ANL_SM_Q_MSG
};

/*
 * Elementekennungen - Portkonfiguration
 */
enum {
	POR_LAND,
	POR_ORT,
	POR_HAUPTANL,
	POR_FIRMA,					/* unbenutzt */
	/*POR_MSN_AVA				/* unbenutzt */
	POR_MSN_AVA_TLN,
	POR_MSN_AVA_PROFIL,
	POR_MSN_AVA_SCHALTZEIT,
	POR_MSN_AVA_RWS_PROFIL,
	POR_AVA_RWS_ZEIT,
	POR_BETRIEBSART,			/* unbenutzt */
	POR_AMT_RNR,
	POR_INT_RNR,
	POR_MODE,
	POR_SER_DIENST,
	POR_SER_RATE,
	POR_DDI_LEN,
	POR_DDI_RNR,
	POR_TFE_AVA_TLN,
	POR_TFE_AVA_PROFIL,
	POR_TFE_AVA_SCHALTZEIT,
	POR_TFE_RUF_DAUER,
	POR_TFE_UMLEITUNG,
	POR_DB_AVA_TLN,
	POR_NACHBAR_MSN_AVA,
	POR_NACHBAR_TFE_AVA
};

/*
 * Elementekennungen - Gebhrenverwaltung
 */
enum {
	GEB_WAEHRUNG,			/* unbenutzt */
	GEB_FAKTOR,
	GEB_PTT_SATZ,
	GEB_TLN_SATZ,
	GEB_TLN_KONTO,
	GEB_ANL_SUMME,
	GEB_PRJ_SUMME,
	GEB_FA_SUMME,
	GEB_TLN_SUMME,
	GEB_TLN_STATUS,
	GEB_ANL_STATUS,
	GEB_TLN_INFO
};

/*
 * Elementekennungen - Organisation
 */
enum {
	ORG_UNKNOWN1,
	ORG_UNKNOWN2,
	ORG_LOGO_FA,
	ORG_TLN_FA,
	ORG_TLN_GRP,			/* unbenutzt */
	ORG_RNR_FA,
	ORG_RNR_GRP,			/* unbenutzt */
	ORG_FA_STATUS,
	ORG_FA_CODE,
	ORG_FA_ANZAHL,
	ORG_FA_MOH,
	ORG_LCR_ONKZ,
	ORG_FA_LCR_PROFIL,
	ORG_FA_LCR_MODE
};

/*
 * Elementekennungen - Teilnehmereinstellungen
 */
enum {
	TLN_REGISTER,
	TLN_SCHALTSTATUS,
	TLN_SCHALTRECHT,			/* unbenutzt */
	TLN_UNKNOWN1,
	TLN_UNKNOWN2,
	TLN_EINRICHT_RECHT,			/* unbenutzt */
	TLN_AMTSRECHT,				/* unbenutzt */
	TLN_MSN,
	TLN_TERMIN,					/* unbenutzt */
	TLN_WECKZEIT,				/* unbenutzt */
	TLN_DIENST,					/* unbenutzt */
	TLN_FUNKTION,				/* unbenutzt */
	TLN_RNR,					/* unbenutzt */
	TLN_PIN,					/* nicht implementiert */
	TLN_SCHLOSS,				/* nicht implementiert */
	TLN_KONTO,					/* unbenutzt */
	TLN_DISPLAY,				/* nicht implementiert */
	TLN_TONRUF,					/* nicht implementiert, */
	TLN_LAUTSPRECHER,			/* nicht implementiert, */
	TLN_VIP_FILTER,				/* nicht implementiert */
	TLN_RUF_DELAY,				/* nicht implementiert */
	TLN_FILTER,					/* nicht implementiert */
	TLN_TASTATUR,				/* nicht implementiert */
	TLN_MAIL,					/* nicht implementiert */
	TLN_UMLEITUNG,
	TLN_IDENT,
	TLN_HOTLINE,
	TLN_TEST_AB_RUFTON,			/* unbenutzt */
	TLN_TEST_AB_HOERTON,		/* unbenutzt */
	TLN_RECHTE,
	TLN_SONSTIGES,
	TLN_GRUPPEN_AVA,
	TLN_CHEF_SEK
};

/*
 * Elementekennungen - Sonderfunktionen
 */
enum {
	AUX_SENSOR_FKT,
	AUX_ALARM_ZEIT,
	AUX_ALARM_DELAY,
	AUX_ALARM_DAUER,
	AUX_ALARM_RNR,
	AUX_ALARM_WAHL,
	AUX_ALARM_UUS,
	AUX_ALARM_STATUS,			/* nicht implementiert */
	AUX_AKTOR_FKT,
	AUX_AKTOR_ZUSTAND,
	AUX_AKTOR_MANUELL,
	AUX_TLN_VIRTUELL,
	AUX_PIN_VIRTUELL
};

/*
 * Elementekennungen - CTI STatus
 */
enum {
	STA_RESSOURCEN,
	STA_HOOK,
	STA_TLN,					/* unbenutzt */
	STA_TLN_LANG,
	STA_TLN_MITTEL,
	STA_TLN_KURZ,
	STA_TEST_DATA
};

/*
 * Elementekennungen - CTI Offline Steuerung
 */
enum {
	OFF_STOP_SENSOR_PROZ,		/* nicht implementiert */
	OFF_NEUE_SEITE,				/* nicht implementiert */
	OFF_POR_UMLEITUNG,			/* nicht implementiert */
	OFF_TUER_OFFNER,			/* nicht implementiert */
	OFF_ANL_ANRUFLISTE,
	OFF_TLN_ANRUFLISTE,
	OFF_TLN_UMLEITUNG,
	OFF_TLN_DISPLAY,			/* nicht implementiert */
	OFF_TLN_MIKROFON,			/* nicht implementiert */
	OFF_TLN_NOTIZ,				/* nicht implementiert */
	OFF_TLN_WWH,				/* nicht implementiert */
	OFF_TLN_ANRUFLISTE_SYS,		/* nicht implementiert */
	OFF_TLN_RUFLISTE,			/* nicht implementiert */
	OFF_TLN_WAHLMODUS,			/* nicht implementiert */
	OFF_TLN_DETAILS,			/* nicht implementiert */
	OFF_TLN_0F					/* nicht implementiert */
};

/*
 * Elementekennungen - CTI Verbindungssteuerung
 */
enum {
	VBG_HOOK_OFF,
	VBG_HOOK_ON,				/* unbenutzt */
	VBG_AMT_SETUP,
	VBG_TLN_SETUP,
	VBG_DISCONNECT,
	VBG_INTRUSION,
	VBG_OVERLAP_SEND,
	VBG_CALL_U_TRANSFER,
	VBG_CALL_I_TRANSFER,
	VBG_CALL_E_TRANSFER,
	VBG_3PTY,
	VBG_CONF,
	VBG_HOLD,
	/*VBG_SUSPEND,				/* unbenutzt, nicht definiert */
	VBG_CONNECT,
	VBG_TLN_WAHLVORBEREITUNG,	/* nicht implementiert */
	VBG_UNKNOWN0F,
	VBG_UNKNOWN10,
	VBG_UNKNOWN11,
	VBG_UNKNOWN12,
	VBG_UNKNOWN13,
	VBG_UNKNOWN14,
	VBG_UNKNOWN15,
	VBG_UNKNOWN16,
	VBG_UNKNOWN17,
	VBG_UNKNOWN18,
	VBG_UNKNOWN19,
	VBG_UNKNOWN1A,
	VBG_UNKNOWN1B,
	VBG_UNKNOWN1C,
	VBG_UNKNOWN1D,
	VBG_UNKNOWN1E,
	VBG_UNKNOWN1F,
	VBG_INT_CALL,
	VBG_EXT_CALL,
	VBG_CHARGING_AOCS_INF,		/* unbenutzt */
	VBG_CHARGING_INFO,
	VBG_DISP_HOOK_OFF_MSG,
	VBG_DISPLAY_STRING,
	VBG_DISPLAY_FMT,
	VBG_USER_2_USER,
	VBG_ALERT,
	VBG_CONNECTED_NUMBER,
	VBG_UNKNOWN2A,
	VBG_UNKNOWN2B,
	VBG_UNKNOWN2C,
	VBG_UNKNOWN2D,
	VBG_UNKNOWN2E,
	VBG_UNKNOWN2F,
	VBG_ANL_VERIFY_LINK_ID,
	VBG_UNKNOWN31,
	VBG_UNKNOWN32,
	VBG_UNKNOWN33,
	VBG_UNKNOWN34,
	VBG_UNKNOWN35,
	VBG_UNKNOWN36,
	VBG_UNKNOWN37,
	VBG_UNKNOWN38,
	VBG_UNKNOWN39,
	VBG_UNKNOWN3A,
	VBG_UNKNOWN3B,
	VBG_UNKNOWN3C,
	VBG_UNKNOWN3D,
	VBG_UNKNOWN3E,
	VBG_UNKNOWN3F,
	VBG_CALL_ROUTING_LOG,
	VBG_CALL_ROUTING
};

/*
 * Elementekennungen - CTI Events
 */
enum {
	EVT_RESSOURCEN,				/* unbenutzt */
	EVT_HOOK_STATUS,
	EVT_TLN_STATUS,
	/*EVT_TLN_STATUS_MITTEL		/* undef */
	/*EVT_TLN_STATUS_KURZ		/* undef */
	EVT_ANL,
	EVT_POR,
	EVT_GEB,
	EVT_ORG,
	EVT_TLN,
	EVT_AUX
};

const char* showFlags(unsigned char n);
const char* txtGrp(unsigned char n);
const char* text(unsigned char group, unsigned char elem);

#endif __ANALYSE_H__
