/********************************************************************
 * lib/cl_utils.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: cl_utils.h,v 1.4 1999/12/30 21:00:15 root Exp root $
 *
 ********************************************************************/
 
#ifndef	__CL_UTILS_H__
#define __CL_UTILS_H__

#include <sys/types.h>
#include <sys/socket.h>

#define MAX_APPSTR_LEN	64
#define MAX_USRSTR_LEN	32

typedef struct cti_client cl_mgmt;
typedef void	(*IProc)(cl_mgmt *cl);
typedef BYTE *(*RProc)(cl_mgmt *cl, int *len);
typedef int		(*WProc)(cl_mgmt *cl, BYTE *buf, int len);

struct io_buffer 
{
        size_t	size;
        size_t	pos;
        BYTE	*ptr;
};


struct cti_client{
	cl_mgmt	*prev;
	cl_mgmt	*next;
	int		magic;
	RProc	readProc;
	WProc	writeProc;
	IProc	idleProc;
	int		fd;
	int		err;
	int		pkg_sent;
	int		pkg_rcvd;
	int		pkg_serr;
	int		pkg_rerr;
	int		pkg_left;
	int		accepted;
	int		closed;
	struct io_buffer read_buf;
	time_t	last_pak;
	time_t	login;
	WORD	ctyp;
	WORD	clinkid;
	BYTE	rndid;
	BYTE	flgnr;
	BYTE	pcknr;
	BYTE	perm;
	BYTE	tln_res;
	BYTE	inval;
	struct sockaddr addr;
	char	appl[MAX_APPSTR_LEN];
	char	user[MAX_USRSTR_LEN];
	char	pasw[MAX_USRSTR_LEN];
};

enum {
    CTI_CL_NOINIT = 0,
    CTI_CL_LISTEN,
    CTI_CL_SERIAL,
    CTI_SVRCTRL,
    CTI_CL_APP,
    CTI_CL_TELNET
};

#define	UNDEF_INFO_HEADER	""
#define	ADDR_INFO_HEADER	"LinkID Username Programm       IP-ADDR"
#define	ADDR_INFO_FORMAT	"0x%04X %-8s %-14s %03d.%03d.%03d.%03d"
#define	NORM_INFO_HEADER	"LinkID Username Programm       Typ        Rechte Res.Tel Loginzeit"
#define	NORM_INFO_FORMAT	"0x%04X %-8s %-14s %-10s   0x%02X      %02X %-14s"
#define	MGMT_INFO_HEADER	"LinkID Username Programm       Random PackNr FolgNr SktNr"
#define	MGMT_INFO_FORMAT	"0x%04X %-8s %-14s 0x%02X   0x%02X   0x%02X %5d"
#define	TECH_INFO_HEADER	"LinkID Username Programm       PkgSent PkgRcvd ErrsSent ErrRcvd PkgLeft SktNr"
#define	TECH_INFO_FORMAT	"0x%04X %-8s %-14s %7d %7d %8d %7d %7d %5d"

cl_mgmt	*Cl_create();
cl_mgmt	*Cl_add(cl_mgmt *this);
cl_mgmt	*Cl_delete(cl_mgmt *this);
cl_mgmt	*Cl_init(cl_mgmt *this, int fd, unsigned short clid, char *appl, char *user);
cl_mgmt	*Cl_last();
cl_mgmt *Cl_first();
cl_mgmt *Cl_index(int idx);
cl_mgmt *Cl_find_serial();
cl_mgmt *Cl_find_invalid();
cl_mgmt *Cl_find_by_lid(WORD lid);
cl_mgmt *Cl_find_by_rndid(BYTE rndid);
cl_mgmt	*Cl_fdset_all(fd_set *set);
cl_mgmt *Cl_touch_all();
cl_mgmt *Cl_send_all_ctrl(void *buf, int len);
cl_mgmt *Cl_send_all(void *buf, int len);
cl_mgmt *Cl_send(cl_mgmt *this, BYTE *buf, int len);
cl_mgmt	*Cl_accept(int fd);
cl_mgmt *Cl_print(cl_mgmt *this, FILE *fp);
cl_mgmt	*Cl_status(FILE *fp);
char	*Cl_strstatus(int idx, int typ);
void	Cl_close(cl_mgmt *this);
int		Cl_checkUser(cl_mgmt *this);
int		Cl_is_ctrl(cl_mgmt *this);
int		Cl_is_app(cl_mgmt *this);
int		Cl_is_closed(cl_mgmt *this);
int		Cl_highfd();

#endif	__CL_UTILS_H__
