/********************************************************************
 * level3.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: level3.c,v 1.4 1999/12/30 21:00:15 root Exp root $
 *
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>

#include <glob.h>
#include <analyse.h>
#include <log.h>
#include <cl_utils.h>
#include <level2pbx.h>
#include <level2srvr.h>
#include <level2clnt.h>
#include <level2common.h>
#include <level3.h>

LONG _serv_pnum = 0x0080;

/********************************************************************
 * Paket analysieren
 ********************************************************************/
void l3_analyse_pack(cl_mgmt *cl, ANLMsgHdr *msg)
{
	char 	*group;
	char 	*command;
	WORD		lid;
	int			i;

	group	= (char *)txtGrp(CTI_TYP(msg->gruppe));
	command = (char *)text(CTI_TYP(msg->gruppe), msg->element);
	lid		= ((msg->linkid_Hi << 8) | msg->linkid_Lo);

	logdate(LOG_L3PACKET);
    logprintf(LOG_L3PACKET,"\nLEVEL 3 Analyse:\n");
    logprintf(LOG_L3PACKET,"\tTYP   : %s [%s]\n", command, group);
    logprintf(LOG_L3PACKET,"\tAPPL  : %-13s\tUSER   : %-13s\tFD:\t%d\n", ((cl) ? cl->appl : "UNKNOWN"), ((cl) ? cl->user : "UNKNOWN"), ((cl) ? cl->fd : -1));
	logprintf(LOG_L3PACKET,"\tLINKID: %04X\t\tPACKNR : %02X\t\tANLNR : %02X\n",lid, msg->paketnummer,msg->anlage);
	logprintf(LOG_L3PACKET,"\tPTYP  : %02X\t\tCOMMAND: %02X\t", CTI_GRP(msg->gruppe), msg->element);
	logprintf(LOG_L3PACKET,"\tFLAGS : %s %s %s\n",
		S_CTI_RSP(msg->gruppe) ? "RSP" : "CMD",
		S_CTI_ERR(msg->gruppe) ? "ERR" : "OK ",
		showFlags(msg->gruppe));
	logprintf(LOG_L3PACKET,"\tLAENGE: %02X\t\tDATA  : ", msg->length);
	for (i=0; i < msg->length; i++)
		logprintf(LOG_L3PACKET,"%02X ", msg->data[i]);
	logprintf(LOG_L3PACKET,"\n");
}

/********************************************************************
 * Speicher fuer neues Paket holen
 ********************************************************************/
void *l3_new_msg(size_t len)
{
	char	*rp;

	if ((rp = calloc(len, 1)) == NULL) {
		logerror("Allocating memory");
		exit(1);
	}

	return((void *)rp);
}

/********************************************************************
 * Weitergabe der ESP-Pakete an LEVEL 5
 * ESP Pakete werden nicht an die Anlage gesendet
 ********************************************************************/
void l3_ESP_packet(cl_mgmt *cl, ESPMsgHdr *msg)
{
	switch (msg->element) {
		case ESP_CONN_REQ :
			l5_conn_req(cl, msg);
			break;
		case ESP_CONN_AUTH :
			l5_conn_auth(cl, msg);
			break;
		case ESP_SRVR_MSG : 
			/* Only sent by server */
			break;
		case ESP_SRVR_STAT : 
			l5_srvr_stat(cl, msg);
			break;
		case ESP_SRVR_KILL : 
			l5_srvr_kill(cl, msg);
			break;
	}
}

/********************************************************************
 * Paketnummer des Superclients (Servers) hochzaehlen
 ********************************************************************/
BYTE l3_inc_server_pnum()
{
	if (_serv_pnum >= 0x00FE)
		_serv_pnum = 0x0080;
	else
		_serv_pnum++;

	return((BYTE)_serv_pnum);
}

/********************************************************************
 * Broadcasts (nur Anlage?) weiterleiten
 ********************************************************************/
void l3_server_broadcast(cl_mgmt *src, ANLMsgHdr *msg)
{
	ESPMsgHdr	*np;
	int			len;

	/* STA_TEST (S0-Monitor) kann wirklich massenhaft Log erzeugen! */
	if (logOK(LOG_BROADCAST)) {
		if (((CTI_TYP(msg->gruppe) == G_STA) && (msg->element == STA_TEST_DATA))) {
			logdate(LOG_MSG);
			logprintf(LOG_MSG,"S0-Bus-Mon Broadcast from %s/%s\n", src->appl, src->user);
			if (logOK(LOG_STA_TEST))
				l3_analyse_pack(src, msg);
		} else {
			logdate(LOG_MSG);
			logprintf(LOG_MSG,"Broadcast MSG from %s/%s\n", src->appl, src->user);
			l3_analyse_pack(src, msg);
		}
	}

	len = ((int)(msg->length) & 0x00FF);

	np = l3_new_msg(sizeof(ESPMsgHdr) + (len - 1));

	np->folgenummer = 0xFF;
	np->anlage		= msg->anlage;
	np->gruppe		= msg->gruppe;
	np->element		= msg->element;
	np->length		= msg->length;

	memcpy(np->data, msg->data, len);

	Cl_send_all(np, sizeof(ESPMsgHdr) + (len - 1));

	free(np);
}

/********************************************************************
 * Falls zu lange keine Pakete von der Anlage kommen, pruefen ob
 * LINK_IDs noch gueltig. Nach RESET der Anlage loest dies sowieso
 * ein INVAL_LINKIDs aus, muss also nur von einem Client gemacht
 * werden. (ab 3D10 scheinbar unnoetig)
 ********************************************************************/
void l3_verfylink_get(cl_mgmt *this)
{
	ANLMsgHdr	*req;
	cl_mgmt		*ser;

	/* Ein Client reicht zur pruefung auf RESET */
	/* Sonst gibt es ein Dauerfeuer an Requests! */
	Cl_touch_all();

	ser = Cl_find_serial();

	req = l3_new_msg(sizeof(ANLMsgHdr) - 1);

	req->paketnummer	= ++this->pcknr;
	req->anlage			= 0;
	req->linkid_Hi		= (BYTE)(this->clinkid >> 8 );
	req->linkid_Lo		= (BYTE)(this->clinkid &0x00FF);
	req->gruppe			= G_ANL | K_CTI_ADD | K_CTI_OK | K_CTI_CMD | K_CTI_GET;
	req->element		= ANL_VERIFY_LINK_ID;
	req->length			= 0;

	l3_send(ser, req, 7);
	logdate(LOG_MSG);
	logprintf(LOG_MSG,"Checking VERIFY_LINKID for %s/%s\n", this->appl, this->user);
	l3_analyse_pack(this, req);

	free(req);

	this->last_pak = time(NULL);
}

/********************************************************************
 * Antworten auf LINK_ID-Pruefung der Anlage. Nur fuer gueltige Clients
 ********************************************************************/
int l3_verfylink_rsp(cl_mgmt *ser, cl_mgmt *cl, ANLMsgHdr *msg)
{
	ANLMsgHdr	*rsp;

	if (!cl) return(MSG_NOT_HANDLED);

	if (Cl_is_closed(cl)) return(MSG_NOT_HANDLED);

	cl->last_pak = time(NULL);

	rsp = l3_new_msg(sizeof(ANLMsgHdr) - 1);

	rsp->paketnummer	= ++cl->pcknr;
	rsp->anlage			= 0;
	rsp->linkid_Hi		= msg->linkid_Hi;
	rsp->linkid_Lo		= msg->linkid_Lo;
	rsp->gruppe			= msg->gruppe | K_CTI_RSP;
	rsp->element		= msg->element;
	rsp->length			= 0;

	l3_send(ser, rsp, 7);
	logdate(LOG_VERIFYPKT);
	logprintf(LOG_VERIFYPKT,"Answering VERIFY_LINKID for %s/%s\n", cl->appl, cl->user);

	free(rsp);

	return(MSG_HANDLED);
}

/********************************************************************
 * Neue LINK_ID beantragen. Auch Uebergabe von Permissions (ab 3.0)
 ********************************************************************/
void l3_linkid_req(cl_mgmt *src)
{
	ANLMsgHdr	*msg;
	cl_mgmt		*ser;
	time_t		now;
	BYTE		rndid;

	if (src->inval > 3) {
		logdate(LOG_MSG);
		logprintf(LOG_MSG, "No further LINKID requesting for Client %d - already requested %d times\n",src->fd,src->inval);
		Cl_close(src);
		return;
	}

	ser = Cl_find_serial();

	/* Zur Erkennung muss eine eindeutige RND_ID mitgesendet werden. */
	do {
		now = time(NULL);
		rndid = src->fd + (BYTE)((now >> src->fd) ^ (now >> (1+(int) (10.0*rand()/(RAND_MAX+1.0)))));
	} while (Cl_find_by_rndid(rndid) && (rndid == 0));
	
	src->rndid = rndid;
//	if (src->inval) 
		src->inval++;

	msg = l3_new_msg(sizeof(ANLMsgHdr));

	msg->paketnummer	= ++src->pcknr;
	msg->anlage		= 0;
	msg->linkid_Hi	= 0;
	msg->linkid_Lo	= 0;
	msg->gruppe		= G_ANL | K_CTI_GET | K_CTI_ADD | K_CTI_OK | K_CTI_CMD;
	msg->element	= ANL_LINK_ID;
	msg->length		= 2;
	msg->data[0]	= src->rndid;
	msg->data[1]	= src->perm;

	l3_send(ser, msg, 9);

	logdate(LOG_MSG);
	logprintf(LOG_MSG, "Requesting LINKID for Client %d RND %02X\n",src->fd, src->rndid);
	l3_analyse_pack(NULL, msg);

	free(msg);
}

/********************************************************************
 * Unbenutzte LINK_ID freigeben (wichtig ab 4.0)
 ********************************************************************/
void l3_linkid_del(cl_mgmt *src)
{
	ANLMsgHdr	*msg;
	cl_mgmt		*ser;
	time_t		now;

	ser = Cl_find_serial();

	msg = l3_new_msg(sizeof(ANLMsgHdr));

	msg->paketnummer	= ++src->pcknr;
	msg->anlage		= 0;
	msg->linkid_Hi	= (src->clinkid >> 8);
	msg->linkid_Lo	= (src->clinkid & 0x00FF);
	msg->gruppe		= G_ANL | K_CTI_SET | K_CTI_DEL | K_CTI_OK | K_CTI_CMD;
	msg->element	= ANL_LINK_ID;
	msg->length		= 0;

	l3_send(ser, msg, 9);

	logdate(LOG_MSG);
	logprintf(LOG_MSG, "Deleting LINKID for Client %d LinkID %04X\n",src->fd, src->clinkid);
	l3_analyse_pack(NULL, msg);

	free(msg);
}

/********************************************************************
 * Antwort auf neue LINK_ID zuordnen. Ab und zu fehlt hier die RND_ID
 * (BUG in der Firmware!), dann gleich nochmal fuer den letzten
 * Client neu beantragen. (FEHLERTRAECHTIG BEI RESET!!!)
 ********************************************************************/
int l3_new_linkid(ANLMsgHdr *msg)
{
	cl_mgmt *cl = NULL;
	WORD	linkid;


	/* Bug in Firmware umgehen */
	if (msg->length == 0) {
			logprintf(LOG_MSG,"PBX reference failure! Should be handled through Cl_idle()\n");
/*
		cl=Cl_last();
		if (Cl_is_app(cl)) {
			l3_linkid_req(cl);
		}
*/
		return(MSG_HANDLED);
	}

	/* Kein passender Client gefunden (anhand der RND_ID) */
	if ((!S_CTI_ERR(msg->gruppe)) && ((cl = Cl_find_by_rndid(msg->data[0])) == NULL)) {
		logdate(LOG_MSG);
		l3_analyse_pack(NULL, msg);
		logprintf(LOG_MSG,"Client not found! %02X\n", msg->data[0]);
		return(MSG_NOT_HANDLED);
	}

	if (!S_CTI_ERR(msg->gruppe)) {
		cl->clinkid = (msg->linkid_Hi << 8) | msg->linkid_Lo;

		logdate(LOG_MSG);
		logprintf(LOG_MSG,"Got LINKID for Client\n");
		Cl_print(cl, NULL);
		cl->last_pak = time(NULL);
		if (cl->inval && cl->accepted) {
			cl->inval = 0;
		} else {
			logprintf(LOG_MSG,"Got LINKID - sending ACCEPT\n");
			cl->inval = 0;
			cl->accepted = 1;
			l5_conn_accept(cl);
			l5_srvr_wake();
		}
	} else {
		if (cl == NULL) cl = Cl_find_by_lid((msg->linkid_Hi << 8) | msg->linkid_Lo);
		logdate(LOG_MSG);
		if (!cl->clinkid) {
			logprintf(LOG_MSG,"PBX rejected LINKID for Client! ANL_LINKID_GET failed!\n");
			l5_conn_deny(cl, "ANL_LINKID_GET failed");
		} else {
			logprintf(LOG_MSG,"PBX rejected LINKID for Client! ANL_LINKID invalid!\n");
			l5_srvr_msg(cl, 0x01, 0x01, "PBX rejected LINKID for Client! ANL_LINKID invalid!");
		}
		Cl_close(cl);
	}

	return(MSG_HANDLED);
}

/********************************************************************
 * Alle LINK_IDs als ungueltig markieren
 ********************************************************************/
int l3_inval_linkids(ANLMsgHdr *msg)
{
	cl_mgmt *this = Cl_first();

	logdate(LOG_MSG);
	l3_analyse_pack(NULL, msg);
	logprintf(LOG_MSG,"Requesting new LINKIDs for all clients!\n");

	while (this != NULL) {
		if (Cl_is_app(this)) {
			this->inval = 1;
			this->clinkid = 0;
			l3_linkid_req(this);
		}
		this = this->next;
	}


	return(MSG_HANDLED);
}

/********************************************************************
 * Reservierte Telefone fuer abgemeldete Clients freigeben, sonst
 * bleiben die Telefone tot...
 ********************************************************************/
void l3_unregister_tln(cl_mgmt *src)
{
	ANLMsgHdr	*msg;
	cl_mgmt		*ser;

	ser = Cl_find_serial();

	logdate(LOG_MSG);
	logprintf(LOG_MSG,"Unregistering telephon (%02X) for client %s/%s!\n",
			src->tln_res, src->appl, src->user);

	msg = l3_new_msg(sizeof(ANLMsgHdr) + 1);

	msg->paketnummer	= ++src->pcknr;
	msg->anlage			= 0;
	msg->linkid_Hi		= (src->clinkid & 0xFF00) >> 8;
	msg->linkid_Lo		= src->clinkid & 0x00FF;
	msg->gruppe			= G_TLN | K_CTI_SET | K_CTI_DEL | K_CTI_OK | K_CTI_CMD;
	msg->element		= TLN_REGISTER;
	msg->length			= 2;
	msg->data[0]		= src->tln_res;
	msg->data[1]		= 0xFF;

	l3_send(ser, msg, 9);

	free(msg);
}

/********************************************************************
 * Pakete die der Server abhandelt:
 ********************************************************************/
int l3_server_handled(cl_mgmt *src, cl_mgmt *dst, ANLMsgHdr *msg)
{
	int	ret = MSG_NOT_HANDLED;

	/* Broadcasts an ALLE clients */
	if (dst == ((cl_mgmt *) -1L) && !((CTI_TYP(msg->gruppe) == G_ANL) && (msg->element == ANL_LINK_ID))) {
			l3_server_broadcast(src, msg);
			return(MSG_HANDLED);
	}

	switch (CTI_TYP(msg->gruppe)) {

		/* Alles was LINK_IDs betrifft, wird transparent fuer den Client */
		/* behandelt */
		case G_ANL :
				switch (msg->element) {
					case ANL_LINK_ID :
						if (!S_CTI_RSP(msg->gruppe) && S_CTI_DEL(msg->gruppe) && S_CTI_SET(msg->gruppe)){
							if (((msg->linkid_Hi == 0xFF) && (msg->linkid_Lo == 0xFF))) {	/* nach Anlagenreset */
								logprintf(LOG_MSG,"Handled by l3_inval_linkids\n");
								ret = l3_inval_linkids(msg);
							} else { /* Client hat Mist gebaut und fliegt raus */
								ret = l3_new_linkid(msg);
							}
						} else {
							if (S_CTI_DEL(msg->gruppe) && S_CTI_RSP(msg->gruppe)) {
								logprintf(LOG_MSG,"Deleting linkid %s\n",(S_CTI_ERR(msg->gruppe) ? "failed!" : "acknowledged."));
							} else {
								logprintf(LOG_MSG,"Handled by l3_new_linkid\n");
								ret = l3_new_linkid(msg);
							}
						}
						
						break;
					case ANL_VERIFY_LINK_ID :
						ret = l3_verfylink_rsp(src, dst, msg);
						break;
				}
			break;

		/* Wenn Telefone reserviert oder freigegeben werden, muessen */
		/* wir hier Buchfuehren, um sie ggf. selbst freizugeben */
		case G_TLN :
				switch (msg->element) {
					case TLN_REGISTER :	/* Fehler in der Firmware, TLN wird nicht fregegeben */
						if (!dst) return(ret);
						if (S_CTI_RSP(msg->gruppe) && !S_CTI_ERR(msg->gruppe)) {
							if (S_CTI_DEL(msg->gruppe)) {
								if (dst->tln_res == msg->data[0])
									dst->tln_res = 0;
							} else {
								dst->tln_res = msg->data[0];
								logdate(LOG_MSG);
								logprintf(LOG_MSG,"Client %s/%s registered phone %02X!\n",
										dst->appl, dst->user, dst->tln_res);
							}
						}
						return(MSG_NOT_HANDLED);
						break;
				}
			break;

		/* Mit reservierten Telefonen aendert sich die ANL_VERIFY_LINK_ID */
		/* in VBG_ANL_VERIFY_LINK_ID (mit zwei zusaetzlichen Bytes -> BUG! */
		case G_VBG :
				switch (msg->element) {
					case VBG_ANL_VERIFY_LINK_ID :
						if (!S_CTI_RSP(msg->gruppe) && !S_CTI_ERR(msg->gruppe))
							ret = l3_verfylink_rsp(src, dst, msg);
						break;
				}
			break;
	}

	return(ret);
}

/********************************************************************
 *
 ********************************************************************/
void l3_foreward_pbx_pack(cl_mgmt *cl, ANLMsgHdr *msg)
{
	ESPMsgHdr	*np;
	cl_mgmt 	*dst;
	WORD		linkid;

	linkid = ((msg->linkid_Hi << 8) | msg->linkid_Lo);

	if (linkid == 0xFFFF)
		dst = (cl_mgmt *) -1L ;				/* filter braodcasts */
	else
		dst = Cl_find_by_lid(linkid);

	if (l3_server_handled(cl, dst, msg))
		return;

	if (dst == NULL) {
		logprintf(LOG_MSG,"Unhandled Packet! [%s ; %s]\n",
			(char *)txtGrp(CTI_TYP(msg->gruppe)),
			(char *)text(CTI_TYP(msg->gruppe), msg->element));
		return;
	}

	l3_analyse_pack(cl, msg);
	np = l3_new_msg(sizeof(ESPMsgHdr) + msg->length);
 
	if (msg->paketnummer)
		np->folgenummer = dst->flgnr;
	else
		np->folgenummer = l3_inc_server_pnum();

	np->anlage	= msg->anlage;
	np->gruppe	= msg->gruppe;
	np->element	= msg->element;
	np->length	= msg->length;

	memcpy(np->data, msg->data, (int)(msg->length &0x00FF));

	l5_handle_packet(cl, dst, np);

	free(np);
}

/********************************************************************
 *
 ********************************************************************/
void l3_handle_cl_data(cl_mgmt *cl)
{
	ESPMsgHdr	*np;
	int		len;

	np = (ESPMsgHdr *)(cl->readProc)(cl, &len);

	if (!np) {
                /* if there was no free space, the client gets kicked */
/* 		if (np) free(np); */
		logdate(LOG_MSG);
		logprintf(LOG_MSG,"\nClosing client\n");
		Cl_print(cl, NULL);
		if (cl->tln_res)
			l3_unregister_tln(cl);
		l3_linkid_del(cl);
		Cl_close(cl);
		l5_srvr_wake();
	} else if (len) {
		switch (np->gruppe & K_CTI_TYP_MASK) {
			case G_ESP :
				l3_ESP_packet(cl, np);
				break;
			default :
				l5_handle_packet(cl, Cl_find_serial(), np);
				break;
		}
		free(np);
        }
        /* else: the packet is not complete, but retry-able. */
}

/********************************************************************
 *
 ********************************************************************/
int l3_handle_pbx_data(cl_mgmt *cl)
{
	BYTE 		*np;
	ANLMsgHdr	*msg;
	int			plen;

	np	= pbx_receive(cl, &plen);
	msg	= (ANLMsgHdr *)np;

	if (plen > 1)
		l3_foreward_pbx_pack(cl, msg);

	if (np) free(np);

	return(plen > 1);
}

/********************************************************************
 *
 ********************************************************************/
void l3_send_EspMsg(cl_mgmt *src, cl_mgmt *dst, ESPMsgHdr *msg)
{
	ANLMsgHdr *np;

	/* Control darf nichts an Anlage schicken */
	if (Cl_is_ctrl(src)) {
		
		return;
	}

	np = l3_new_msg(sizeof(ANLMsgHdr) + msg->length);

	src->flgnr			= msg->folgenummer;
	np->paketnummer		= ++src->pcknr;
	np->anlage			= msg->anlage;
	np->linkid_Hi		= (src->clinkid & 0xFF00) >> 8;
	np->linkid_Lo		= src->clinkid & 0x00FF;
	np->gruppe			= msg->gruppe;
	np->element			= msg->element;
	np->length			= msg->length;
	
	memcpy(np->data, msg->data, ((int)msg->length & 0x00FF));
	
	l3_send(dst, np, 7 + ((int)msg->length & 0x00FF));
	
	free(np);
}

/********************************************************************
 *
 ********************************************************************/
void l3_send(cl_mgmt *cl, void *data, size_t len)
{
	(cl->writeProc)(cl, data, len);
}

