/********************************************************************
 * Timm/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.5 1999/12/30 21:00:15 root Exp root $
 *
 ********************************************************************/
 
/*
	partly but shamelessly stolen from:
	mdial 9/1996 by G. Zander <hifi@scorpio.in-berlin.de>
*/

#include <stdio.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/termios.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>

#include "../version.h"
#include <../config.h>

#include <glob.h>
#include <cl_utils.h>
#include <level2common.h>
#include <level2pbx.h>
#include <level2srvr.h>
#include <log.h>

#include "../config.h"

/*
#ifndef	SERIALPORT
#define SERIALPORT	"/dev/ttyS10"
#endif

#ifndef	TCPPORT
#define TCPPORT		2001
#endif
*/

extern cl_mgmt	*clptr;
extern int		debug;
extern FILE 	*logfp;

int				quit=0;

void setSignals();

/*****************************************************************
 * 
 *****************************************************************/
void SplashScreen()
{
	fprintf(stdout,"\n\n\n");
	fprintf(stdout,"        ################################################\n");
	fprintf(stdout,"        #                  T I M M                     #\n");
	fprintf(stdout,"        #     Telephone Integration Main Module        #\n");
	fprintf(stdout,"        #     Copyright 1998  by Chris Benndorf        #\n");
	fprintf(stdout,"        ################################################\n");
	fprintf(stdout,"\n\n\n");
	sleep(1);

}

/*************************************************************************
 *
 * Eigenen Prozess in den Background bringen
 *
 *************************************************************************/
void background() {
	int	child_pid;
	int	nulldev_fd;
	int	i;

	child_pid = fork() ;
	switch (child_pid) {
		case -1:
			perror("fork") ;
			exit(ECHILD) ;
		case 0:
			setsid() ;
			if ((nulldev_fd = open(NULL_DEVICE, O_RDWR, 0)) != -1) {
				for (i = 0; i < 3; i++) {
					if (isatty(i)) {
						(i==0) ?
						dup2(nulldev_fd, i) :
						dup2(fileno(logfp), i) ;
					}
				}
				close(nulldev_fd) ;
			}
			break ;
		default:
		exit(0) ;
	}
}

/*****************************************************************
 * 
 *****************************************************************/
void cleanup(int signum){
	quit = 1;
	setSignals();
}

/*****************************************************************
 * 
 *****************************************************************/
void Status(int signum)
{
	Cl_status(logfp);
	setSignals();
}

/*****************************************************************
 * 
 *****************************************************************/
void ClearLog(int signum)
{
	if (logfp) {
		ftruncate(fileno(logfp),0);
		logdate(LOG_MSG);
		logprintf(LOG_MSG," Logfile truncated...\n\n");
	}
	setSignals();
}

/*****************************************************************
 * 
 *****************************************************************/
void handle_SIGPIPE(int signum)
{
        logerror("Received SIGPIPE!\n");
        setSignals();
}

/*****************************************************************
 * 
 *****************************************************************/
#define SAFE_SIGNAL(sig, handler) { if (signal (sig, handler) == SIG_IGN) \
                                        signal (sig, SIG_IGN);            \
                                  }

void setSignals(){
        SAFE_SIGNAL(SIGHUP, cleanup);
        SAFE_SIGNAL(SIGINT, cleanup);
        SAFE_SIGNAL(SIGPIPE, handle_SIGPIPE);
        SAFE_SIGNAL(SIGTERM, cleanup);
        SAFE_SIGNAL(SIGUSR1, Status);
        SAFE_SIGNAL(SIGUSR2, ClearLog);
}

/*****************************************************************
 * 
 *****************************************************************/
int parseSpeed(char *speed)
{
			if (!strcmp(speed,"B2400"))	 return(B2400);
	else	if (!strcmp(speed,"B4800"))  return(B4800);
	else	if (!strcmp(speed,"B9600"))  return(B9600);
	else	if (!strcmp(speed,"B19200")) return(B19200);
	else	if (!strcmp(speed,"B38400")) return(B38400);
	else	if (!strcmp(speed,"2400"))   return(B2400);
	else	if (!strcmp(speed,"4800"))   return(B4800);
	else	if (!strcmp(speed,"9600"))   return(B9600);
	else	if (!strcmp(speed,"19200"))  return(B19200);
	else	if (!strcmp(speed,"38400"))  return(B38400);
	else	return(SERSPEED);
}

/*****************************************************************
 * 
 *****************************************************************/
char *prSpeed(int speed)
{
	switch (speed) {
		case B2400:		return("B2400");
		case B4800:		return("B4800");
		case B9600:		return("B9600");
		case B19200:	return("B19200");
		case B38400:	return("B38400");
		default:		return("UNKNOWN");
	}
}

/*****************************************************************
 * Usage
 *****************************************************************/
void usage(char *prog)
{
	fprintf(stderr,"usage: %s [OPTIONS]\n",prog);
	fprintf(stderr,"\tOPTIONS:\n"
			"\t-d <debuglevel>\n"
			"\t-l <logfile>         (default: ./Timm.log)\n" 
			"\t-p <permission file> (default: ./timm.perm)\n" 
			"\t-s <serialport>      (default: " SERIALPORT ")\n"
			"\t-S <serialspeed>     (default: %s)\n"
			"\t-P <tcpport>         (default: %d)\n"
			"\t%s\n", prSpeed(SERSPEED), TCPPORT, rcsid);
	exit(1);
}

/*****************************************************************
 * MAIN
 *****************************************************************/
main(int argc, char **argv) 
{
	extern char	*optarg;
	extern int	optind;
	cl_mgmt		*cl;
	char 		*permfile	= "/usr/local/maKs/etc/timm.perm";
	char 		*logfile	= "/var/log/Timm.log";
	char 		*serport	= SERIALPORT;
	int			tcpport		= TCPPORT;
	int			serspeed	= SERSPEED;
	int			c;

	while ((c=getopt(argc,argv,"hd:l:s:S:P:p:")) != EOF)
		switch(c) {
			case 'S':
				serspeed=parseSpeed(optarg);
				break;
			case 'P':
				sscanf(optarg,"%i",&tcpport);
				break;
			case 's':
				serport = optarg;
				break;
			case 'p':
				permfile = optarg;
				break;
			case 'l':
				logfile = optarg;
				break;
			case 'd':
				sscanf(optarg,"%i",&debug);
				break;
			default:
				usage(argv[0]);
		}
	
	SplashScreen();

	fprintf(stdout,"\n"
		"TCP-Port   : %d\n"
		"PBX-Port   : %s\n"
		"PBX-Speed  : %s\n"
		"Logfile    : %s\n"
		"Permissions: %s\n"
		"Debuglevel : %d (0x%04X)\n",
		tcpport, serport, prSpeed(serspeed), logfile, permfile, debug, debug);

	auth_cleartxt_read(permfile);

	if ((logfp = fopen(logfile,"a")) == NULL) {
                perror ("unable to open logfile");
		exit(1);
	}

	background();
	setSignals();

	logdate(LOG_MSG);
	fprintf(logfp,"Timm is starting up...\n");
	fprintf(logfp,"\n"
		"TCP-Port   : %d\n"
		"PBX-Port   : %s\n"
		"PBX-Speed  : %s\n"
		"Logfile    : %s\n"
		"Permissions: %s\n"
		"Debuglevel : %d (0x%04X)\n",
		tcpport, serport, prSpeed(serspeed), logfile, permfile, debug, debug);
	fflush(logfp);

	clptr = srv_init_proc(tcpport);
	clptr->login = time(NULL);

	cl = pbx_init_proc(serport, serspeed);
	Cl_add(cl);
	cl->login = time(NULL);

	while (!quit)
		common_mainloop();

	logdate(LOG_MSG);
	fprintf(logfp,"\tExiting...\n");

	pbx_close();
	srv_close();
}

