/* ******************************************************************
 * lcrcfg.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: lcrcfg.cpp,v 1.1 2000/01/25 19:53:44 root Exp root $
 *
 * ******************************************************************/

#include <fstream>
#include <iostream>
#include <strstream>
#include <iomanip>
#include <string>
#include <vector>
#include <PNumber.h>
#include "lcrcfg.h"

static struct provider_dat {
	LONG	nr;
	char*	nm;
} provlist[] = {
	{1011,	"o.tel.o"},
	{1015,	"debitel CbC"},
	{1019,	"MobilCom"},
	{1024,	"TelePassport"},
	{1030,	"TelDaFax"},
	{1033,	"Deutsche Telekom"},
	{1039,	"First Telecom"},
	{1040,	"Esprit"},
	{1050,	"Talkline"},
	{1051,	"01051 Telecom"},
	{1066,	"Interroute"},
	{1070,	"Arcor"},
	{1075,	"Call Callino"},
	{1078,	"3U"},
	{1079,	"Viatel"},
	{1085,	"GTS-WESTCom"},
	{1090,	"Viag Interkom"},
	{1013,	"Tele 2"},
	{1015,	"RSL COM"},
	{1018,	"debitel Preselection"},
	{1024,	"SUPER 24"},
	{1028,	"COLT: TeleBridge/UPX"},
	{1046,	"KomTel"},
	{1049,	"ACC"},
	{1053,	"WOBCOM"},
	{1088,	"UUnet"},
	{1098,	"Mox"},
	{1099,	"Econophone"},
	{1014,	"EWE TEL"},
	{1020,	"ISIS"},
	{1022,	"NetCologne"},
	{1023,	"tesion"},
	{1025,	"CityKom"},
	{1027,	"Nordcom"},
	{1041,	"HanseNet"},
	{1042,	"TeleBeL"},
	{1044,	"VEW TELNET"},
	{1048,	"DOKOM"},
	{10050,	"Drillisch Offener CbC"},
	{10100,	"Meocom"}
};

#define PLSIZE sizeof(provlist)/sizeof(struct provider_dat)
static const char* const unkn_prov = "Unbekannt";

const char* LcrConf::getProviderName(const string& num) {
	LONG x = atoi(num.c_str());
	for (size_t i=0; i < PLSIZE; i++) {
		if (provlist[i].nr == x)
			return provlist[i].nm;
	}
	return unkn_prov;
}

/************************************
 *
 * Provider
 *
 ************************************/
provider::provider(const provider& p) {
	firma	= p.firma;
	name	= p.name;
	comment = p.comment;
	provnum	= new BaseNumber(*p.provnum);
}

provider::provider(const string& n, const string& c, const char* const p, const int idx, const int f = 0) {
	firma	= (BYTE)(f & 0x01);
	name	= n;
	comment	= c;
	provnum	= new BaseNumber(p);
}

provider::~provider() {
	delete provnum;
}

const vector<BYTE>& provider::getData() const {
	return provnum->NumberData();
}
ostream& operator<< ( ostream& strm, const provider& p) {
	strm << "Firma: " << (int)p.firma << endl;
	strm << "Name:  " << p.name << endl;
	strm << "Bem.:  " << p.comment << endl;
	strm << *p.provnum << endl;
	return strm;
}
		
/************************************
 *
 * Provider List
 *
 ************************************/

provider_list::provider_list() {
}

provider_list::~provider_list() {
}

void provider_list::add(const provider& p) {
	provis[p.firma].push_back(p);
}

void provider_list::add(const string& n, const string& c, const char* const p, const int idx, const int f = 0) {
	provider pr(n, c, p, idx, f);
	provis[pr.firma].push_back(pr);
}

size_t provider_list::size(const BYTE firma) {
	return provis[firma].size();
}

const vector<BYTE>& provider_list::getData(const BYTE firma,
					   const size_t idx) const {
	
        static vector<BYTE>	data;
        if (!data.empty())
		data.clear();

	data.push_back(PNumber::PROVIDER);
	data.push_back(firma);
	data.push_back(0); // Index 
	data.push_back(idx); // Index 

	vector<const provider>::iterator cur = provis[firma].begin();
	cur += idx;
	
	int i = 0;
	while(cur != provis[firma].end()) {
		const vector<BYTE> x = (*cur).getData();
		data.push_back((BYTE)x.size());
		copy(x.begin(), x.end(), back_inserter(data));
		if (++i == 10) break;
		cur++;
	}
	return data;
}

ostream& operator<< ( ostream& strm, const provider_list& p) {
	for (int i = 0; i < 2; i++) {
		vector<provider>::const_iterator cur = p.provis[i].begin();
		while (cur != p.provis[i].end()) 
			strm << (*cur++) << endl;
	}

	return strm;
}

/************************************
 *
 * Routing
 *
 ************************************/
routing::routing(const routing& r) {
	carrier		= r.carrier;
	minute		= r.minute;
	hour		= r.hour;
	wotag		= r.wotag;
	fbc1		= r.fbc1;
	fbc2		= r.fbc2;
};

routing::routing(const BYTE c, const WORD m, const WORD h, const BYTE w, const BYTE f1, const BYTE f2) {
	carrier		= c;
	minute		= bin2bcd(m);
	hour		= bin2bcd(h);
	wotag		= w;
	fbc1		= f1;
	fbc2		= f2;
}

routing::routing(const vector<BYTE>& buf, const size_t offs) {
	wotag		= buf[offs + 1];
	hour		= buf[offs + 2];
	minute		= buf[offs + 3];
	carrier		= buf[offs + 4];
	fbc1		= buf[offs + 5];
	fbc2		= buf[offs + 6];
}

routing::routing(const BYTE* const buf) {
	carrier		= buf[0];
	minute		= bin2bcd((WORD)(buf[2] + (buf[3] * 256)));
	hour		= bin2bcd((WORD)(buf[4] + (buf[5] * 256)));
	wotag		= buf[6];
	fbc1		= buf[8];
	fbc2		= buf[9];
}

routing::~routing() {
}

BYTE routing::bin2bcd(WORD b) {
	BYTE tens = 0;
	BYTE ones = (b & 0x00FF);

	if (b > 99) return b;

	if (b > 10)	
		tens = b / 10;
	if (tens) 
		ones = b - (10 * tens);

	tens = (tens << 4) + ones;
		
	return (tens);
}

const vector<BYTE>& routing::getData(const BYTE zone) const {
        static vector<BYTE>	data;
	if (!data.empty())
		data.clear();
	data.push_back(zone);
	data.push_back(wotag);
	data.push_back(hour);
	data.push_back(minute);
	data.push_back(carrier);
	data.push_back(fbc1);
	data.push_back(fbc2);
	return data;
}
		
/*
ostrstream& operator<< ( ostrstream& strm, const routing& r) {
	strm << r.wotag;
	strm << r.hour;
	strm << r.minute;
	strm << r.carrier;
	strm << 0xFF << 0xFF;
	return(strm);
}
*/

ostream& operator<< ( ostream& strm, const routing& r) {
	strm << setfill('0');
	strm << "Carrier: " ;
	if (r.carrier == 0xFF)
		strm <<  "AUS" ;
	else
		strm << (int)r.carrier ;
	strm << "\tZeit:\t" ;
	if (r.hour == 0xFF)
		strm <<  "--" ;
	else
		strm << hex << setw(2) << (int)r.hour << dec;
	strm << ":";
	if (r.minute == 0xFF)
		strm <<  "--" ;
	else
		strm << hex << setw(2) << (int)r.minute << dec;
	strm << "\tTage:\t";
	if (r.wotag & 0x01) strm << "W" ; else strm << "-";
	if (r.wotag & 0x02) strm << "s" ; else strm << "-";
	if (r.wotag & 0x04) strm << "S" ; else strm << "-";
	if (r.wotag & 0x08) strm << "F" ; else strm << "-";

	strm << "\tFallback:\t" << hex << (int)r.fbc1 << "\t" << (int)r.fbc2;
	strm << dec << setfill(' ');
	return strm;
}

/************************************
 *
 * Routing Zones
 *
 ************************************/

routing_zone::routing_zone(const routing_zone& rz) {
	zonenum = rz.zonenum;
	routes = rz.routes;
}

routing_zone::routing_zone(const int z) {
	zonenum = z;
}

routing_zone::routing_zone(const vector<BYTE>& buf,const size_t offs) {
	vector<BYTE>::const_iterator ptr = buf.begin();
	ptr += offs;
	zonenum = (*ptr++);
	while (ptr < buf.end()) {
		routing temp(buf, ptr - buf.begin());
		routes.push_back(temp);
		ptr += 7;	
	}
}

routing_zone::routing_zone(const BYTE* buf) {
	const BYTE* ptr = buf;
	zonenum = *ptr++;
	for(int i = 0; i < 15; i++) {
		routing temp(ptr);
		routes.push_back(temp);
		ptr += 10;	
	}
}

routing_zone::~routing_zone() {
}

const vector<BYTE>& routing_zone::getData(const BYTE zone) const {
        static vector<BYTE>	data;
	if (!data.empty())
		data.clear();

	data.push_back(zone);

	vector<const routing>::iterator cur = routes.begin();

	BYTE i=0;
	while (cur != routes.end()) {
		const vector<BYTE> x = (*cur++).getData(i);
		copy(x.begin(), x.end(), back_inserter(data));
		i++;
	}

	return data;
}

ostrstream& operator<< ( ostrstream& strm, const routing_zone& z) {
	vector<routing>::const_iterator cur = z.routes.begin();
	BYTE i=0;
	while (cur != z.routes.end()) {
		strm << i;
		strm << (*cur);
		i++;
	}
	return strm;
}

ostream& operator<< ( ostream& strm, const routing_zone& z) {
	strm << "Zone:\t" << z.zonenum << endl;
	vector<routing>::const_iterator cur = z.routes.begin();
	while (cur != z.routes.end()) 
		strm << (*cur++) << endl;
	return strm;
}

/************************************
 *
 * Routing List
 *
 ************************************/

void routing_list::add(const vector<BYTE>& buf) {
	vector<BYTE>::const_iterator i = buf.begin();
	i++;	// skip firma
	routing_zone rz(buf, i - buf.begin());
	routezone.push_back(rz);
}

routing_list::routing_list() {
}

routing_list::~routing_list() {
}

const vector<BYTE>& routing_list::getData(const size_t zone) const {
        static vector<BYTE>	data;
	if (!data.empty())
		data.clear();

	data.push_back(firma);

	vector<const routing_zone>::iterator cur = routezone.begin();
	cur += zone;

	const vector<BYTE> x = (*cur).getData((BYTE)(zone & 0x00FF));
	copy(x.begin(), x.end(), back_inserter(data));

	return data;
}

ostream& operator<< ( ostream& strm, const routing_list& z) {
	vector<routing_zone>::const_iterator cur = z.routezone.begin();
	while (cur != z.routezone.end()) 
		strm << (*cur++) << endl;
	return strm;
}

/************************************
 *
 * Zones
 *
 ************************************/
zone::zone(const zone& z) {
	name	= z.name;
	comment	= z.comment;
}

zone::zone(const char* const n , const char* const c) {
	name	= n;
	comment	= c;
}

zone::zone(const string& n, const string& c) {
	name	= n;
	comment	= c;
}

zone::~zone() { };

ostream& operator<< ( ostream& strm, const zone& z) {
	strm << "Zonenname:\t\"" << z.name << "\"\t";
	strm << "Bem.:\t\"" << z.comment << "\"";
	return strm;
}

/************************************
 *
 * Sperr-Zonen
 *
 ************************************/
sperre::sperre(const sperre& s) {
	number	= s.number;
	comment	= s.comment;
}

sperre::sperre(const string& n, const string& c) {
	number	= BaseNumber(n);
	comment	= c;
}

sperre::~sperre() {
}

const vector<BYTE>& sperre::getData() const {
	return number.data();
}

ostream& operator<< ( ostream& strm, const sperre& s) {
	strm << "Sperrnummer:\t" << s.number.Phone() << "\t";
	strm << "Bem.:\t\"" << s.comment << "\"";
	return strm;
}

/************************************
 *
 * Sperrzonen List
 *
 ************************************/

sperr_list::sperr_list() {
}

sperr_list::~sperr_list() {
}

void sperr_list::add(const string& n, const string& c, const int f = 0) {
	sperre sp(n, c);
	sperren[f].push_back(sp);
}

size_t sperr_list::size(BYTE firma) {
	return sperren[firma].size();
}

const vector<BYTE>& sperr_list::getData( const BYTE firma,
					 const size_t idx ) const
{
        static vector<BYTE>	data;
	if (!data.empty())
		data.clear();

	data.push_back(PNumber::LCRSPERRE);
	data.push_back(firma);
	data.push_back(0); // Index 
	data.push_back(idx); // Index 

	vector<const sperre>::iterator cur = sperren[firma].begin();
	cur += idx;
	
	while(cur != sperren[firma].end()) {
		const vector<BYTE> x = (*cur).getData();
		copy(x.begin(), x.end(), back_inserter(data));
		cur++;
	}
	return data;
}

ostream& operator<< ( ostream& strm, const sperr_list& s) {
	for (int i = 0; i < 2; i++) {
		vector<sperre>::const_iterator cur = s.sperren[i].begin();
		while (cur != s.sperren[i].end()) 
			strm << (*cur++) << endl;
	}

	return strm;
}



/************************************
 *
 * LCR Gesamtkonfig
 *
 ************************************/

LcrConf::LcrConf(const int dbg) {
	ident		= "";
	debug		= dbg;
	version		= 0;
	ftrenn		= 0;
	lcr_aktiv	= 0;
	lcr_easy	= 0;
	routes[0].firma	= 0;
	routes[1].firma	= 1;
}

LcrConf::~LcrConf() {
}

ostream& operator<< (ostream& strm, const LcrConf& cfg) {
	return strm;
}

void LcrConf::printConf(ostream& out) {
	out << "LCR-Datei-Version: " << version << endl;

	out << "Die Firmentrennung ist ";
	if (ftrenn)
		out << "ein" ;
	else
		out << "aus" ;
	out << "geschaltet!" << endl;
	
	out << "Der LCR ist ";
	if (lcr_aktiv && !lcr_easy)
		out << "voll aktiviert." ;
	else if (lcr_aktiv && lcr_easy)
		out << "im Easy-Modus." ;
	else
		out << "ausgeschaltet!" ;
	out << endl << endl;

	out << "Vorwahlen: " << vorwahlen.size() << endl;
	if (debug > 1) {
		vector<ONumber>::iterator cur = vorwahlen.begin();
		while (cur != vorwahlen.end()) 
			out << (*cur++) << endl << endl;
	}

	out << endl;
	out << "Anzahl Sperrnummern (Firma 1): " << sperrliste.sperren[0].size() << endl;
	out << "Anzahl Sperrnummern (Firma 2): " << sperrliste.sperren[1].size() << endl;
	if (debug > 1 ) { 
		out << sperrliste << endl;
	}

	out << "Anzahl Provider (Firma 1): " << provis.size(0) << endl;
	out << "Anzahl Provider (Firma 2): " << provis.size(1) << endl;
	if (debug > 1) {
		out << endl;
		out << "Providerinfos:" << endl;
		out << provis;
	}

	out << "Anzahl Zonennamen: " << zones.size() << endl;
	if (debug > 1) {
		vector<zone>::iterator cur = zones.begin();
		while (cur != zones.end()) 
			out << (*cur++) << endl;
	}

	out << endl;
	out << "Anzahl Routen (Firma 1): " << routes[0].routezone.size() << endl;
	out << "Anzahl Routen (Firma 2): " << routes[1].routezone.size() << endl;
	out << endl;
	if (debug > 1) {
		out << "Firma1 Routen: " << routes[0] << endl;
		out << "Firma2 Routen: " << routes[1] << endl;
	}

}

int LcrConf::writeConf(const string& filename) {
	ofstream  cfgfile;
	cfgfile.open(filename.c_str(), ios::out | ios::bin);
	if (!cfgfile.is_open()) {
		cerr << "Die Datei \"" << filename << "\" konnte nicht geoeffnet werden!";
		return -1;
	}

	writestring(cfgfile, string("aalcr"));		// Kennung
	writelong(cfgfile, 2);						// Version
	writelong(cfgfile, ftrenn);					// Firmentrennung
	writelong(cfgfile, lcr_easy);				// Easy Konfig
	writelong(cfgfile, lcr_aktiv);				// LCR-Schalter

	BYTE f;

	//provider
	writelong(cfgfile, (provis.provis[0].size() + provis.provis[1].size()));	//Anzahl Provider
	for(f = 0; f < 2; f++) {
		vector<provider>::iterator cur = provis.provis[f].begin();
		while (cur < provis.provis[f].end()) {
			writestring(cfgfile, (*cur).name);									//Providername
			writestring(cfgfile, (*cur).provnum->Phone());						//Providernummer
			writestring(cfgfile, (*cur).comment);								//Kommentar
			cur++;
		}
	}

	//Vorwahlnummern
	{
		writelong(cfgfile,vorwahlen.size());
		vector<ONumber>::iterator cur = vorwahlen.begin();
		while (cur < vorwahlen.end()) {
			writeword(cfgfile, (WORD)(*cur).getZone());
			writestring(cfgfile, (*cur).Phone());
			cur++;
		}
	}

	// Routingtable
	for(f = 0; f < 2; f++) {
		vector<routing_zone>::iterator rz = routes[f].routezone.begin();
		while (rz < routes[f].routezone.end()) {
			writelong(cfgfile, (*rz).routes.size());
			vector<routing>::iterator cur = (*rz).routes.begin();
			while (cur < (*rz).routes.end()) {
				writeword(cfgfile, (WORD)(*cur).carrier);
				if ((*cur).minute == 0xFF)
					writeword(cfgfile, 0xFFFF);
				else
					writeword(cfgfile, (WORD)((((*cur).minute >> 4) * 10) + ((*cur).minute & 0x0F)));
				if ((*cur).hour == 0xFF)
					writeword(cfgfile, 0xFFFF);
				else
					writeword(cfgfile, (WORD)((((*cur).hour   >> 4) * 10) + ((*cur).hour   & 0x0F)));
				writeword(cfgfile, (WORD)(*cur).wotag);
				writeword(cfgfile, 0xFFFF);
				cur++;
			}
			rz++;
		}
	}

	//sperrliste
	writelong(cfgfile, (sperrliste.sperren[0].size() + sperrliste.sperren[1].size()));
	for(f = 0; f < 2; f++) {
		vector<sperre>::iterator cur = sperrliste.sperren[f].begin();
		while (cur < sperrliste.sperren[f].end()) {
			writestring(cfgfile, (*cur).number.Phone());
			writestring(cfgfile, (*cur).comment);
			cur++;
		}
	}

	//zonennamen
	{
		writelong(cfgfile, zones.size());
		vector<zone>::iterator cur = zones.begin();
		while (cur < zones.end()) {
			writestring(cfgfile, (*cur).name);
			writestring(cfgfile, (*cur).comment);
			cur++;
		}
	}

	cfgfile.close();
	return -1;
}

int LcrConf::readConf(const string& filename) {
	ifstream  cfgfile;
	cfgfile.open(filename.c_str(), ios::nocreate | ios::in | ios::bin);
	if (!cfgfile.is_open()) {
		cerr << "Die Datei \"" << filename << "\" konnte nicht geoeffnet werden!";
		return -1;
	}

	ident	= readstring(cfgfile);

	if (ident != "aalcr") {
		cerr << "Keine Ackermann LCR-Datei!" << endl;
		return -1;
	}

	version = readlong(cfgfile);
	ftrenn	= readlong(cfgfile);

	if (version == 2) {
		lcr_easy	= readlong(cfgfile);
		lcr_aktiv	= readlong(cfgfile);
	}

	LONG anzahl = readlong(cfgfile);
	int firm = 0;
	while (anzahl--) {
		string name		= readstring(cfgfile);
		string nummer	= readstring(cfgfile);
		string comment	= readstring(cfgfile);
		if (ftrenn && (anzahl <= 10))
			firm = 1;
		provis.add(name, comment, nummer.c_str(), provis.size(firm), firm);
	}
	
	anzahl = readlong(cfgfile);
	while (anzahl--) {
		WORD	zone	= readword(cfgfile);
		string	nummer	= readstring(cfgfile);;
		if (nummer.size() > 6) {
			cerr << "Die Vorwahl \"" << nummer << "\" ist zu lang (mehr als 6 Ziffern) und wird ignoriert!" << endl;
			continue;
		}
		ONumber v(nummer, zone);
		vorwahlen.push_back(v);
	}

	int z = 20;
	while (z--) {
		anzahl = readlong(cfgfile);
		routing_zone rzone((19 - z));
		while (anzahl--) {
			BYTE buf[10];
			cfgfile.read(&buf,10);
			routing r(buf);
			rzone.routes.push_back(r);
		}
		routes[0].routezone.push_back(rzone);
	}

	z = 20;
	while (z--) {
		anzahl = readlong(cfgfile);
		routing_zone rzone((19 - z));
		while (anzahl--) {
			BYTE buf[10];
			cfgfile.read(&buf,10);
			routing r(buf);
			rzone.routes.push_back(r);
		}
		routes[1].routezone.push_back(rzone);
	}

	anzahl = readlong(cfgfile);
	firm = 0;
	while (anzahl--) {
		string	nummer	= readstring(cfgfile);;
		string	comment	= readstring(cfgfile);;
		if (ftrenn && (anzahl <= 10))
			firm = 1;
		sperrliste.add(nummer,comment, firm);
	}

	anzahl = readlong(cfgfile);
	while (anzahl--) {
		string	name	= readstring(cfgfile);;
		string	comment	= readstring(cfgfile);;
		zone z(name, comment);
		zones.push_back(z);
	}

	cfgfile.close();

	return 0;
}

void LcrConf::writebyte(ofstream& cfgfile, const BYTE c) 
{
	cfgfile << c;
	return;
}

void LcrConf::writeword(ofstream& cfgfile, const WORD c) 
{
	BYTE a = c & 0x00FF;
	BYTE b = c >> 8;
	writebyte(cfgfile,a);
	writebyte(cfgfile,b);
	return;
}

void LcrConf::writelong(ofstream& cfgfile, const LONG c) 
{
	WORD a = c & 0x00FFFF;
	WORD b = c >> 16;
	writeword(cfgfile,a);
	writeword(cfgfile,b);
	return;
}

void LcrConf::writestring(ofstream& cfgfile, const string& s)
{
	writebyte(cfgfile, (BYTE)s.size());
	copy(s.begin(), s.end(), ostream_iterator<BYTE>(cfgfile));
	return;
}

BYTE LcrConf::readbyte(ifstream& cfgfile) 
{
	BYTE ret;
	cfgfile.read(&ret,sizeof(char));
	return ret;
}

WORD LcrConf::readword(ifstream& cfgfile) 
{
	return (readbyte(cfgfile) + (256 * readbyte(cfgfile)));
}

LONG LcrConf::readlong(ifstream& cfgfile) 
{
	return (readword(cfgfile) + (65536 * readword(cfgfile)));
}

const char* const LcrConf::readstring(ifstream& cfgfile) 
{
	static char buf[256];
	char *pb = buf;

	BYTE len = readbyte(cfgfile);

	while (len--)
		*pb++ = readbyte(cfgfile);
		
	*pb++ = 0;
	*pb++ = 0;
	return buf;
}

