/********************************************************************
 * lib/maKs.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: maKs.h,v 1.1 1999/12/30 21:04:17 root Exp root $
 *
 ********************************************************************/
 
#ifndef __MAXIMAKS_H__
#define __MAXIMAKS_H__

#define BUFFSIZE	512

#define BYTE	unsigned char
#define WORD	unsigned short
#define LONG	unsigned long

struct packet {
    BYTE   folgenr;
    BYTE   anlage;
    BYTE   gruppe;
    BYTE   element;
    BYTE   length;
    BYTE   data[1];
};

typedef struct packet EspMsg;

#define TRUE	(1==1)
#define FALSE	(1==0)
#define ASYNC	TRUE
#define SYNC	FALSE

#define	DBG_MESSAGES	0x01
#define	DBG_PACKETS		0x02
#define	DBG_VERIFYLID	0x04
#define	DBG_HANDSHAKE	0x08

#define	DODEBUG(x,y)	((x & y))

#define HNIBBLE_MASK	0xF0
#define LNIBBLE_MASK	0x0F
#define BYTE_MASK		0x00FF


#define	PERM_CTI		0x01
#define	PERM_KONFIG		0x02
#define	PERM_GEBUER		0x04
#define	PERM_FERNWART	0x08
#define	PERM_BMON		0x10
#define	PERM_S0MON		0x20
#define	PERM_LCR		0x40
#define	PERM_USER		0x80
#define	PERM_SCTRL		0x00

#define	AUTH_NONE		0x00
#define	AUTH_CLEARTXT	0x01
#define	AUTH_ENCRYPT	0x02
#define	AUTH_CHALLENGE	0x04
#define	AUTH_DENY		0xFF

#define K_CTI_GS_MASK	0x01
#define K_CTI_GET		0x00
#define K_CTI_SET		0x01
#define K_CTI_AD_MASK	0x02
#define K_CTI_ADD		0x00
#define K_CTI_DEL		0x02
#define K_CTI_OE_MASK	0x04
#define K_CTI_OK		0x00
#define K_CTI_ERR		0x04
#define K_CTI_CR_MASK	0x08
#define K_CTI_CMD		0x00
#define K_CTI_RSP		0x08

#define K_CTI_TYP_MASK	0xF0
#define K_CTI_FLG_MASK	0x0F

#define	S_CTI_RSP(x)	((x & K_CTI_CR_MASK) == K_CTI_RSP)
#define	S_CTI_ERR(x)	((x & K_CTI_OE_MASK) == K_CTI_ERR)
#define	S_CTI_DEL(x)	((x & K_CTI_AD_MASK) == K_CTI_DEL)
#define	S_CTI_SET(x)	((x & K_CTI_GS_MASK) == K_CTI_SET)

/* Eigene Pakettypen */
#define K_CTI_ESP					0x00 /* Client/Server Commands */

/* Ackermannsche Pakettypen */
#define UNKNOWN_0					0x00 /* Undifined by Ackermann */
#define K_CTI_ANL					0x10
#define K_CTI_POR					0x20
#define K_CTI_GEB					0x30
#define K_CTI_ORG					0x40
#define K_CTI_TLN					0x50
#define K_CTI_AUX					0x60
#define UNKNOWN_7					0x70
#define UNKNOWN_8					0x80
#define UNKNOWN_9					0x90
#define K_CTI_STA					0xA0
#define K_CTI_OFF					0xB0
#define K_CTI_VBG					0xC0
#define K_CTI_EVT					0xD0
#define K_CTI_TST					0xE0
#define K_CTI_FAB					0xF0

/* eigene Kommandos */
#define	ESP_CONN_REQ				0x00
#define	ESP_CONN_AUTH				0x01
#define	ESP_SRVR_MSG				0x02
#define	ESP_SRVR_STAT				0x03
#define	ESP_SRVR_KILL				0x04
#define	ESP_SRVR_WAKE				0x05

/* Ackermannsche Kommandos */
#define	ANL_LINK_ID_GET				0x00
#define	ANL_HERSTELLER				0x01
#define	ANL_TYP						0x02
#define	ANL_SW_VERSION				0x03
#define	ANL_DB_VERSION				0x04
#define	ANL_BETRIEBSART				0x05
#define	ANL_ALLE_INT_RNR			0x06
#define	ANL_DATUM_ZEIT				0x07
#define	ANL_KEYPAD					0x08
#define	ANL_MAX_PARA				0x0A
#define	ANL_FWZ_OLD					0x0B
#define	ANL_DEFAULT					0x0C
#define	ANL_PLUG_AND_GO				0x0D
#define	ANL_FERNSCHALTEN			0x0F
#define	ANL_X31_KFG					0x11
#define	ANL_TLN_INFO				0x12
#define	ANL_MASTER_SLAVE			0x13
#define	ANL_KFG_START				0x14
#define	ANL_KFG_ENDE				0x15
#define	ANL_KFG_DATA_START			0x16
#define	ANL_KFG_DATA_ENDE			0x17
#define	ANL_KFG_DATA_ERROR			0x18
#define	ANL_CRC_EPROM				0x19
#define	ANL_CRC_EEPROM				0x1A
#define	ANL_FEHLERPROTOKOLL			0x1B
#define	ANL_RESET					0x1C
#define	ANL_FW_KENNWORT				0x1D
#define	ANL_FW_ENDE					0x1E
#define	ANL_B_KANAELE				0x1F
#define	ANL_VERIFY_LINK_ID			0x20
#define	ANL_DB_Q_MSG				0x21
#define	ANL_DB_Q_ANL_INFO			0x22
#define	ANL_FW_VBG					0x23
#define	ANL_FW_KW_QUITTUNG			0x24
#define	ANL_FW_REL_1_X				0x25
#define	ANL_FW_REL_1_X_AKTIV		0x26
#define	ANL_SM_Q_MSG				0x27

#define POR_LAND					0x00
#define POR_ORT						0x01
#define POR_HAUPTANL				0x02
#define POR_FIRMA					0x03
#define POR_MSN_AVA_TLN				0x04
#define POR_MSN_AVA_PROFIL			0x05
#define POR_MSN_AVA_SCHALTZEIT		0x06
#define POR_MSN_AVA_RWS_PROFIL		0x07
#define POR_MSN_AVA_RWS_ZEIT		0x08
#define POR_BETRIEBSART				0x09
#define POR_AMT_RNR					0x0A
#define POR_INT_RNR					0x0B
#define POR_MODE					0x0C
#define POR_SER_DIENST				0x0D
#define POR_SER_RATE				0x0E
#define POR_DDI_LEN					0x0F
#define POR_DDI_RNR					0x10
#define POR_TFE_AVA_TLN				0x11
#define POR_TFE_AVA_PROFIL			0x12
#define POR_TFE_AVA_SCHALTZEIT		0x13
#define POR_TFE_RUF_DAUER			0x14
#define POR_TFE_UMLEITUNG			0x15
#define POR_DB_AVA_TLN				0x16
#define POR_NACHBAR_MSN_AVA			0x17
#define POR_NACHBAR_TFE_AVA			0x18

#define GEB_WAEHRUNG				0x00
#define GEB_FAKTOR					0x01
#define GEB_PTT_SATZ				0x02
#define GEB_TLN_SATZ				0x03
#define GEB_TLN_KONTO				0x04
#define GEB_ANL_SUMME				0x05
#define GEB_PRJ_SUMME				0x06
#define GEB_FA_SUMME				0x07
#define GEB_TLN_SUMME				0x08
#define GEB_TLN_STATUS				0x09
#define GEB_ANL_STATUS				0x0A
#define GEB_TLN_INFO				0x0B

#define ORG_LOGO_FA					0x02
#define ORG_TLN_FA					0x03
#define ORG_TLN_GRP					0x04
#define ORG_RNR_FA					0x05
#define ORG_RNR_STATUS				0x07
#define ORG_FA_CODE					0x08
#define ORG_FA_ANZAHL				0x09
#define ORG_FA_MOH					0x0A
#define ORG_LCR_ONKZ				0x0B
#define ORG_FA_LCR_PROFIL			0x0C
#define ORG_FA_LCR_SELECT			0x0D
	
#define TLN_REGISTER				0x00
#define TLN_UMLEITUNG				0x18
#define TLN_IDENT				0x19
#define TLN_HOTLINE_RNR				0x1A
#define TLN_RECHTE					0x1D
#define TLN_SONSTIGES				0x1E

#define AUX_ALARM_AKTIV				0x00
#define AUX_ALARM_ZEIT				0x01
#define AUX_ALARM_DELAY				0x02
#define AUX_ALARM_DAUER				0x03
#define AUX_ALARM_RNR				0x04
#define AUX_ALARM_WAHL				0x05
#define AUX_ALARM_UUS				0x06
#define AUX_AKTOR_FKT				0x08
#define AUX_AKTOR_ZUSTAND			0x09
#define AUX_AKTOR_MANUELL			0x0A
#define AUX_TLN_VIRTUELL			0x0B
#define AUX_PIN_VIRTUELL			0x0C

#define OFF_REQUEST_GEB				0x02
#define OFF_TUEROEFFNER				0x03
#define OFF_ANL_ANRUFLISTE			0x04
#define OFF_TLN_ANRUFLISTE			0x0B
#define OFF_TLN_RUFLISTE			0x0C
#define OFF_TLN_WAHLMODUS			0x0D
#define OFF_TLN_DETAILS				0x0E
#define OFF_ANL_GEB_STATUS			0x0F

#define STA_RESSOURCEN				0x00
#define STA_HOOK					0x01
#define STA_TLN						0x02
#define STA_TLN_LANG				0x03
#define STA_TLN_MITTEL				0x04
#define STA_TLN_KURZ				0x05
#define STA_TEST					0x06

#define VBG_HOOK_OFF				0x00
#define VBG_HOOK_ON					0x01
#define VBG_AMT_SETUP				0x02
#define VBG_TLN_SETUP				0x03
#define VBG_DISCONNECT				0x04
#define VBG_INTRUSION				0x05
#define VBG_OVERLAP_SEND			0x06
#define VBG_CALL_U_TRANSFER			0x07
#define VBG_CALL_I_TRANSFER			0x08
#define VBG_CALL_E_TRANSFER			0x09
#define VBG_3PTY					0x0A
#define VBG_CONF					0x0B
#define VBG_HOLD					0x0C
#define VBG_CONNECT					0x0D
#define VBG_TLN_WAHLVORBEREITUNG	0x0E
#define VBG_INT_CALL				0x20
#define VBG_EXT_CALL				0x21
#define VBG_CHARGING_AOCS_INF		0x22
#define VBG_CHARGING_INFO			0x23
#define VBG_DISP_HOOK_OFF_MSG		0x24
#define VBG_DISPLAY_STRING			0x25
#define VBG_DISPLAY_FMT				0x26
#define VBG_USER_TO_USER			0x27
#define VBG_ALERT					0x28
#define VBG_CONNECTED_NUMBER		0x29
#define VBG_READY					0x30
#define VBG_ANL_VERIFY_LINK_ID		0x30 /*Alias fuer VBG_READY*/
#define VBG_CALL_ROUTING_LOG		0x40
#define VBG_CALL_ROUTING			0x41

#define EVT_HOOK_STATUS				0x01


/* Definitionen und Structuren fuer Konfig */

#define MAX_NR_LEN					24

/************************************
 * Anlage
 ************************************/
typedef struct anl_cfg {
	BYTE				anl_betriebsart;
											/* struct anl_x31_kfg	nur prof. anwender */
} ANL_CFG_TYPE;

/************************************
 * Port
 ************************************/
#define MAX_POR_MSNS				10
#define MAX_POR_AVAS				5
#define MAX_POR_WOCHENTAG			7
#define	MAX_TLN						40

typedef	struct msn_ava_tln{
	BYTE				IntRnr[MAX_TLN];
} MSN_AVA_TLN_TYPE;

typedef	struct msn_ava_profil{
	BYTE				TagAvaIdx;
	BYTE				NachtAvaIdx;
} MSN_AVA_PROFIL_TYPE;

typedef	struct msn_ava_szeit{
	BYTE				TagStunden;
	BYTE				TagMinuten;
	BYTE				NachtStunden;
	BYTE				NachtMinuten;
} MSN_AVA_SZEIT_TYPE;

typedef struct por_cfg {
	BYTE				por_typ;
	BYTE				por_nr;
	BYTE				por_land[MAX_NR_LEN+1];
	BYTE				por_ort[MAX_NR_LEN+1];
	BYTE				por_hauptanl[MAX_NR_LEN+1];
	MSN_AVA_TLN_TYPE	por_msn_ava_tln[MAX_POR_MSNS][MAX_POR_AVAS];
	MSN_AVA_PROFIL_TYPE	por_msn_ava_profil[MAX_POR_MSNS][MAX_POR_WOCHENTAG];
	MSN_AVA_SZEIT_TYPE	por_msn_ava_szeit[MAX_POR_MSNS][MAX_POR_WOCHENTAG];
} POR_CFG_TYPE;

#endif __MAXIMAKS_H__
