/* ******************************************************************
 * MsgHandler.h
 * Part of maKs
 * Copyright Chris Benndorf und Roland Schulz, 1998, 1999, 2000
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: MsgHandler.h,v 1.9 2000/01/18 20:00:49 root Exp root $
 *
 * ******************************************************************/

#ifndef MSGHANDLER_H_
#define MSGHANDLER_H_

#define K_CTI_GS_MASK   0x03
#define K_CTI_GET       0x00
#define K_CTI_SET       0x01
#define K_CTI_ADD       0x02
#define K_CTI_DEL       0x03
#define K_CTI_OE_MASK   0x04
#define K_CTI_OK        0x00
#define K_CTI_ERR       0x04
#define K_CTI_CR_MASK   0x08
#define K_CTI_CMD       0x00
#define K_CTI_RSP       0x08

#define K_CTI_TYP_MASK  HNIBBLE_MASK
#define K_CTI_FLG_MASK  LNIBBLE_MASK
 
#define	MSG_HANDLED		true
#define	MSG_NOT_HANDLED	false

class MsgHandler {

public:
	// Contructors & Destructors
	MsgHandler(const char *, const char *, const char *, const BYTE, const BYTE);
	virtual ~MsgHandler(){};

	// Synch Handler
	const ESPMsgHdr& syncSend(BYTE grp, BYTE elem, const vector<BYTE> &buf = vector<BYTE>(), BYTE anlg = 0) ;
	const ESPMsgHdr& syncSend(BYTE grp, BYTE elem, BYTE data, BYTE anlg = 0) {return syncSend(grp, elem, vector<BYTE>(1,data), anlg); }

	// Async Handler
	void asyncSend(BYTE grp, BYTE elem, const vector<BYTE> &buf = vector<BYTE>(), BYTE anlg = 0);
	void asyncSend(BYTE grp, BYTE elem, BYTE data, BYTE anlg = 0)	{ asyncSend(grp, elem, vector<BYTE>(1,data), anlg); }

	// Debug Ausgabe
	void				fake_asyncSend(BYTE grp, BYTE elem, const vector<BYTE> &buf = vector<BYTE>(), BYTE anlg = 0);
	const ESPMsgHdr&	fake_syncSend(BYTE grp, BYTE elem, const vector<BYTE> &buf = vector<BYTE>(), BYTE anlg = 0);

	// Packet Handler
	virtual void msg_Emitter(ESPMsgHdr *)=0;

	// Standart Packets 
	bool esp_conn_req() ;
	bool esp_conn_auth() ;
	bool config_start(bool);	// Rein synchron!
	bool config_end(bool);		// Rein synchron!

	void Handle();

	virtual void sendrdyMsg(ESPMsgHdr *)=0;
	virtual void sendfailMsg(char* msg, int fatal=false)=0;

	// Tools
	const char*		failReason()		{ return _fail; }
	int				failFatal()			{ return _fatal==1; }
	virtual int		socket()=0;
	BYTE			fnum();
	void			switchDebug(bool);
	bool			isConnected()		{ return connected; }

protected:
	virtual void	processEvents()=0;

	bool		connected;
	bool		warte;
	int			retry;
	ESPMsgHdr	wp;
	bool		debug;
	BYTE 		_flgnr;
	BYTE 		_auth;
	BYTE 		_perm;
	BYTE 		_resv;
	char 		_app[32];
	char 		_usr[32];
	char		_psw[32];
	char		*_fail;		// Grund des Fehlers
	int			_fatal;		// 1 = Programm wird beendet, 0 = wird nicht beendet

public:

enum {
	severInfo	= 0x00,
	severWarning	= 0x01,
	severCritical	= 0x02,
};

enum {
	PERM_SCTRL      = 0x00,
	PERM_CTI        = 0x01,
	PERM_KONFIG     = 0x02,
	PERM_GEBUER     = 0x04,
	PERM_FERNWART   = 0x08,
	PERM_BMON       = 0x10,
	PERM_S0MON      = 0x20,
	PERM_LCR        = 0x40,
	PERM_USER       = 0x80
};

enum {
	AUTH_NONE		= 0x00,
	AUTH_CLEARTXT	= 0x01,
	AUTH_ENCRYPT	= 0x02,
	AUTH_CHALLENGE	= 0x04,
	AUTH_DENY		= 0xFF
};

enum {
	G_ESP =	0x00,	/* Euracom Server Protokoll */
	G_ANL =	0x10,	/* Anlagendaten */
	G_POR =	0x20,	/* Portkonfiguration */
	G_GEB =	0x30,	/* Gebhrenmanagement */
	G_ORG =	0x40,	/* Organisation */
	G_TLN =	0x50,	/* Teilnehmereinstellungen */
	G_AUX =	0x60,	/* Sonderfkt (Alarm...) */
	G_STA =	0xA0,	/* CTI-Status (Hook Infos, Zustnde, ....) */
	G_OFF =	0xB0,	/* CTI-Offline-Stg */
	G_VBG =	0xC0,	/* CTI-Verbindungsstg. (Auf- u. Abbau) */
	G_EVT =	0xD0,	/* CTI-Event */
	G_TST =	0xE0,	/* Testmodi (LCB Monitoring) */
	G_FAB =	0xF0	/* Fabriktest */
};

/*
 * Elementkennungen - Euracom Server Protokoll
 */
enum {
	ESP_CONN_REQ=0x00,
	ESP_CONN_AUTH,
	ESP_SRVR_MSG,
	ESP_SRVR_STAT,
	ESP_SRVR_KILL,
	ESP_SRVR_WAKE
};

/*
 * Elementkennungen - Anlagenbasisdaten
 */
enum {
	ANL_LINK_ID=0x00,
	ANL_HERSTELLER,			
	ANL_TYP,				
	ANL_SW_VERSION,
	ANL_DB_VERSION,			/* unbenutzt */
	ANL_BETRIEBSART,
	ANL_ALLE_INT_RNR,		/* unbenutzt */
	ANL_DATUM_ZEIT,			/* unbenutzt */
	ANL_KEYPAD,				/* unbenutzt */
	ANL_UNKNOWN1,
	ANL_MAX_PARA,
	ANL_VWZ_OLD,			/* unbenutzt */
	ANL_DEFAULT,
	ANL_PLUG_AND_GO,		/* unbenutzt */
	ANL_UNKNOWN2,
	ANL_FERNSCHALTEN,
	ANL_UNKNOWN3,
	ANL_X31_KFG,
	ANL_TLN_INFO,
	ANL_MASTER_SLAVE,
	ANL_KFG_START,
	ANL_KFG_ENDE,
	ANL_KFG_DATA_START,
	ANL_KFG_DATA_ENDE,
	ANL_KFG_DATA_ERROR,
	ANL_CRC_EPROM,
	ANL_CRC_EEPROM,
	ANL_FEHLERPROTOKOLL,
	ANL_RESET,				/* unbenutzt */
	ANL_FW_KENNWORT,
	ANL_FW_ENDE,
	ANL_B_KANAELE,			/* unbenutzt */
	ANL_VERIFY_LINK_ID,
	ANL_DB_Q_MSG,
	ANL_DB_Q_ANL_INFO,
	ANL_FW_VBG,
	ANL_FW_KW_QUITTUNG,
	ANL_FW_REL_1_X,
	ANL_FW_REL_1_X_AKTIV,
	ANL_SM_Q_MSG
};

/*
 * Elementekennungen - Portkonfiguration
 */
enum {
	POR_LAND,
	POR_ORT,
	POR_HAUPTANL,
	POR_FIRMA,					/* unbenutzt */
	//POR_MSN_AVA				/* unbenutzt */
	POR_MSN_AVA_TLN,
	POR_MSN_AVA_PROFIL,
	POR_MSN_AVA_SCHALTZEIT,
	POR_MSN_AVA_RWS_PROFIL,
	POR_AVA_RWS_ZEIT,
	POR_BETRIEBSART,			/* unbenutzt */
	POR_AMT_RNR,
	POR_INT_RNR,
	POR_MODE,
	POR_SER_DIENST,
	POR_SER_RATE,
	POR_DDI_LEN,
	POR_DDI_RNR,
	POR_TFE_AVA_TLN,
	POR_TFE_AVA_PROFIL,
	POR_TFE_AVA_SCHALTZEIT,
	POR_TFE_RUF_DAUER,
	POR_TFE_UMLEITUNG,
	POR_DB_AVA_TLN,
	POR_NACHBAR_MSN_AVA,
	POR_NACHBAR_TFE_AVA
};

/*
 * Elementekennungen - Gebhrenverwaltung
 */
enum {
	GEB_WAEHRUNG,			/* unbenutzt */
	GEB_FAKTOR,
	GEB_PTT_SATZ,
	GEB_TLN_SATZ,
	GEB_TLN_KONTO,
	GEB_ANL_SUMME,
	GEB_PRJ_SUMME,
	GEB_FA_SUMME,
	GEB_TLN_SUMME,
	GEB_TLN_STATUS,
	GEB_ANL_STATUS,
	GEB_TLN_INFO
};

/*
 * Elementekennungen - Organisation
 */
enum {
	ORG_UNKNOWN1,
	ORG_UNKNOWN2,
	ORG_LOGO_FA,
	ORG_TLN_FA,
	ORG_TLN_GRP,			/* unbenutzt */
	ORG_RNR_FA,
	ORG_RNR_GRP,			/* unbenutzt */
	ORG_FA_STATUS,
	ORG_FA_CODE,
	ORG_FA_ANZAHL,
	ORG_FA_MOH,
	ORG_LCR_ONKZ,
	ORG_FA_LCR_PROFIL,
	ORG_FA_LCR_SELECT			/* auch ORG_FA_LCR_MODE */
};

/*
 * Elementekennungen - Teilnehmereinstellungen
 */
enum {
	TLN_REGISTER,
	TLN_SCHALTSTATUS,
	TLN_SCHALTRECHT,			/* unbenutzt */
	TLN_UNKNOWN1,
	TLN_UNKNOWN2,
	TLN_EINRICHT_RECHT,			/* unbenutzt */
	TLN_AMTSRECHT,				/* unbenutzt */
	TLN_MSN,
	TLN_TERMIN,					/* unbenutzt */
	TLN_WECKZEIT,				/* unbenutzt */
	TLN_DIENST,					/* unbenutzt */
	TLN_FUNKTION,				/* unbenutzt */
	TLN_RNR,					/* unbenutzt */
	TLN_PIN,					/* nicht implementiert */
	TLN_SCHLOSS,				/* nicht implementiert */
	TLN_KONTO,					/* unbenutzt */
	TLN_DISPLAY,				/* nicht implementiert */
	TLN_TONRUF,					/* nicht implementiert, */
	TLN_LAUTSPRECHER,			/* nicht implementiert, */
	TLN_VIP_FILTER,				/* nicht implementiert */
	TLN_RUF_DELAY,				/* nicht implementiert */
	TLN_FILTER,					/* nicht implementiert */
	TLN_TASTATUR,				/* nicht implementiert */
	TLN_MAIL,					/* nicht implementiert */
	TLN_UMLEITUNG,
	TLN_IDENT,
	TLN_HOTLINE,
	TLN_TEST_AB_RUFTON,			/* unbenutzt */
	TLN_TEST_AB_HOERTON,		/* unbenutzt */
	TLN_RECHTE,
	TLN_SONSTIGES
};

/*
 * Elementekennungen - Sonderfunktionen
 */
enum {
	AUX_SENSOR_FKT,
	AUX_ALARM_ZEIT,
	AUX_ALARM_DELAY,
	AUX_ALARM_DAUER,
	AUX_ALARM_RNR,
	AUX_ALARM_WAHL,
	AUX_ALARM_UUS,
	AUX_ALARM_STATUS,			/* nicht implementiert */
	AUX_AKTOR_FKT,
	AUX_AKTOR_ZUSTAND,
	AUX_AKTOR_MANUELL,
	AUX_TLN_VIRTUELL,
	AUX_PIN_VIRTUELL
};

/*
 * Elementekennungen - CTI STatus
 */
enum {
	STA_RESSOURCEN = 0,
	STA_HOOK,
	STA_TLN,					/* unbenutzt */
	STA_TLN_LANG,
	STA_TLN_MITTEL,
	STA_TLN_KURZ,
	STA_TEST
};

/*
 * Elementekennungen - CTI Offline Steuerung
 */
enum {
	OFF_STOP_SENSOR_PROZ = 0,	/* nicht implementiert */
	OFF_NEUE_SEITE,				/* nicht implementiert */
	OFF_POR_UMLEITUNG,			/* nicht implementiert */
	OFF_TUER_OFFNER,			/* nicht implementiert */
	OFF_ANL_ANRUFLISTE,
	OFF_ANL_RUFLISTE,
	OFF_TLN_UMLEITUNG,
	OFF_TLN_DISPLAY,			/* nicht implementiert */
	OFF_TLN_MIKROFON,			/* nicht implementiert */
	OFF_TLN_NOTIZ,				/* nicht implementiert */
	OFF_TLN_WWH,				/* nicht implementiert */
	OFF_TLN_ANRUFLISTE,
	OFF_TLN_RUFLISTE,			/* nicht implementiert */
	OFF_TLN_WAHLMODUS,			/* nicht implementiert */
	OFF_TLN_DETAILS				/* nicht implementiert */
};

/*
 * Elementekennungen - CTI Verbindungssteuerung
 */
enum {
	VBG_HOOK_OFF	=0x00,
	VBG_HOOK_ON,				/* unbenutzt */
	VBG_AMT_SETUP,
	VBG_TLN_SETUP,
	VBG_DISCONNECT,
	VBG_INTRUSION,
	VBG_OVERLAP_SEND,
	VBG_CALL_U_TRANSFER,
	VBG_CALL_I_TRANSFER,
	VBG_CALL_E_TRANSFER,
	VBG_3PTY,
	VBG_CONF,
	VBG_HOLD,
	VBG_CONNECT,
	VBG_TLN_WAHLVORBEREITUNG,	/* nicht implementiert */
	VBG_UNKNOWN_0F,
	VBG_UNKNOWN_10,
	VBG_UNKNOWN_11,
	VBG_UNKNOWN_12,
	VBG_UNKNOWN_13,
	VBG_UNKNOWN_14,
	VBG_UNKNOWN_15,
	VBG_UNKNOWN_16,
	VBG_UNKNOWN_17,
	VBG_UNKNOWN_18,
	VBG_UNKNOWN_19,
	VBG_UNKNOWN_1A,
	VBG_UNKNOWN_1B,
	VBG_UNKNOWN_1C,
	VBG_UNKNOWN_1D,
	VBG_UNKNOWN_1E,
	VBG_UNKNOWN_1F,
	VBG_INT_CALL,
	VBG_EXT_CALL,
	VBG_CHARGING_AOCS_INF,		/* unbenutzt */
	VBG_CHARGING_INFO,
	VBG_DISP_HOOK_OFF_MSG,
	VBG_DISPLAY_STRING,
	VBG_DISPLAY_FMT,
	VBG_USER_TO_USER,
	VBG_ALERT,
	VBG_CONNECTED_NUMBER,
	VBG_UNKNOWN_2A,
	VBG_UNKNOWN_2B,
	VBG_UNKNOWN_2C,
	VBG_UNKNOWN_2D,
	VBG_UNKNOWN_2E,
	VBG_UNKNOWN_2F,
	VBG_READY,
	VBG_UNKNOWN_31,
	VBG_UNKNOWN_32,
	VBG_UNKNOWN_33,
	VBG_UNKNOWN_34,
	VBG_UNKNOWN_35,
	VBG_UNKNOWN_36,
	VBG_UNKNOWN_37,
	VBG_UNKNOWN_38,
	VBG_UNKNOWN_39,
	VBG_UNKNOWN_3A,
	VBG_UNKNOWN_3B,
	VBG_UNKNOWN_3C,
	VBG_UNKNOWN_3D,
	VBG_UNKNOWN_3E,
	VBG_UNKNOWN_3F,
	VBG_CALL_ROUTING_LOG,
	VBG_CALL_ROUTING
	//VBG_SUSPEND,				/* unbenutzt, nicht definiert */
};

/*
 * Elementekennungen - CTI Events
 */
enum {
	EVT_RESSOURCEN,				/* unbenutzt */
	EVT_HOOK_STATUS,
	EVT_TLN_STATUS,
	//EVT_TLN_STATUS_MITTEL		/* undef */
	//EVT_TLN_STATUS_KURZ		/* undef */
	EVT_ANL,
	EVT_POR,
	EVT_GEB,
	EVT_ORG,
	EVT_TLN,
	EVT_AUX
};

};
#endif MSGHANDLER_H_
