/* ******************************************************************
 * MsgHandlerNG.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: MsgHandlerNG.cpp,v 1.3 2000/01/09 12:29:56 root Exp root $
 *
 * ******************************************************************/
 
#include <NGApp.h>
#include <MsgHandlerNG.h>

/********************************************************************
 *
 ********************************************************************/
MsgHandlerNG::MsgHandlerNG(const char *host, const int port, const char *app, const char *user, const char *passwd, const BYTE auth, const BYTE perm) :
	MsgHandler(app, user, passwd, auth, perm)
{
	conn = new NGSocket(host, port);

	if ( socket() == -1) 
		sendfailMsg("Could not connect to server",true);

	conn->readEvent.connect(slot(this,&MsgHandlerNG::slotHandle));
	conn->closeEvent.connect(slot(this,&MsgHandlerNG::slotClose));

	conn->enableRead(true);
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerNG::slotClose(NGSocket* s)
{
	conn->enableRead(false);
	sendfailMsg("Server closed connection",true);
	exit(1);
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerNG::slotHandle(NGSocket* s)
{
  Handle();
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerNG::msg_Emitter(ESPMsgHdr *ep)
{
	switch ((ep->gruppe & 0xF0)) {
		case G_ESP :
		  if (ep->isError()) {
			sendfailMsg((char *)&ep->data[2], (int)ep->data[0]);
		  }
		  	espMsg.emit(ep);
		  break;
		case G_ANL :
		  anlMsg.emit(ep);
		  break;
		case G_POR :
		  porMsg.emit(ep);
		  break;
		case G_GEB :
		  gebMsg.emit(ep);
		  break;
		case G_ORG :
		  orgMsg.emit(ep);
		  break;
		case G_TLN :
		  tlnMsg.emit(ep);
		  break;
		case G_AUX :
		  auxMsg.emit(ep);
		  break;
		case G_STA :
		  staMsg.emit(ep);
		  break;
		case G_OFF :
		  offMsg.emit(ep);
		  break;
		case G_VBG :
		  vbgMsg.emit(ep);
		  break;
		case G_EVT :
		  evtMsg.emit(ep);
		  break;
		case G_TST :
		  tstMsg.emit(ep);
		  break;
		case G_FAB :
		  fabMsg.emit(ep);
		  break;
	}
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerNG::sendfailMsg(char *msg, int fatal) 
{
	_fail = msg;
	_fatal = fatal;
	failMsg.emit(this);
}

void MsgHandlerNG::processEvents()
{
	globalApp->processEvents();
}




