/* This file _was_ part of the KDE libraries
	and was modified the way that it does not depend on Qt
	anymore :)
	This aweful work has been done by
	Chris Benndorf to use it with the maKs package...
*/

/* This file is part of the KDE libraries
    Copyright (C) 1997 Torben Weis (weis@kde.org)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef NGSOCKET_H
#define NGSOCKET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef NULL
#undef NULL
#endif
#define NULL 0

#include <sys/types.h>
#include <sys/socket.h> 
#include <sys/un.h>
#include <netinet/in.h>

#include <NGNotifier.h>

/** 
 * A TCP/IP client socket. You can connect this socket to any internet address. 
 *
 * The socket gives you three signals: When ready for reading/writing or 
 * if the connection is broken.  Using socket() you get a file descriptor
 * which you can use with usual unix commands like write(..) or read(...). 
 * If you have already such a socket identifier you can construct a KSocket
 * on this identifier. 
 *  
 * If socket() delivers a value of -1 or less, the connection 
 * had no success.
 *
 * @author Torben Weis <weis@uni-frankfurt.de>
 * @version $Id: NGSocket.h,v 1.1 2000/01/08 10:18:27 root Exp $
 * @short a TCP/IP client socket.
 */
class NGSocket : public Object
{
public:
    /** 
     * Create a Socket with the provided file descriptor.
     * @param _sock	the file descriptor to use.
     */
    NGSocket( int _sock )
	: sock(_sock), domain(0), 
	readNotifier(0), writeNotifier(0) {}
    
    /** 
     * Create a socket and connect to a host.
     * @param _host	the remote host to which to connect.
     * @param _port	the port on the remote host.
     */
    NGSocket( const char *_host, unsigned short int _port );
    NGSocket( const char *_host, unsigned short int _port, int timeOut);
    
    /** 
     * Connects to a UNIX domain socket.
     * @param _path    the filename of the socket
     */
    NGSocket( const char *_path );

    /** 
     * Destructor. Closes the socket if it is still open.
     */
    ~NGSocket();
    
    /** 
     * Returns a file descriptor for this socket.
     */
    int socket() const { return sock; }
    
    /** 
     * Enable the socket for reading.
     *
     * If you enable read mode, the socket will emit the signal
     * readEvent whenever there is something to read out of this
     * socket.
     */
    void enableRead( bool );
    
    /** 
     * Enable the socket for writing.
     *
     * If you enable write mode, the socket will emit the signal
     * writeEvent whenever the socket is ready for writing.
     */
    void enableWrite( bool );
    
    /**
     * Return address.
     */
    unsigned long getAddr();

    /** 
     * Connected to the writeNotifier.
     */
    void slotWrite( int );
    
    /** 
     * Connected to the readNotifier.
     */
    void slotRead( int );
    
//signals:
    /** 
     * Data has arrived for reading.
     *
     * This signal will only be raised if enableRead( TRUE ) was called
     * first.
     */
    Signal1<void,NGSocket*> readEvent;	//void readEvent( NGSocket * );
    
    /** 
     * Socket is ready for writing.
     *
     * This signal will only be raised if enableWrite( TRUE ) was called
     * first.
     */
    Signal1<void,NGSocket*> writeEvent;	//void writeEvent( NGSocket * );
    
    /** 
     * Raised when the connection is broken.
     */
    Signal1<void,NGSocket*> closeEvent;	//void closeEvent( NGSocket * );
    
protected:
    bool connect( const char *_host, unsigned short int _port );
    bool connect( const char *_path );
  
    bool init_sockaddr( const char *hostname, unsigned short int port );
    
    struct sockaddr_in server_name;
    struct sockaddr_un unix_addr;

    /******************************************************
     * The file descriptor for this socket. sock may be -1.
     * This indicates that it is not connected.
     */
    int sock;

    int domain;
  
    NGNotifier *readNotifier;
    NGNotifier *writeNotifier;

private:
    int timeOut;
    
    NGSocket(const NGSocket&);
    NGSocket& operator=(const NGSocket&);
};

#endif  NGSOCKET_H
