/* ******************************************************************
 * PNumber.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: PNumber.cpp,v 1.5 2000/01/20 14:06:27 root Exp root $
 *
 * ******************************************************************/
#include <iomanip>
#include "PNumber.h"

// Copy Constructor
BaseNumber::BaseNumber(const BaseNumber& b)
{
	numdata	= b.NumberData();
	numstr	= b.Phone();
	entry	= b.entry;
}

// Build from vector
BaseNumber::BaseNumber(const vector<BYTE>& v)
{
	numdata = v;
	decode();
	setup();
}

//Build from char*
BaseNumber::BaseNumber(const char* const s)
{
	numstr = s;
	encode();
	setup();
}

//Build from string
BaseNumber::BaseNumber(const string& s)
{
	numstr = s;
	encode();
	setup();
}

const vector<BYTE>& BaseNumber::data() const
{
	return entry;
}

const vector<BYTE>& BaseNumber::NumberData() const
{
	return numdata;
}

const string& BaseNumber::Phone() const
{
	return numstr;
}

void BaseNumber::setup()
{
	entry.clear();
	if (numdata.empty()) {
		entry.push_back(0);
		return;
	}
	entry.push_back((BYTE)numdata.size());
	copy(numdata.begin(), numdata.end(), back_inserter(entry));
}

void BaseNumber::encode()
{
	if (numstr.empty())
		return;

	// Die Nummern zusammenbauen, mit 0x?F oder 0xFF abschliessend
	int i = 0;
	while ( 1 ) {
		BYTE a,b;

		// Maximum size is 24 digits = 24 nibble = 12 Bytes
		if (numdata.size() == 12) return;

		a = numstr.c_str()[i++];
		if ((!a) || (a < 0x30) || (a > 0x39)) {
			// numdata.push_back(0xFF);
			break;
		}

		b = numstr.c_str()[i++];
		if ((!b) || (b < 0x30) || (b > 0x39)) {
			b = 0x0F | ((a - '0') << 4);
			numdata.push_back(b);
			break;
		}

		numdata.push_back(((a - '0') << 4) | (b - '0'));
	}
}

void BaseNumber::decode()
{
	if (numdata.size() == 0)
		return;

	vector<BYTE>::iterator pos = numdata.begin();
	while (pos != numdata.end()) {
		char a = (((*pos) >> 4) & 0x0F);
		char b = (((*pos)     ) & 0x0F);
		
		if (a == 0x0F) break;
		numstr += (char)(a + '0');
			
		if (b == 0x0F) break;
		numstr += (char)(b + '0');

		pos++;
	}
}

ostream& operator<< ( ostream& strm , const BaseNumber& n) {
	strm << "Nummer: "<< n.numstr << endl;
	strm << "Data  : ";
	strm << hex << setfill('0');
	for (const BYTE* bp = n.numdata.begin(); bp < n.numdata.end(); bp++)
		strm << setw(2) << (int)*bp << ' ';
	strm << dec << setfill(' ');
	return strm;
}


ONumber::ONumber(const ONumber& o)
{
	zone	= o.zone;
	entry	= o.entry;
	numstr	= o.numstr;
	numdata	= o.numdata;
}

ONumber::ONumber(const vector<BYTE>& v, BYTE z)
	:	BaseNumber(v)
{
	setup(z);
}

ONumber::ONumber(const char* const s, BYTE z)
	:	BaseNumber(s)
{
	setup(z);
}

ONumber::ONumber(const string& s, BYTE z)
	:	BaseNumber(s)
{
	setup(z);
}

const vector<BYTE>& ONumber::data() const
{
	return entry;
}

const BYTE ONumber::getZone() const
{
	return zone;
}

void ONumber::setup(BYTE z)
{
	zone = z;
	entry.push_back(zone);
	entry.push_back((BYTE)numdata.size());
	copy(numdata.begin(),numdata.end(),back_inserter(entry));
}

ostream& operator<< ( ostream& strm , const ONumber& n) {
	strm << "Zone  : "<< (int)n.zone << endl;
	strm << "Nummer: "<< n.numstr << endl;
	strm << "Data  : ";
	strm << hex << setfill('0');
	for (const BYTE* bp = n.entry.begin(); bp < n.entry.end(); bp++)
		strm << setw(2) << (int)*bp << ' ';
	strm << dec << setfill(' ');
	return strm;
}



PNumber::PNumber(const PNumber& p)
{
	indexH	= p.indexH;
	indexL	= p.indexL;
	klasse	= p.klasse;
	firma	= p.firma;
	entry	= p.entry;
	numstr	= p.numstr;
	numdata	= p.numdata;
}

PNumber::PNumber(const vector<BYTE>& v, WORD n, BYTE k, BYTE f)
	: BaseNumber(v)
{
	setup(n, k, f);
}

PNumber::PNumber(const char* const s, WORD n, BYTE k, BYTE f)
	: BaseNumber(s)
{
	setup(n, k, f);
}

PNumber::PNumber(const string& s, WORD n, BYTE k, BYTE f)
	: BaseNumber(s)
{
	setup(n, k, f);
}

void PNumber::setup(WORD n, BYTE k, BYTE f)
{
	indexH	= (BYTE)(n >> 8);
	indexL	= (BYTE)(n);
	klasse	= k;
	firma	= f;

	entry.push_back(klasse);
	entry.push_back(firma);
	entry.push_back(indexH);
	entry.push_back(indexL);
	entry.push_back((BYTE)numdata.size());
	copy(numdata.begin(),numdata.end(),back_inserter(entry));
}

const vector<BYTE>& PNumber::data() const
{
	return entry;
}

const WORD PNumber::getIndex()
{
	return (WORD)(((WORD)indexH << 8) + (WORD)indexL);
}

const BYTE PNumber::getKlasse()
{
	return klasse;
}

const BYTE PNumber::getFirma()
{
	return firma;
}

ostream& operator<< ( ostream& strm , const PNumber& n) {
	strm << "Nummer: " << n.numstr;
	strm << " Index  : " << (WORD)(((WORD)n.indexH << 8) + (WORD)n.indexL);
	strm << " Klasse: " << (int)n.klasse;
	strm << " Firma : " << (int)n.firma << endl;
	strm << "Data   : ";
	strm << hex << setfill('0');
	for (const BYTE* bp = n.entry.begin(); bp < n.entry.end(); bp++)
		strm << setw(2) << (int)*bp << ' ';
	strm << dec << setfill(' ');
	return strm;
}
