/* ******************************************************************
 * PNumber.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: PNumber.h,v 1.5 2000/01/08 10:18:27 root Exp root $
 *
 * ******************************************************************/

#ifndef PNUMBER_H_
#define PNUMBER_H_

#ifndef BYTE
#define BYTE unsigned char
#endif

#ifndef WORD
#define WORD unsigned short
#endif

#ifndef ostream
#include <iostream>
#endif

#ifndef vector
#include <vector>
#endif

#ifndef string
#include <string>
#endif

/**
 * Basisklasse fuer alle codierten Nummern
 **/
class BaseNumber {
public:
	BaseNumber()					{ };
	BaseNumber(const BaseNumber&);
	BaseNumber(const vector<BYTE>&);
	BaseNumber(const char* const);
	BaseNumber(const string&);

	const vector<BYTE>&	data() const;
	const vector<BYTE>&	NumberData() const;
	const string&		Phone() const;
	void				encode();
	void				decode();

	friend ostream& operator<< ( ostream&, const BaseNumber&);

protected:
	void				setup();
	vector<BYTE>		numdata;
	vector<BYTE>		entry;
	string				numstr;
};


/**
 * Ortsnetznummern fuer ORG_LCR_ONKZ mit bis zu 6 Stellen
 **/
class ONumber : public BaseNumber {
public:
	ONumber(const ONumber&);
	ONumber(const vector<BYTE>&, BYTE = 0);
	ONumber(const char* const, BYTE = 0);
	ONumber(const string&, BYTE = 0);

	const vector<BYTE>&		data() const;
	const BYTE				getZone() const;

	friend ostream& operator<< ( ostream&, const ONumber&);

protected:
	void			setup(BYTE);

	BYTE			zone;
	vector<BYTE>	entry;
};

//Telefonnummern fuer ORG_RNR_FA mit bis zu 24 Stellen
class PNumber : public BaseNumber {
public:
	PNumber(const PNumber&);
	PNumber(const vector<BYTE>&, WORD = 0, BYTE = 0, BYTE = 0 );
	PNumber(const char* const, WORD = 0, BYTE = 0, BYTE = 0);
	PNumber(const string&, WORD = 0, BYTE = 0, BYTE = 0);
	
	const vector<BYTE>&		data() const;
	const WORD				getIndex();
	const BYTE				getKlasse();
	const BYTE				getFirma();

	enum {
		FREI = 0,
		SPERRE,
		NOTRUF,
		KURZWAHL,
		LCRSPERRE,
		PROVIDER
	};

	friend ostream& operator<< ( ostream&, const PNumber&);

protected:
	void			setup(WORD, BYTE, BYTE);

	BYTE					indexH;
	BYTE					indexL;
	BYTE					klasse;
	BYTE					firma;
	vector<BYTE>			entry;
};
#endif PNUMBER_H_
