/* ******************************************************************
 * led.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: led.cpp,v 1.1 1998/04/19 11:49:20 root Exp $
 *
 * ******************************************************************/

/*
    File: led.cpp
    Version: 0.2.1
    Date: 1998/14/01
    Copyright: Bernhard Kuhn <kuhn@lpr.e-technik.tu-muenchen.de>

    This file defines an Led, which can be set to on or off :-)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include <led.h>
#include <led.moc>



// constructor (default: led=off)

led::led(QWidget* parent=0, const char *name=0L)
  : 
	QFrame(parent, name),
	s(Dead),
    	dadColor(LED_STD_DEAD),
	ohkColor(LED_STD_OFFHOOK),
	rngColor(LED_STD_RINGING),
	idlColor(LED_STD_IDLE),
	hldColor(LED_STD_ONHOLD) {
  
	// Apperance
	setFrameStyle(WinPanel|Sunken);

	// select brush for future drawing
	ledBrush=QBrush(dadColor);
 
};


// set led state

void led::switchLed(State newstate=Dead) {

	setLed(newstate);

};

// set led state

void led::setLed(State newstate=Dead) {

	if(newstate != s) {

		// store new value
		s = newstate;
	
		// select brush for future drawing
		switch (s) {
			case OnHold:
    				ledBrush = QBrush(hldColor);
				break;
			case Dead:
    				ledBrush = QBrush(dadColor);
				break;
			case OffHook:
    				ledBrush = QBrush(ohkColor);
				break;
			case Ringing:
    				ledBrush = QBrush(rngColor);
				break;
			case Idle:
    				ledBrush = QBrush(idlColor);
				break;
		}
    
		repaint(false);

	};

};



// draw led content
void led::drawContents(QPainter* p) {

	// no comment
	p->fillRect(contentsRect(),ledBrush); 
};
