/********************************************************************
 * lib/db_util.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: db_util.c,v 1.2 1999/12/30 21:00:56 root Exp root $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <db_util.h>

DB_CONN   def_conn;

void exit_nicely(DB_CONN *conn)
{
	fprintf(stderr,"\nSQL ERROR: %s",DB_ERR(conn));
	DB_CLOSE(conn);
	exit(1);
}

void close_DB(DB_CONN *conn)
{
  DB_CLOSE(conn);
}

int exec_DB(DB_CONN *conn, char *command)
{
	DB_RES	res;

	res = DB_EXEC(conn,command);
	if (DB_CHKRES(res)) {
		fprintf(stderr,"SQL ERROR: %s",DB_ERR(conn));
		return(0);
	}

#ifdef DB_POSTGRES
/*
	{
		char *oid_stat;

		oid_stat=PQoidStatus(res);
		if (!oid_stat[0]) {
			fprintf(stderr,"\ncommand failed: %s - %s\n",command,oid_stat);
			PQclear(res);
			return(0);
		}
	}
*/
	PQclear(res);
#endif

	return(1);
}


DB_CONN *open_DB(char *database)
{
	DB_CONN *conn=NULL;

#ifdef DB_MYSQL
	char *host="localhost";
	char *user=getenv("USER");

	if (!(conn = mysql_connect(&def_conn,host,user,"")))
		exit_nicely(&def_conn);

	if (mysql_select_db(conn, database) == -1)
		exit_nicely(&def_conn);
#endif

#ifdef DB_POSTGRES
	char	*pghost, *pgport, *pgoptions, *pgtty;

	pghost = NULL; pgport = NULL; pgoptions = NULL; pgtty = NULL;

	conn = PQsetdb(pghost, pgport, pgoptions, pgtty, database);
	if (PQstatus(conn) == CONNECTION_BAD) 
		exit_nicely(conn);
#endif

	return(conn);
}
