/********************************************************************
 * lib/server_conn.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: server_conn.c,v 1.3 1999/12/30 21:00:56 root Exp root $
 *
 ********************************************************************/
 
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <unistd.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <maKs.h>
#include <Packet.h>
#include <server_conn.h>


#undef std_Procs
#undef user_Procs

HPRC	std_Procs[16];
HPRC	usr_Procs[16];
HPRC	startProc = NULL;

EspMsg*	_last_esp = NULL;
char *	_user;
char *	_pw;

int		done;
int		_flgnum = 0;

FILE	*__dbgfp__;
int	__debug__ = 0;

int set_user(char *user)
{
	if (user) {
		_user = strdup(user);
		return(1);
	} else 
		return(0);

}

int set_passwd(char *pw)
{
	if (pw) {
		_pw = strdup(pw);
		return(1);
	} else 
		return(0);
}

char *get_user()
{
	return(_user);
}

char *get_passwd()
{
	return(_pw);
}

void set_config(FILE *rcfp, char *token, char **user, char **pw, char **host, int *port) 
{
	char dummy[255];
	int _port;
	int ret;

	if (!strncasecmp(token,"port",4)) {
		if ((ret = fscanf(rcfp,"%d",&_port)) != EOF)
		*port = _port;
	} else {
		if ((ret = fscanf(rcfp,"%s",dummy)) == EOF) return;

		if (!strncasecmp(token,"user",4)) {
			*user = strdup(dummy);
		} else if (!strncasecmp(token,"pass",4)) {
			*pw = strdup(dummy);
		} else if (!strncasecmp(token,"host",4)) {
			*host = strdup(dummy);
		} 
		/* alles andere wird ignoriert */
	}
}

int read_maKsrc(char **user, char **pw, char **host, int *port)
{
	char rcfile[255];
	char configword[255];
	FILE *rcfp;

	sprintf(rcfile,"%s/.maKsrc",getenv("HOME"));

	if (access(rcfile,F_OK | R_OK))
		return(0);

	if ((rcfp = fopen(rcfile,"r")) == NULL)
		return(0);

	while(!feof(rcfp)) {
		fscanf(rcfp,"%s",configword);
		if  ((configword[0]=='#') || (configword[0]=='\n')) {
			fgets(configword,255,rcfp);
			continue;
		}

		if (strlen(configword))
			set_config(rcfp, configword, user, pw, host, port);
	}

	fclose(rcfp);

	return(1);
}

/********************************************************************
 *
 ********************************************************************/
BYTE inc_folgenr()
{
	_flgnum++;

	if ((_flgnum & 0x0080) == 0x0080)
		_flgnum = 0;

	return((BYTE)_flgnum);
}

/********************************************************************
 *
 ********************************************************************/
int Server_Connect(char *machine, int port)
{
	struct sockaddr_in   sockp;
	struct hostent      *hostp;
	struct hostent       defaulthost;
	struct in_addr       defaultaddr;
	char               **p;
	int                  c;
	int                  s;

	hostp = gethostbyname(machine);

	if (!hostp) return(-1);

	memset((char *)&sockp, '\0', sizeof(struct sockaddr_in));
	sockp.sin_family = hostp->h_addrtype;
	sockp.sin_port   = htons(port);

	c = -1;
	s = -1;

	for (p = hostp->h_addr_list; ((p) && (*p)); p++)
	{
		s = socket(hostp->h_addrtype, SOCK_STREAM, 0);

		if (s < 0) return(-1);

		memcpy((char *)&sockp.sin_addr, *p, hostp->h_length);

		c = connect(s, (struct sockaddr *)&sockp, sizeof(struct sockaddr_in));

		if (c == 0) break;
		else perror("connect");

		close(s);
	}

	if (c < 0) return(-1);

	return(s);

}

/********************************************************************
 *
 ********************************************************************/
void send_server(int fd, BYTE a, BYTE g, BYTE e, BYTE l, BYTE *d)
{
	EspMsg  *esp;
	int		len;

	len = (int)l & 0x00FF;

	esp = allocEsp(len);

	esp->folgenr    = inc_folgenr();
	esp->anlage     = a;
	esp->gruppe     = g;
	esp->element    = e;
	esp->length     = l;

	memcpy(esp->data, d, len);
	
	sendPacket(fd, esp);

	free(esp);
}

/********************************************************************
 *
 ********************************************************************/
EspMsg  *send_request(int fd, BYTE a, BYTE g, BYTE e, BYTE l, BYTE *d)
{
	EspMsg  *esp;
	EspMsg  *rsp = NULL;
	int		len;
	int retry = 0;

	len = (int)l & 0x00FF;

	esp = allocEsp(len);

	esp->folgenr    = inc_folgenr();
	esp->anlage     = a;
	esp->gruppe     = g;
	esp->element    = e;
	esp->length     = l;

	memcpy(esp->data, d, len);
	
	sendPacket(fd, esp);

	do {
		rsp = PacketWaitSync(fd);
		if (rsp && (rsp->gruppe & 0xF0) == (esp->gruppe &0xF0) && (esp->element == rsp->element)) {
			break;
		} else if (rsp) {
			PacketParse(fd,rsp);
			free(rsp);
		} 
		retry++;
	} while (retry < 6);
	
	free(esp);

	return(rsp);
}

/********************************************************************
 *
 ********************************************************************/
EspMsg  *conn_request(int fd, BYTE perm, BYTE auth, char *appname, int async)
{
	EspMsg	*esp;
	EspMsg  *rsp;

	esp = allocEsp(strlen(appname) + 3);

	esp->folgenr	= inc_folgenr();
	esp->anlage		= 0;
	esp->gruppe		= K_CTI_ESP;
	esp->element	= ESP_CONN_REQ;
	esp->length		= (BYTE)((strlen(appname) + 3) & 0x00FF);
	esp->data[0]	= perm;
	esp->data[1]	= auth;

	memcpy(&esp->data[2], appname, strlen(appname) + 1);
	
	sendPacket(fd, esp);
	free(esp);

	if (async) return(NULL);

	rsp = PacketWaitSync(fd);
	return(rsp);
}

/********************************************************************
 *
 ********************************************************************/
EspMsg  *conn_auth_cleartext(int fd, char *user, char *passw, int async)
{
	EspMsg	*esp;
	EspMsg  *rsp;
	int		len;

	len = strlen(user) + strlen(passw) + 3;

	esp = allocEsp(len);

	esp->folgenr	= inc_folgenr();
	esp->anlage		= 0;
	esp->gruppe		= K_CTI_ESP;
	esp->element	= ESP_CONN_AUTH;
	esp->length		= (BYTE)(len & 0x00FF);
	esp->data[0]	= AUTH_CLEARTXT;

	sprintf(&esp->data[1],"%s;%s", user, passw);
	
	sendPacket(fd, esp);
	free(esp);

	if (async) return(NULL);

	rsp = PacketWaitSync(fd);
	return(rsp);
}

/********************************************************************
 *
 ********************************************************************/
int std_CTI_ESP(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_ANL(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_POR(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_GEB(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_ORG(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_TLN(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_AUX(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_STA(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_OFF(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_VBG(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_EVT(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_TST(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

int std_CTI_FAB(int fd, EspMsg *esp)
{
	return(NOT_HANDLED);
}

/********************************************************************
 *
 ********************************************************************/
EspMsg *PacketWaitSync(int sock)
{
	EspMsg *esp;
	struct timeval	timeout;
	BYTE		buffer[BUFFSIZE];
	fd_set		readable_fds;
	int		l_in;
	int		fdrange;

	if (!std_Procs[0]) {
		fprintf(stderr,"Functions not initialized!\n");
		exit(1);
	}

	fdrange = sock + 1;

	timeout.tv_sec = 60;
	timeout.tv_usec = 0;

	FD_ZERO(&readable_fds);
	FD_SET(sock,&readable_fds);

	errno = 0;

	select(fdrange,&readable_fds,NULL,NULL,&timeout);
	if (FD_ISSET(sock, &readable_fds)) {
		if ((esp = recv_Packet(sock, __debug__, __dbgfp__))!= NULL) {
			return(esp);
		}
	}
	fprintf(__dbgfp__,"Query Timeout Error! Anlage antwortet nicht!\n");
	exit(1);
	return(NULL);
}

/********************************************************************
 *
 ********************************************************************/
int PacketParse(int fd, EspMsg *esp)
{
	int handled = 0;
	int which;

	which = (int)((esp->gruppe >> 4) & 0x0F);

	if (usr_Procs[which])
		handled = (usr_Procs[which])(fd, esp);

	if ((!handled) && (std_Procs[which]))
		handled = (std_Procs[which])(fd, esp);

	return(handled);
}

/********************************************************************
 *
 ********************************************************************/
int PacketWaitAsync(int sock)
{
	EspMsg *esp;
	struct timeval	timeout;
	BYTE		buffer[BUFFSIZE];
	fd_set		readable_fds;
	int		l_in;
	int		fdrange;

	if (!std_Procs[0]) {
		fprintf(stderr,"Functions not initialized!\n");
		exit(1);
	}

	fdrange = sock + 1;

	timeout.tv_sec = 4;
	timeout.tv_usec = 0;

	FD_ZERO(&readable_fds);
	FD_SET(sock,&readable_fds);

	while(!done && select(fdrange,&readable_fds,NULL,NULL,NULL) >= 0) {
		if (FD_ISSET(sock, &readable_fds)) {
			if ((esp = recv_Packet(sock, __debug__, __dbgfp__))!= NULL) {
				PacketParse(sock,esp);
				free(esp);
			} else {
				switch (errno) {
					 case EPIPE: return(EPIPE);
				}
			}
		}

		timeout.tv_sec = 4;
		timeout.tv_usec = 0;
	
		FD_ZERO(&readable_fds);
		FD_SET(sock,&readable_fds);
	}
}

/********************************************************************
 *
 ********************************************************************/
void SetStartProc(void *proc)
{
	startProc = proc;
}

/********************************************************************
 *
 ********************************************************************/
void SetStdProc(BYTE which, void *proc)
{
	int where = ((int)(which >> 4) & 0x0F);

	std_Procs[where] = proc;
}

/********************************************************************
 *
 ********************************************************************/
void SetUsrProc(BYTE which, void *proc)
{
	int where = ((int)(which >> 4) & 0x0F);

#ifdef DEBUG
	fprintf(stderr,"Installing UsrProc[%d] = 0x%08X\n",where,(int)proc);
#endif
	usr_Procs[where] = proc;
}

/********************************************************************
 *
 ********************************************************************/
void InitProcs(int debug, FILE *logfp)
{
	__dbgfp__ = logfp;
	__debug__ = debug;

	std_Procs[ 0] = &std_CTI_ESP;
	std_Procs[ 1] = &std_CTI_ANL;
	std_Procs[ 2] = &std_CTI_POR;
	std_Procs[ 3] = &std_CTI_GEB;
	std_Procs[ 4] = &std_CTI_ORG;
	std_Procs[ 5] = &std_CTI_TLN;
	std_Procs[ 6] = &std_CTI_AUX;
	std_Procs[ 7] = NULL;
	std_Procs[ 8] = NULL;
	std_Procs[ 9] = NULL;
	std_Procs[10] = &std_CTI_STA;
	std_Procs[11] = &std_CTI_OFF;
	std_Procs[12] = &std_CTI_VBG;
	std_Procs[13] = &std_CTI_EVT;
	std_Procs[14] = &std_CTI_TST;
	std_Procs[15] = &std_CTI_FAB;

	usr_Procs[ 0] = NULL;
	usr_Procs[ 1] = NULL;
	usr_Procs[ 2] = NULL;
	usr_Procs[ 3] = NULL;
	usr_Procs[ 4] = NULL;
	usr_Procs[ 5] = NULL;
	usr_Procs[ 6] = NULL;
	usr_Procs[ 7] = NULL;
	usr_Procs[ 8] = NULL;
	usr_Procs[ 9] = NULL;
	usr_Procs[10] = NULL;
	usr_Procs[11] = NULL;
	usr_Procs[12] = NULL;
	usr_Procs[13] = NULL;
	usr_Procs[14] = NULL;
	usr_Procs[15] = NULL;
}

